@@ -1,5 +1,5 @@
 /*
- * $Id: Document.java,v 1.70 2004/02/10 16:36:02 blowagie Exp $
+ * $Id: Document.java,v 1.113 2006/09/14 10:12:10 blowagie Exp $
  * $Name:  $
  *
  * Copyright 1999, 2000, 2001, 2002 by Bruno Lowagie.
@@ -50,6 +50,7 @@
 
 package com.lowagie.text;
 
+import java.text.SimpleDateFormat;
 import java.util.ArrayList;
 import java.util.Iterator;
 
@@ -59,138 +60,148 @@ import java.util.Date;
  * A generic Document class.
  * <P>
  * All kinds of Text-elements can be added to a <CODE>HTMLDocument</CODE>.
- * The <CODE>Document</CODE> signals all the listeners when an element
- * has been added.
+ * The <CODE>Document</CODE> signals all the listeners when an element has
+ * been added.
  * <P>
  * Remark:
  * <OL>
  *     <LI>Once a document is created you can add some meta information.
  *     <LI>You can also set the headers/footers.
  *     <LI>You have to open the document before you can write content.
- *     <LI>You can only write content (no more meta-formation!) once a document is opened.
- *     <LI>When you change the header/footer on a certain page, this will be effective starting on the next page.
- *     <LI>Ater closing the document, every listener (as well as its <CODE>OutputStream</CODE>) is closed too.
+ * <LI>You can only write content (no more meta-formation!) once a document is
+ * opened.
+ * <LI>When you change the header/footer on a certain page, this will be
+ * effective starting on the next page.
+ * <LI>Ater closing the document, every listener (as well as its <CODE>
+ * OutputStream</CODE>) is closed too.
  * </OL>
- * Example:
- * <BLOCKQUOTE><PRE>
- * // creation of the document with a certain size and certain margins
- * <STRONG>Document document = new Document(PageSize.A4, 50, 50, 50, 50);</STRONG>
- * try {
- *    // creation of the different writers
- *    HtmlWriter.getInstance(<STRONG>document</STRONG>, System.out);
- *    PdfWriter.getInstance(<STRONG>document</STRONG>, new FileOutputStream("text.pdf"));
+ * Example: <BLOCKQUOTE>
  *
+ * <PRE>// creation of the document with a certain size and certain margins
+ * <STRONG>Document document = new Document(PageSize.A4, 50, 50, 50, 50);
+ * </STRONG> try { // creation of the different writers HtmlWriter.getInstance(
+ * <STRONG>document </STRONG>, System.out); PdfWriter.getInstance(
+ * <STRONG>document </STRONG>, new FileOutputStream("text.pdf"));
  *    // we add some meta information to the document
- *    <STRONG>document.addAuthor("Bruno Lowagie");</STRONG>
- *    <STRONG>document.addSubject("This is the result of a Test.");</STRONG>
- *
- *    // we define a header and a footer
- *    HeaderFooter header = new HeaderFooter(new Phrase("This is a header."), false);
- *    HeaderFooter footer = new HeaderFooter(new Phrase("This is page "), new Phrase("."));
+ * <STRONG>document.addAuthor("Bruno Lowagie"); </STRONG>
+ * <STRONG>document.addSubject("This is the result of a Test."); </STRONG>
+ *  // we define a header and a footer HeaderFooter header = new
+ * HeaderFooter(new Phrase("This is a header."), false); HeaderFooter footer =
+ * new HeaderFooter(new Phrase("This is page "), new Phrase("."));
  *    footer.setAlignment(Element.ALIGN_CENTER);
- *    <STRONG>document.setHeader(header);</STRONG>
- *	  <STRONG>document.setFooter(footer);</STRONG>
- *    // we open the document for writing
- *    <STRONG>document.open();</STRONG>
- *    <STRONG>document.add(new Paragraph("Hello world"));</STRONG>
- * }
- * catch(DocumentException de) {
- *    System.err.println(de.getMessage());
- * }
- * <STRONG>document.close();</CODE>
- * </PRE></BLOCKQUOTE>
+ * <STRONG>document.setHeader(header); </STRONG>
+ * <STRONG>document.setFooter(footer); </STRONG>// we open the document for
+ * writing <STRONG>document.open(); </STRONG> <STRONG>document.add(new
+ * Paragraph("Hello world")); </STRONG>} catch(DocumentException de) {
+ * System.err.println(de.getMessage()); } <STRONG>document.close(); </STRONG>
+ * </CODE>
+ * </PRE>
+ * 
+ * </BLOCKQUOTE>
  */
 
 public class Document implements DocListener {
     
     // membervariables
     
-/** This constant may only be changed by Paulo Soares and/or Bruno Lowagie. */
-    private static final String ITEXT_VERSION = "iText by lowagie.com (r1.02b;p128)";
+	/** This constant may only be changed by Paulo Soares and/or Bruno Lowagie. */
+	private static final String ITEXT_VERSION = "iText 1.4.5 (by lowagie.com)";
     
-/** Allows the pdf documents to be produced without compression for debugging purposes. */
+	/**
+	 * Allows the pdf documents to be produced without compression for debugging
+	 * purposes.
+	 */
     public static boolean compress = true; 
     
-/** The DocListener. */
+	/** The DocListener. */
     private ArrayList listeners = new ArrayList();
     
-/** Is the document open or not? */
+	/** Is the document open or not? */
     protected boolean open;
     
-/** Has the document already been closed? */
+	/** Has the document already been closed? */
     protected boolean close;
     
     // membervariables concerning the layout
     
-/** The size of the page. */
+	/** The size of the page. */
     protected Rectangle pageSize;
     
-/** The watermark on the pages. */
+	/** The watermark on the pages. */
     protected Watermark watermark = null;
     
-/** margin in x direction starting from the left */
+	/** margin in x direction starting from the left */
     protected float marginLeft = 0;
     
-/** margin in x direction starting from the right */
+	/** margin in x direction starting from the right */
     protected float marginRight = 0;
     
-/** margin in y direction starting from the top */
+	/** margin in y direction starting from the top */
     protected float marginTop = 0;
     
-/** margin in y direction starting from the bottom */
+	/** margin in y direction starting from the bottom */
     protected float marginBottom = 0;
     
-/** Content of JavaScript onLoad function */
+    protected boolean marginMirroring = false;
+    
+	/** Content of JavaScript onLoad function */
     protected String javaScript_onLoad = null;
 
-/** Content of JavaScript onUnLoad function  */
+	/** Content of JavaScript onUnLoad function */
     protected String javaScript_onUnLoad = null;
 
-/** Style class in HTML body tag */
+	/** Style class in HTML body tag */
     protected String htmlStyleClass = null;
 
     // headers, footers
     
-/** Current pagenumber */
+	/** Current pagenumber */
     protected int pageN = 0;
     
-/** This is the textual part of a Page; it can contain a header */
+	/** This is the textual part of a Page; it can contain a header */
     protected HeaderFooter header = null;
     
-/** This is the textual part of the footer */
+	/** This is the textual part of the footer */
     protected HeaderFooter footer = null;
     
     // constructor
     
-/**
- * Constructs a new <CODE>Document</CODE>-object.
+	/**
+	 * Constructs a new <CODE>Document</CODE> -object.
  */
     
     public Document() {
         this(PageSize.A4);
     }
     
-/**
- * Constructs a new <CODE>Document</CODE>-object.
+	/**
+	 * Constructs a new <CODE>Document</CODE> -object.
  *
- * @param	pageSize	the pageSize
+	 * @param pageSize
+	 *            the pageSize
  */
     
     public Document(Rectangle pageSize) {
         this(pageSize, 36, 36, 36, 36);
     }
     
-/**
- * Constructs a new <CODE>Document</CODE>-object.
+	/**
+	 * Constructs a new <CODE>Document</CODE> -object.
  *
- * @param	pageSize		the pageSize
- * @param	marginLeft		the margin on the left
- * @param	marginRight		the margin on the right
- * @param	marginTop		the margin on the top
- * @param	marginBottom	the margin on the bottom
+	 * @param pageSize
+	 *            the pageSize
+	 * @param marginLeft
+	 *            the margin on the left
+	 * @param marginRight
+	 *            the margin on the right
+	 * @param marginTop
+	 *            the margin on the top
+	 * @param marginBottom
+	 *            the margin on the bottom
  */
     
-    public Document(Rectangle pageSize, float marginLeft, float marginRight, float marginTop, float marginBottom) {
+	public Document(Rectangle pageSize, float marginLeft, float marginRight,
+			float marginTop, float marginBottom) {
         this.pageSize = pageSize;
         this.marginLeft = marginLeft;
         this.marginRight = marginRight;
@@ -200,20 +211,22 @@ public class Document implements DocListener {
     
     // listener methods
     
-/**
+	/**
  * Adds a <CODE>DocListener</CODE> to the <CODE>Document</CODE>.
  *
- * @param	listener	the new DocListener.
+	 * @param listener
+	 *            the new DocListener.
  */
     
     public void addDocListener(DocListener listener) {
         listeners.add(listener);
     }
     
-/**
+	/**
  * Removes a <CODE>DocListener</CODE> from the <CODE>Document</CODE>.
  *
- * @param	listener	the DocListener that has to be removed.
+	 * @param listener
+	 *            the DocListener that has to be removed.
  */
     
     public void removeDocListener(DocListener listener) {
@@ -222,155 +235,162 @@ public class Document implements DocListener {
     
     // methods implementing the DocListener interface
     
-/** Adds an <CODE>Element</CODE> to the <CODE>Document</CODE>.
+	/**
+	 * Adds an <CODE>Element</CODE> to the <CODE>Document</CODE>.
  *
- * @param element the <CODE>Element</CODE> to add
- * @return <CODE>true</CODE> if the element was added, <CODE>false</CODE> if not
- * @throws DocumentException when a document isn't open yet, or has been closed
+	 * @param element
+	 *            the <CODE>Element</CODE> to add
+	 * @return <CODE>true</CODE> if the element was added, <CODE>false
+	 *         </CODE> if not
+	 * @throws DocumentException
+	 *             when a document isn't open yet, or has been closed
  */
     
     public boolean add(Element element) throws DocumentException {
         if (close) {
-            throw new DocumentException("The document has been closed. You can't add any Elements.");
+			throw new DocumentException(
+					"The document has been closed. You can't add any Elements.");
         }
         int type = element.type();
         if (open) {
-            if (! (type == Element.CHUNK ||
-            type == Element.PHRASE ||
-            type == Element.PARAGRAPH ||
-            type == Element.TABLE ||
-            type == Element.PTABLE ||
-            type == Element.ANCHOR ||     
-            type == Element.ANNOTATION ||
-            type == Element.CHAPTER ||
-            type == Element.SECTION ||
-            type == Element.LIST ||
-            type == Element.LISTITEM ||
-            type == Element.RECTANGLE ||
-            type == Element.JPEG ||    
-            type == Element.IMGRAW ||
-            type == Element.IMGTEMPLATE ||
-            type == Element.GRAPHIC)) {
-                throw new DocumentException("The document is open; you can only add Elements with content.");
-            }
-        }
-        else {
-            if (! (type == Element.HEADER ||
-            type == Element.TITLE ||
-            type == Element.SUBJECT ||
-            type == Element.KEYWORDS ||
-            type == Element.AUTHOR ||
-            type == Element.PRODUCER ||
-            type == Element.CREATOR ||
-            type == Element.CREATIONDATE)) {
-                throw new DocumentException("The document is not open yet; you can only add Meta information.");
+			if (!(type == Element.CHUNK || type == Element.PHRASE
+					|| type == Element.PARAGRAPH || type == Element.TABLE
+					|| type == Element.PTABLE
+					|| type == Element.MULTI_COLUMN_TEXT
+					|| type == Element.ANCHOR || type == Element.ANNOTATION
+					|| type == Element.CHAPTER || type == Element.SECTION
+					|| type == Element.LIST || type == Element.LISTITEM
+					|| type == Element.RECTANGLE || type == Element.JPEG
+					|| type == Element.IMGRAW || type == Element.IMGTEMPLATE || type == Element.GRAPHIC)) {
+				throw new DocumentException(
+						"The document is open; you can only add Elements with content.");
+			}
+		} else {
+			if (!(type == Element.HEADER || type == Element.TITLE
+					|| type == Element.SUBJECT || type == Element.KEYWORDS
+					|| type == Element.AUTHOR || type == Element.PRODUCER
+					|| type == Element.CREATOR || type == Element.CREATIONDATE)) {
+				throw new DocumentException(
+						"The document is not open yet; you can only add Meta information.");
             }
         }
         boolean success = false;
         DocListener listener;
-        for (Iterator iterator = listeners.iterator(); iterator.hasNext(); ) {
+		for (Iterator iterator = listeners.iterator(); iterator.hasNext();) {
             listener = (DocListener) iterator.next();
             success |= listener.add(element);
         }
         return success;
     }
     
-/**
+	/**
  * Opens the document.
  * <P>
- * Once the document is opened, you can't write any Header- or Meta-information
- * anymore. You have to open the document before you can begin to add content
- * to the body of the document.
+	 * Once the document is opened, you can't write any Header- or
+	 * Meta-information anymore. You have to open the document before you can
+	 * begin to add content to the body of the document.
  */
     
     public void open() {
-        if (! close) {
+		if (!close) {
             open = true;
         }
         DocListener listener;
-        for (Iterator iterator = listeners.iterator(); iterator.hasNext(); ) {
+		for (Iterator iterator = listeners.iterator(); iterator.hasNext();) {
             listener = (DocListener) iterator.next();
             listener.setPageSize(pageSize);
-            listener.setMargins(marginLeft, marginRight, marginTop, marginBottom);
+			listener.setMargins(marginLeft, marginRight, marginTop,
+					marginBottom);
             listener.open();
         }
     }
     
-/**
+	/**
  * Sets the pagesize.
  *
- * @param	pageSize	the new pagesize
+	 * @param pageSize
+	 *            the new pagesize
  * @return	a <CODE>boolean</CODE>
  */
     
     public boolean setPageSize(Rectangle pageSize) {
         this.pageSize = pageSize;
         DocListener listener;
-        for (Iterator iterator = listeners.iterator(); iterator.hasNext(); ) {
+		for (Iterator iterator = listeners.iterator(); iterator.hasNext();) {
             listener = (DocListener) iterator.next();
             listener.setPageSize(pageSize);
         }
         return true;
     }
     
-/**
+	/**
  * Sets the <CODE>Watermark</CODE>.
  *
- * @param watermark the watermark to add
- * @return <CODE>true</CODE> if the element was added, <CODE>false</CODE> if not.
+	 * @param watermark
+	 *            the watermark to add
+	 * @return <CODE>true</CODE> if the element was added, <CODE>false
+	 *         </CODE> if not.
  */
     
     public boolean add(Watermark watermark) {
         this.watermark = watermark;
         DocListener listener;
-        for (Iterator iterator = listeners.iterator(); iterator.hasNext(); ) {
+		for (Iterator iterator = listeners.iterator(); iterator.hasNext();) {
             listener = (DocListener) iterator.next();
             listener.add(watermark);
         }
         return true;
     }
     
-/**
+	/**
  * Removes the <CODE>Watermark</CODE>.
  */
     
     public void removeWatermark() {
         this.watermark = null;
         DocListener listener;
-        for (Iterator iterator = listeners.iterator(); iterator.hasNext(); ) {
+		for (Iterator iterator = listeners.iterator(); iterator.hasNext();) {
             listener = (DocListener) iterator.next();
             listener.removeWatermark();
         }
     }
     
-/**
+	/**
  * Sets the margins.
  *
- * @param	marginLeft		the margin on the left
- * @param	marginRight		the margin on the right
- * @param	marginTop		the margin on the top
- * @param	marginBottom	the margin on the bottom
+	 * @param marginLeft
+	 *            the margin on the left
+	 * @param marginRight
+	 *            the margin on the right
+	 * @param marginTop
+	 *            the margin on the top
+	 * @param marginBottom
+	 *            the margin on the bottom
  * @return	a <CODE>boolean</CODE>
  */
     
-    public boolean setMargins(float marginLeft,float marginRight,float marginTop,float marginBottom) {
+	public boolean setMargins(float marginLeft, float marginRight,
+			float marginTop, float marginBottom) {
         this.marginLeft = marginLeft;
         this.marginRight = marginRight;
         this.marginTop = marginTop;
         this.marginBottom = marginBottom;
         DocListener listener;
-        for (Iterator iterator = listeners.iterator(); iterator.hasNext(); ) {
+		for (Iterator iterator = listeners.iterator(); iterator.hasNext();) {
             listener = (DocListener) iterator.next();
-            listener.setMargins(marginLeft, marginRight, marginTop, marginBottom);
+			listener.setMargins(marginLeft, marginRight, marginTop,
+					marginBottom);
         }
         return true;
     }
     
-/**
+	/**
  * Signals that an new page has to be started.
  *
- * @return	<CODE>true</CODE> if the page was added, <CODE>false</CODE> if not.
- * @throws	DocumentException	when a document isn't open yet, or has been closed
+	 * @return <CODE>true</CODE> if the page was added, <CODE>false</CODE>
+	 *         if not.
+	 * @throws DocumentException
+	 *             when a document isn't open yet, or has been closed
  */
     
     public boolean newPage() throws DocumentException {
@@ -378,98 +398,101 @@ public class Document implements DocListener {
             return false;
         }
         DocListener listener;
-        for (Iterator iterator = listeners.iterator(); iterator.hasNext(); ) {
+		for (Iterator iterator = listeners.iterator(); iterator.hasNext();) {
             listener = (DocListener) iterator.next();
             listener.newPage();
         }
         return true;
     }
     
-/**
+	/**
  * Changes the header of this document.
  *
- * @param	header		the new header
+	 * @param header
+	 *            the new header
  */
     
     public void setHeader(HeaderFooter header) {
         this.header = header;
         DocListener listener;
-        for (Iterator iterator = listeners.iterator(); iterator.hasNext(); ) {
+		for (Iterator iterator = listeners.iterator(); iterator.hasNext();) {
             listener = (DocListener) iterator.next();
             listener.setHeader(header);
         }
     }
     
-/**
+	/**
  * Resets the header of this document.
  */
     
     public void resetHeader() {
         this.header = null;
         DocListener listener;
-        for (Iterator iterator = listeners.iterator(); iterator.hasNext(); ) {
+		for (Iterator iterator = listeners.iterator(); iterator.hasNext();) {
             listener = (DocListener) iterator.next();
             listener.resetHeader();
         }
     }
     
-/**
+	/**
  * Changes the footer of this document.
  *
- * @param	footer		the new footer
+	 * @param footer
+	 *            the new footer
  */
     
     public void setFooter(HeaderFooter footer) {
         this.footer = footer;
         DocListener listener;
-        for (Iterator iterator = listeners.iterator(); iterator.hasNext(); ) {
+		for (Iterator iterator = listeners.iterator(); iterator.hasNext();) {
             listener = (DocListener) iterator.next();
             listener.setFooter(footer);
         }
     }
     
-/**
+	/**
  * Resets the footer of this document.
  */
     
     public void resetFooter() {
         this.footer = null;
         DocListener listener;
-        for (Iterator iterator = listeners.iterator(); iterator.hasNext(); ) {
+		for (Iterator iterator = listeners.iterator(); iterator.hasNext();) {
             listener = (DocListener) iterator.next();
             listener.resetFooter();
         }
     }
     
-/**
+	/**
  * Sets the page number to 0.
  */
     
     public void resetPageCount() {
         pageN = 0;
         DocListener listener;
-        for (Iterator iterator = listeners.iterator(); iterator.hasNext(); ) {
+		for (Iterator iterator = listeners.iterator(); iterator.hasNext();) {
             listener = (DocListener) iterator.next();
             listener.resetPageCount();
         }
     }
     
-/**
+	/**
  * Sets the page number.
  *
- * @param	pageN		the new page number
+	 * @param pageN
+	 *            the new page number
  */
     
     public void setPageCount(int pageN) {
         this.pageN = pageN;
         DocListener listener;
-        for (Iterator iterator = listeners.iterator(); iterator.hasNext(); ) {
+		for (Iterator iterator = listeners.iterator(); iterator.hasNext();) {
             listener = (DocListener) iterator.next();
             listener.setPageCount(pageN);
         }
     }
     
-/**
+	/**
  * Returns the current page number.
  *
  * @return the current page number
@@ -479,20 +502,20 @@ public class Document implements DocListener {
         return this.pageN;
     }
     
-/**
+	/**
  * Closes the document.
  * <P>
- * Once all the content has been written in the body, you have to close
- * the body. After that nothing can be written to the body anymore.
+	 * Once all the content has been written in the body, you have to close the
+	 * body. After that nothing can be written to the body anymore.
  */
     
     public void close() {
-        if (! close) {
+		if (!close) {
             open = false;
             close = true;
         }
         DocListener listener;
-        for (Iterator iterator = listeners.iterator(); iterator.hasNext(); ) {
+		for (Iterator iterator = listeners.iterator(); iterator.hasNext();) {
             listener = (DocListener) iterator.next();
             listener.close();
         }
@@ -500,104 +523,105 @@ public class Document implements DocListener {
     
     // methods concerning the header or some meta information
     
-/**
+	/**
  * Adds a user defined header to the document.
  *
- * @param	name	the name of the header
- * @param	content	the content of the header
+	 * @param name
+	 *            the name of the header
+	 * @param content
+	 *            the content of the header
  * @return	<CODE>true</CODE> if successful, <CODE>false</CODE> otherwise
  */
     
     public boolean addHeader(String name, String content) {
         try {
             return add(new Header(name, content));
-        }
-        catch(DocumentException de) {
+		} catch (DocumentException de) {
             throw new ExceptionConverter(de);
         }
     }
     
-/**
+	/**
  * Adds the title to a Document.
  *
- * @param	title	the title
+	 * @param title
+	 *            the title
  * @return	<CODE>true</CODE> if successful, <CODE>false</CODE> otherwise
  */
     
     public boolean addTitle(String title) {
         try {
             return add(new Meta(Element.TITLE, title));
-        }
-        catch(DocumentException de) {
+		} catch (DocumentException de) {
             throw new ExceptionConverter(de);
         }
     }
     
-/**
+	/**
  * Adds the subject to a Document.
  *
- * @param	subject		the subject
+	 * @param subject
+	 *            the subject
  * @return	<CODE>true</CODE> if successful, <CODE>false</CODE> otherwise
  */
     
     public boolean addSubject(String subject) {
         try {
             return add(new Meta(Element.SUBJECT, subject));
-        }
-        catch(DocumentException de) {
+		} catch (DocumentException de) {
             throw new ExceptionConverter(de);
         }
     }
     
-/**
+	/**
  * Adds the keywords to a Document.
  *
- * @param keywords adds the keywords to the document
+	 * @param keywords
+	 *            adds the keywords to the document
  * @return <CODE>true</CODE> if successful, <CODE>false</CODE> otherwise
  */
     
     public boolean addKeywords(String keywords) {
         try {
             return add(new Meta(Element.KEYWORDS, keywords));
-        }
-        catch(DocumentException de) {
+		} catch (DocumentException de) {
             throw new ExceptionConverter(de);
         }
     }
     
-/**
+	/**
  * Adds the author to a Document.
  *
- * @param	author		the name of the author
+	 * @param author
+	 *            the name of the author
  * @return	<CODE>true</CODE> if successful, <CODE>false</CODE> otherwise
  */
     
     public boolean addAuthor(String author) {
         try {
             return add(new Meta(Element.AUTHOR, author));
-        }
-        catch(DocumentException de) {
+		} catch (DocumentException de) {
             throw new ExceptionConverter(de);
         }
     }
     
-/**
+	/**
  * Adds the creator to a Document.
  *
- * @param	creator		the name of the creator
+	 * @param creator
+	 *            the name of the creator
  * @return	<CODE>true</CODE> if successful, <CODE>false</CODE> otherwise
  */
     
     public boolean addCreator(String creator) {
         try {
             return add(new Meta(Element.CREATOR, creator));
-        }
-        catch(DocumentException de) {
+		} catch (DocumentException de) {
             throw new ExceptionConverter(de);
         }
     }
     
-/**
+	/**
  * Adds the producer to a Document.
  *
  * @return	<CODE>true</CODE> if successful, <CODE>false</CODE> otherwise
@@ -606,13 +630,12 @@ public class Document implements DocListener {
     public boolean addProducer() {
         try {
             return add(new Meta(Element.PRODUCER, "iText by lowagie.com"));
-        }
-        catch(DocumentException de) {
+		} catch (DocumentException de) {
             throw new ExceptionConverter(de);
         }
     }
     
-/**
+	/**
  * Adds the current date and time to a Document.
  *
  * @return	<CODE>true</CODE> if successful, <CODE>false</CODE> otherwise
@@ -620,16 +643,18 @@ public class Document implements DocListener {
     
     public boolean addCreationDate() {
         try {
-            return add(new Meta(Element.CREATIONDATE, new Date().toString()));
-        }
-        catch(DocumentException de) {
+			/* bugfix by 'taqua' (Thomas) */
+			final SimpleDateFormat sdf = new SimpleDateFormat(
+					"EEE MMM dd HH:mm:ss zzz yyyy");
+			return add(new Meta(Element.CREATIONDATE, sdf.format(new Date())));
+		} catch (DocumentException de) {
             throw new ExceptionConverter(de);
         }
     }
     
     // methods to get the layout of the document.
     
-/**
+	/**
  * Returns the left margin.
  *
  * @return	the left margin
@@ -639,7 +664,7 @@ public class Document implements DocListener {
         return marginLeft;
     }
     
-/**
+	/**
  * Return the right margin.
  *
  * @return	the right margin
@@ -649,7 +674,7 @@ public class Document implements DocListener {
         return marginRight;
     }
     
-/**
+	/**
  * Returns the top margin.
  *
  * @return	the top margin
@@ -659,7 +684,7 @@ public class Document implements DocListener {
         return marginTop;
     }
     
-/**
+	/**
  * Returns the bottom margin.
  *
  * @return	the bottom margin
@@ -669,7 +694,7 @@ public class Document implements DocListener {
         return marginBottom;
     }
     
-/**
+	/**
  * Returns the lower left x-coordinate.
  *
  * @return	the lower left x-coordinate
@@ -679,7 +704,7 @@ public class Document implements DocListener {
         return pageSize.left(marginLeft);
     }
     
-/**
+	/**
  * Returns the upper right x-coordinate.
  *
  * @return	the upper right x-coordinate
@@ -689,7 +714,7 @@ public class Document implements DocListener {
         return pageSize.right(marginRight);
     }
     
-/**
+	/**
  * Returns the upper right y-coordinate.
  *
  * @return	the upper right y-coordinate
@@ -699,7 +724,7 @@ public class Document implements DocListener {
         return pageSize.top(marginTop);
     }
     
-/**
+	/**
  * Returns the lower left y-coordinate.
  *
  * @return	the lower left y-coordinate
@@ -709,10 +734,11 @@ public class Document implements DocListener {
         return pageSize.bottom(marginBottom);
     }
     
-/**
+	/**
  * Returns the lower left x-coordinate considering a given margin.
  *
- * @param	margin			a margin
+	 * @param margin
+	 *            a margin
  * @return	the lower left x-coordinate
  */
     
@@ -720,10 +746,11 @@ public class Document implements DocListener {
         return pageSize.left(marginLeft + margin);
     }
     
-/**
+	/**
  * Returns the upper right x-coordinate, considering a given margin.
  *
- * @param	margin			a margin
+	 * @param margin
+	 *            a margin
  * @return	the upper right x-coordinate
  */
     
@@ -731,10 +758,11 @@ public class Document implements DocListener {
         return pageSize.right(marginRight + margin);
     }
     
-/**
+	/**
  * Returns the upper right y-coordinate, considering a given margin.
  *
- * @param	margin			a margin
+	 * @param margin
+	 *            a margin
  * @return	the upper right y-coordinate
  */
     
@@ -742,10 +770,11 @@ public class Document implements DocListener {
         return pageSize.top(marginTop + margin);
     }
     
-/**
+	/**
  * Returns the lower left y-coordinate, considering a given margin.
  *
- * @param	margin			a margin
+	 * @param margin
+	 *            a margin
  * @return	the lower left y-coordinate
  */
     
@@ -753,42 +782,48 @@ public class Document implements DocListener {
         return pageSize.bottom(marginBottom + margin);
     }
     
-/**
+	/**
  * Gets the pagesize.
+	 * 
  * @return the page size
  */
     
-    public Rectangle getPageSize()
-    {
+	public Rectangle getPageSize() {
         return this.pageSize;
     }
     
-    /** Checks if the document is open.
+	/**
+	 * Checks if the document is open.
+	 * 
      * @return <CODE>true</CODE> if the document is open
      */    
     public boolean isOpen() {
         return open;
     }
     
-    /** Gets the iText version.
+	/**
+	 * Gets the iText version.
+	 * This method may only be changed by Paulo Soares and/or Bruno Lowagie.
      * @return iText version
      */    
-    public static String getVersion() {
+    public static final String getVersion() {
         return ITEXT_VERSION;
     }
 
-/**
+	/**
  * Adds a JavaScript onLoad function to the HTML body tag
  *
- * @param	code		the JavaScript code to be executed on load of the HTML page
+	 * @param code
+	 *            the JavaScript code to be executed on load of the HTML page
  */
     
     public void setJavaScript_onLoad(String code) {
         this.javaScript_onLoad = code;
     }
 
-/**
+	/**
  * Gets the JavaScript onLoad command.
+	 * 
  * @return the JavaScript onLoad command
  */
 
@@ -796,18 +831,20 @@ public class Document implements DocListener {
         return this.javaScript_onLoad;
     }
 
-/**
+	/**
  * Adds a JavaScript onUnLoad function to the HTML body tag
  *
- * @param	code		the JavaScript code to be executed on unload of the HTML page
+	 * @param code
+	 *            the JavaScript code to be executed on unload of the HTML page
  */
     
     public void setJavaScript_onUnLoad(String code) {
         this.javaScript_onUnLoad = code;
     }
 
-/**
+	/**
  * Gets the JavaScript onUnLoad command.
+	 * 
  * @return the JavaScript onUnLoad command
  */
 
@@ -815,17 +852,18 @@ public class Document implements DocListener {
         return this.javaScript_onUnLoad;
     }
 
-/**
+	/**
  * Adds a style class to the HTML body tag
  *
- * @param	htmlStyleClass		the style class for the HTML body tag
+	 * @param htmlStyleClass
+	 *            the style class for the HTML body tag
  */
     
     public void setHtmlStyleClass(String htmlStyleClass) {
         this.htmlStyleClass = htmlStyleClass;
     }
 
-/**
+	/**
  * Gets the style class of the HTML body tag
  *
  * @return		the style class of the HTML body tag
@@ -841,10 +879,38 @@ public class Document implements DocListener {
 	public void clearTextWrap() throws DocumentException {
 		if (open && !close) {
 			DocListener listener;
-			for (Iterator iterator = listeners.iterator(); iterator.hasNext(); ) {
+			for (Iterator iterator = listeners.iterator(); iterator.hasNext();) {
 				listener = (DocListener) iterator.next();
 				listener.clearTextWrap();
 			}
 		}
 	}
+    
+    /**
+     * Set the margin mirroring. It will mirror margins for odd/even pages.
+     * <p>
+     * Note: it will not work with {@link Table}.
+	 * 
+	 * @param marginMirroring
+	 *            <CODE>true</CODE> to mirror the margins
+     * @return always <CODE>true</CODE>
+     */    
+    public boolean setMarginMirroring(boolean marginMirroring) {
+        this.marginMirroring = marginMirroring;
+        DocListener listener;
+		for (Iterator iterator = listeners.iterator(); iterator.hasNext();) {
+            listener = (DocListener) iterator.next();
+            listener.setMarginMirroring(marginMirroring);
+        }
+        return true;
+    }
+    
+    /**
+     * Gets the margin mirroring flag.
+	 * 
+     * @return the margin mirroring flag
+     */    
+    public boolean isMarginMirroring() {
+        return marginMirroring;
+    }
 }
