@@ -1,5 +1,5 @@
 /*
- * $Id: HtmlWriter.java,v 1.91 2004/02/07 10:28:33 blowagie Exp $
+ * $Id: HtmlWriter.java,v 1.111 2006/07/21 15:15:54 psoares33 Exp $
  * $Name:  $
  *
  * Copyright 1999, 2000, 2001, 2002 by Bruno Lowagie.
@@ -61,6 +61,7 @@ import java.util.Stack;
 import java.util.EmptyStackException;
 
 import com.lowagie.text.*;
+import com.lowagie.text.pdf.BaseFont;
 import com.lowagie.text.markup.MarkupTags;
 
 /**
@@ -178,11 +179,11 @@ public class HtmlWriter extends DocWriter implements DocListener {
     
     public boolean newPage() throws DocumentException {
         try {
-            writeStart(MarkupTags.DIV);
+            writeStart(HtmlTags.DIV);
             write(" ");
-            write(MarkupTags.STYLE);
+            write(HtmlTags.STYLE);
             write("=\"");
-            writeCssProperty(MarkupTags.PAGE_BREAK_BEFORE, MarkupTags.ALWAYS);
+            writeCssProperty(MarkupTags.CSS_KEY_PAGE_BREAK_BEFORE, MarkupTags.CSS_VALUE_ALWAYS);
             write("\" /");
             os.write(GT);
         }
@@ -194,7 +195,8 @@ public class HtmlWriter extends DocWriter implements DocListener {
     
 /**
  * Signals that an <CODE>Element</CODE> was added to the <CODE>Document</CODE>.
- *
+ * 
+ * @param element a high level object that has to be translated to HTML
  * @return  <CODE>true</CODE> if the element was added, <CODE>false</CODE> if not.
  * @throws  DocumentException when a document isn't open yet, or has been closed
  */
@@ -209,7 +211,7 @@ public class HtmlWriter extends DocWriter implements DocListener {
                 case Element.HEADER:
                     try {
                         Header h = (Header) element;
-                        if (MarkupTags.STYLESHEET.equals(h.name())) {
+                        if (HtmlTags.STYLESHEET.equals(h.name())) {
                             writeLink(h);
                         }
                         else if (HtmlTags.JAVASCRIPT.equals(h.name())) {
@@ -266,7 +268,7 @@ public class HtmlWriter extends DocWriter implements DocListener {
     public void open() {
         super.open();
         try {
-            writeComment("Producer: iTextXML by lowagie.com");
+            writeComment(Document.getVersion());
             writeComment("CreationDate: " + new Date().toString());
             addTabs(1);
             writeEnd(HtmlTags.HEAD);
@@ -294,7 +296,7 @@ public class HtmlWriter extends DocWriter implements DocListener {
                 write(HtmlTags.JAVASCRIPT_ONUNLOAD, HtmlEncoder.encode(document.getJavaScript_onUnLoad()));
             }
             if (document.getHtmlStyleClass() != null) {
-                write(MarkupTags.CLASS, document.getHtmlStyleClass());
+                write(MarkupTags.HTML_ATTR_CSS_CLASS, document.getHtmlStyleClass());
             }
             os.write(GT);
             initHeader(); // line added by David Freels
@@ -395,9 +397,9 @@ public class HtmlWriter extends DocWriter implements DocListener {
     
     protected void writeLink(Header header) throws IOException {
         addTabs(2);
-        writeStart(MarkupTags.LINK);
-        write(MarkupTags.REL, header.name());
-        write(MarkupTags.TYPE, MarkupTags.CSS);
+        writeStart(HtmlTags.LINK);
+        write(HtmlTags.REL, header.name());
+        write(HtmlTags.TYPE, HtmlTags.TEXT_CSS);
         write(HtmlTags.REFERENCE, header.content());
         writeEnd();
     }
@@ -431,7 +433,7 @@ public class HtmlWriter extends DocWriter implements DocListener {
            * //-->
            * </script>
            */ 
-          write(MarkupTags.TYPE, MarkupTags.JAVASCRIPT);
+          write(HtmlTags.TYPE, MarkupTags.HTML_VALUE_JAVASCRIPT);
           os.write(GT);
           addTabs(2);
           write(new String(BEGINCOMMENT) + "\n");
@@ -536,7 +538,8 @@ public class HtmlWriter extends DocWriter implements DocListener {
     
 /**
  * Signals that a <CODE>String</CODE> was added to the <CODE>Document</CODE>.
- *
+ * 
+ * @param string a String to add to the HTML
  * @return  <CODE>true</CODE> if the string was added, <CODE>false</CODE> if not.
  * @throws  DocumentException when a document isn't open yet, or has been closed
  */
@@ -560,6 +563,7 @@ public class HtmlWriter extends DocWriter implements DocListener {
  *
  * @param   element     the element
  * @param   indent      the indentation
+ * @throws IOException
  */
     
     protected void write(Element element, int indent) throws IOException {
@@ -590,12 +594,12 @@ public class HtmlWriter extends DocWriter implements DocListener {
                 if (tag) {
                     // start span tag
                     addTabs(indent);
-                    writeStart(MarkupTags.SPAN);
+                    writeStart(HtmlTags.SPAN);
                     if (isOtherFont(chunk.font())) {
                         write(chunk.font(), styleAttributes);
                     }
                     if (hasMarkupAttributes(chunk)) {
-                        writeMarkupAttributes((MarkupAttributes)chunk);
+                        writeMarkupAttributes(chunk);
                     }
                     os.write(GT);
                 }
@@ -625,7 +629,7 @@ public class HtmlWriter extends DocWriter implements DocListener {
                 }
                 if (tag) {
                     // end tag
-                    writeEnd(MarkupTags.SPAN);
+                    writeEnd(MarkupTags.HTML_TAG_SPAN);
                 }
                 return;
             }
@@ -633,13 +637,13 @@ public class HtmlWriter extends DocWriter implements DocListener {
             {
                 Phrase phrase = (Phrase) element;
                 styleAttributes = new Properties();
-                if (phrase.leadingDefined()) styleAttributes.setProperty(MarkupTags.CSS_LINEHEIGHT, String.valueOf(phrase.leading()) + "pt");
+                if (phrase.leadingDefined()) styleAttributes.setProperty(MarkupTags.CSS_KEY_LINEHEIGHT, String.valueOf(phrase.leading()) + "pt");
                 
                 // start tag
                 addTabs(indent);
-                writeStart(MarkupTags.SPAN);
+                writeStart(MarkupTags.HTML_TAG_SPAN);
                 if (hasMarkupAttributes(phrase)) {
-                    writeMarkupAttributes((MarkupAttributes)phrase);
+                    writeMarkupAttributes(phrase);
                 }
                 write(phrase.font(), styleAttributes);
                 os.write(GT);
@@ -650,7 +654,7 @@ public class HtmlWriter extends DocWriter implements DocListener {
                 }
                 // end tag
                 addTabs(indent);
-                writeEnd(MarkupTags.SPAN);
+                writeEnd(MarkupTags.HTML_TAG_SPAN);
                 currentfont.pop();
                 return;
             }
@@ -658,7 +662,7 @@ public class HtmlWriter extends DocWriter implements DocListener {
             {
                 Anchor anchor = (Anchor) element;
                 styleAttributes = new Properties();
-                if (anchor.leadingDefined()) styleAttributes.setProperty(MarkupTags.CSS_LINEHEIGHT, String.valueOf(anchor.leading()) + "pt");
+                if (anchor.leadingDefined()) styleAttributes.setProperty(MarkupTags.CSS_KEY_LINEHEIGHT, String.valueOf(anchor.leading()) + "pt");
                 
                 // start tag
                 addTabs(indent);
@@ -670,7 +674,7 @@ public class HtmlWriter extends DocWriter implements DocListener {
                     write(HtmlTags.REFERENCE, anchor.reference());
                 }
                 if (hasMarkupAttributes(anchor)) {
-                    writeMarkupAttributes((MarkupAttributes)anchor);
+                    writeMarkupAttributes(anchor);
                 }
                 write(anchor.font(), styleAttributes);
                 os.write(GT);
@@ -689,13 +693,12 @@ public class HtmlWriter extends DocWriter implements DocListener {
             {
                 Paragraph paragraph = (Paragraph) element;
                 styleAttributes = new Properties();
-                if (paragraph.leadingDefined()) styleAttributes.setProperty(MarkupTags.CSS_LINEHEIGHT, String.valueOf(paragraph.leading()) + "pt");
-                
+                if (paragraph.leadingDefined()) styleAttributes.setProperty(MarkupTags.CSS_KEY_LINEHEIGHT, String.valueOf(paragraph.leading()) + "pt");
                 // start tag
                 addTabs(indent);
-                writeStart(MarkupTags.DIV);
+                writeStart(HtmlTags.DIV);
                 if (hasMarkupAttributes(paragraph)) {
-                    writeMarkupAttributes((MarkupAttributes)paragraph);
+                    writeMarkupAttributes(paragraph);
                 }
                 String alignment = HtmlEncoder.getAlignment(paragraph.alignment());
                 if (!"".equals(alignment)) {
@@ -710,7 +713,7 @@ public class HtmlWriter extends DocWriter implements DocListener {
                 }
                 // end tag
                 addTabs(indent);
-                writeEnd(MarkupTags.DIV);
+                writeEnd(HtmlTags.DIV);
                 currentfont.pop();
                 return;
             }
@@ -733,7 +736,7 @@ public class HtmlWriter extends DocWriter implements DocListener {
                     writeStart(HtmlTags.UNORDEREDLIST);
                 }
                 if (hasMarkupAttributes(list)) {
-                    writeMarkupAttributes((MarkupAttributes)list);
+                    writeMarkupAttributes(list);
                 }
                 os.write(GT);
                 // contents
@@ -754,13 +757,13 @@ public class HtmlWriter extends DocWriter implements DocListener {
             {
                 ListItem listItem = (ListItem) element;
                 styleAttributes = new Properties();
-                if (listItem.leadingDefined()) styleAttributes.setProperty(MarkupTags.CSS_LINEHEIGHT, String.valueOf(listItem.leading()) + "pt");
+                if (listItem.leadingDefined()) styleAttributes.setProperty(MarkupTags.CSS_KEY_LINEHEIGHT, String.valueOf(listItem.leading()) + "pt");
                 
                 // start tag
                 addTabs(indent);
                 writeStart(HtmlTags.LISTITEM);
                 if (hasMarkupAttributes(listItem)) {
-                    writeMarkupAttributes((MarkupAttributes)listItem);
+                    writeMarkupAttributes(listItem);
                 }
                 write(listItem.font(), styleAttributes);
                 os.write(GT);
@@ -788,7 +791,7 @@ public class HtmlWriter extends DocWriter implements DocListener {
                     writeStart(HtmlTags.CELL);
                 }
                 if (hasMarkupAttributes(cell)) {
-                    writeMarkupAttributes((MarkupAttributes)cell);
+                    writeMarkupAttributes(cell);
                 }
                 if (cell.borderWidth() != Rectangle.UNDEFINED) {
                     write(HtmlTags.BORDERWIDTH, String.valueOf(cell.borderWidth()));
@@ -846,7 +849,7 @@ public class HtmlWriter extends DocWriter implements DocListener {
                 addTabs(indent);
                 writeStart(HtmlTags.ROW);
                 if (hasMarkupAttributes(row)) {
-                    writeMarkupAttributes((MarkupAttributes)row);
+                    writeMarkupAttributes(row);
                 }
                 os.write(GT);
                 // contents
@@ -863,13 +866,23 @@ public class HtmlWriter extends DocWriter implements DocListener {
             }
             case Element.TABLE:
             {
-                Table table = (Table) element;
+            	Table table;
+            	try {
+            		table = (Table) element;
+            	}
+            	catch(ClassCastException cce) {
+            		try {
+						table = ((SimpleTable)element).createTable();
+					} catch (BadElementException e) {
+						throw new ExceptionConverter(e);
+					}
+            	}
                 table.complete();
                 // start tag
                 addTabs(indent);
                 writeStart(HtmlTags.TABLE);
                 if (hasMarkupAttributes(table)) {
-                    writeMarkupAttributes((MarkupAttributes)table);
+                    writeMarkupAttributes(table);
                 }
                 os.write(SPACE);
                 write(HtmlTags.WIDTH);
@@ -916,7 +929,7 @@ public class HtmlWriter extends DocWriter implements DocListener {
                 writeComment(annotation.title() + ": " + annotation.content());
                 if (hasMarkupAttributes(annotation)) {
                     os.write(BEGINCOMMENT);
-                    writeMarkupAttributes((MarkupAttributes)annotation);
+                    writeMarkupAttributes(annotation);
                     os.write(ENDCOMMENT);
                 }
                 return;
@@ -943,22 +956,22 @@ public class HtmlWriter extends DocWriter implements DocListener {
                     }
                 }
                 write(HtmlTags.URL, path);
-                if ((image.alignment() & Image.LEFT) > 0) {
-                    write(HtmlTags.ALIGN, HtmlTags.ALIGN_LEFT);
-                }
-                else if ((image.alignment() & Image.RIGHT) > 0) {
+                if ((image.alignment() & Image.RIGHT) > 0) {
                     write(HtmlTags.ALIGN, HtmlTags.ALIGN_RIGHT);
                 }
                 else if ((image.alignment() & Image.MIDDLE) > 0) {
                     write(HtmlTags.ALIGN, HtmlTags.ALIGN_MIDDLE);
                 }
+                else {
+                    write(HtmlTags.ALIGN, HtmlTags.ALIGN_LEFT);
+                }
                 if (image.alt() != null) {
                     write(HtmlTags.ALT, image.alt());
                 }
                 write(HtmlTags.PLAINWIDTH, String.valueOf(image.scaledWidth()));
                 write(HtmlTags.PLAINHEIGHT, String.valueOf(image.scaledHeight()));
                 if (hasMarkupAttributes(image)){
-                    writeMarkupAttributes((MarkupAttributes)image);
+                    writeMarkupAttributes(image);
                 }
                 writeEnd();
                 return;
@@ -974,6 +987,7 @@ public class HtmlWriter extends DocWriter implements DocListener {
  *
  * @param   section     the section to write
  * @param   indent      the indentation
+ * @throws IOException
  */
     
     protected void writeSection(Section section, int indent) throws IOException {
@@ -983,7 +997,7 @@ public class HtmlWriter extends DocWriter implements DocListener {
                 depth = 5;
             }
             Properties styleAttributes = new Properties();
-            if (section.title().leadingDefined()) styleAttributes.setProperty(MarkupTags.CSS_LINEHEIGHT, String.valueOf(section.title().leading()) + "pt");
+            if (section.title().leadingDefined()) styleAttributes.setProperty(MarkupTags.CSS_KEY_LINEHEIGHT, String.valueOf(section.title().leading()) + "pt");
             // start tag
             addTabs(indent);
             writeStart(HtmlTags.H[depth]);
@@ -993,7 +1007,7 @@ public class HtmlWriter extends DocWriter implements DocListener {
                 write(HtmlTags.ALIGN, alignment);
             }
             if (hasMarkupAttributes(section.title())) {
-                writeMarkupAttributes((MarkupAttributes)section.title());
+                writeMarkupAttributes(section.title());
             }
             os.write(GT);
             currentfont.push(section.title().font());
@@ -1016,12 +1030,13 @@ public class HtmlWriter extends DocWriter implements DocListener {
      *
      * @param font              a <CODE>Font</CODE>
      * @param styleAttributes   the style of the font
+     * @throws IOException
      */
     
     protected void write(Font font, Properties styleAttributes) throws IOException {
-        if (font == null || !isOtherFont(font) || styleAttributes == null) return;
+        if (font == null || !isOtherFont(font) /* || styleAttributes == null*/) return;
         write(" ");
-        write(MarkupTags.STYLE);
+        write(HtmlTags.STYLE);
         write("=\"");
         if (styleAttributes != null) {
             String key;
@@ -1031,37 +1046,51 @@ public class HtmlWriter extends DocWriter implements DocListener {
             }
         }
         if (isOtherFont(font)) {
-            writeCssProperty(MarkupTags.CSS_FONTFAMILY, font.getFamilyname());
+            writeCssProperty(MarkupTags.CSS_KEY_FONTFAMILY, font.getFamilyname());
             
             if (font.size() != Font.UNDEFINED) {
-                writeCssProperty(MarkupTags.CSS_FONTSIZE, String.valueOf(font.size()) + "pt");
+                writeCssProperty(MarkupTags.CSS_KEY_FONTSIZE, String.valueOf(font.size()) + "pt");
             }
             if (font.color() != null) {
-                writeCssProperty(MarkupTags.CSS_COLOR, HtmlEncoder.encode(font.color()));
+                writeCssProperty(MarkupTags.CSS_KEY_COLOR, HtmlEncoder.encode(font.color()));
             }
             
             int fontstyle = font.style();
+            BaseFont bf = font.getBaseFont();
+            if (bf != null) {
+                String ps = bf.getPostscriptFontName().toLowerCase();
+                if (ps.indexOf("bold") >= 0) {
+                    if (fontstyle == Font.UNDEFINED)
+                        fontstyle = 0;
+                    fontstyle |= Font.BOLD;
+                }
+                if (ps.indexOf("italic") >= 0 || ps.indexOf("oblique") >= 0) {
+                    if (fontstyle == Font.UNDEFINED)
+                        fontstyle = 0;
+                    fontstyle |= Font.ITALIC;
+                }
+            }
             if (fontstyle != Font.UNDEFINED && fontstyle != Font.NORMAL) {
                 switch (fontstyle & Font.BOLDITALIC) {
                     case Font.BOLD:
-                        writeCssProperty(MarkupTags.CSS_FONTWEIGHT, MarkupTags.CSS_BOLD);
+                        writeCssProperty(MarkupTags.CSS_KEY_FONTWEIGHT, MarkupTags.CSS_VALUE_BOLD);
                         break;
                     case Font.ITALIC:
-                        writeCssProperty(MarkupTags.CSS_FONTSTYLE, MarkupTags.CSS_ITALIC);
+                        writeCssProperty(MarkupTags.CSS_KEY_FONTSTYLE, MarkupTags.CSS_VALUE_ITALIC);
                         break;
                     case Font.BOLDITALIC:
-                        writeCssProperty(MarkupTags.CSS_FONTWEIGHT, MarkupTags.CSS_BOLD);
-                        writeCssProperty(MarkupTags.CSS_FONTSTYLE, MarkupTags.CSS_ITALIC);
+                        writeCssProperty(MarkupTags.CSS_KEY_FONTWEIGHT, MarkupTags.CSS_VALUE_BOLD);
+                        writeCssProperty(MarkupTags.CSS_KEY_FONTSTYLE, MarkupTags.CSS_VALUE_ITALIC);
                         break;
                 }
                 
                 // CSS only supports one decoration tag so if both are specified
                 // only one of the two will display
                 if ((fontstyle & Font.UNDERLINE) > 0) {
-                    writeCssProperty(MarkupTags.CSS_TEXTDECORATION, MarkupTags.CSS_UNDERLINE);
+                    writeCssProperty(MarkupTags.CSS_KEY_TEXTDECORATION, MarkupTags.CSS_VALUE_UNDERLINE);
                 }
                 if ((fontstyle & Font.STRIKETHRU) > 0) {
-                    writeCssProperty(MarkupTags.CSS_TEXTDECORATION, MarkupTags.CSS_LINETHROUGH);
+                    writeCssProperty(MarkupTags.CSS_KEY_TEXTDECORATION, MarkupTags.CSS_VALUE_LINETHROUGH);
                 }
             }
         }
@@ -1070,6 +1099,9 @@ public class HtmlWriter extends DocWriter implements DocListener {
     
     /**
      * Writes out a CSS property.
+     * @param prop a CSS property
+     * @param value the value of the CSS property
+     * @throws IOException
      */
     protected void writeCssProperty(String prop, String value) throws IOException {
         write(new StringBuffer(prop).append(": ").append(value).append("; ").toString());
