@@ -1,8 +1,8 @@
 /*
- * $Id: BaseFont.java,v 1.50 2004/02/07 10:28:35 blowagie Exp $
+ * $Id: BaseFont.java,v 1.70 2006/09/10 15:35:56 psoares33 Exp $
  * $Name:  $
  *
- * Copyright 2000, 2001, 2002 by Paulo Soares.
+ * Copyright 2000-2006 by Paulo Soares.
  *
  * The contents of this file are subject to the Mozilla Public License Version 1.1
  * (the "License"); you may not use this file except in compliance with the License.
@@ -52,6 +52,9 @@ package com.lowagie.text.pdf;
 import java.io.*;
 import com.lowagie.text.DocumentException;
 import java.util.HashMap;
+import java.util.ArrayList;
+import java.util.Iterator;
+import java.util.StringTokenizer;
 
 /**
  * Base class for the several font types supported
@@ -133,9 +136,13 @@ public abstract class BaseFont {
      */    
     public static final int BBOXURY = 8;
     
+    /** java.awt.Font property */
     public static final int AWT_ASCENT = 9;
+    /** java.awt.Font property */
     public static final int AWT_DESCENT = 10;
+    /** java.awt.Font property */
     public static final int AWT_LEADING = 11;
+    /** java.awt.Font property */
     public static final int AWT_MAXADVANCE = 12;
     
     /** The font is Type 1.
@@ -153,6 +160,9 @@ public abstract class BaseFont {
     /** A font already inside the document.
      */    
     public static final int FONT_TYPE_DOCUMENT = 4;
+    /** A Type3 font.
+     */    
+    public static final int FONT_TYPE_T3 = 5;
     /** The Unicode encoding with horizontal writing.
      */    
     public static final String IDENTITY_H = "Identity-H";
@@ -175,7 +185,11 @@ public abstract class BaseFont {
     /** A possible encoding. */    
     public static final String MACROMAN = "MacRoman";
     
-    
+    public static final int[] CHAR_RANGE_LATIN = {0, 0x17f, 0x2000, 0x206f, 0x20a0, 0x20cf, 0xfb00, 0xfb06};
+    public static final int[] CHAR_RANGE_ARABIC = {0, 0x7f, 0x0600, 0x067f, 0x20a0, 0x20cf, 0xfb50, 0xfbff, 0xfe70, 0xfeff};
+    public static final int[] CHAR_RANGE_HEBREW = {0, 0x7f, 0x0590, 0x05ff, 0x20a0, 0x20cf, 0xfb1d, 0xfb4f};
+    public static final int[] CHAR_RANGE_CYRILLIC = {0, 0x7f, 0x0400, 0x052f, 0x2000, 0x206f, 0x20a0, 0x20cf};
+
 /** if the font has to be embedded */
     public static final boolean EMBEDDED = true;
     
@@ -191,6 +205,7 @@ public abstract class BaseFont {
     /** The fake CID code that represents a newline. */    
     public static final char CID_NEWLINE = '\u7fff';
     
+    protected ArrayList subsetRanges;
     /** The font type.
      */    
     int fontType;
@@ -205,6 +220,7 @@ public abstract class BaseFont {
 /** same as differences but with the unicode codes */
     protected char unicodeDifferences[] = new char[256];
     
+    protected int charBBoxes[][] = new int[256][];
 /** encoding used with this font */
     protected String encoding;
     
@@ -241,6 +257,12 @@ public abstract class BaseFont {
     
     protected boolean fastWinansi = false;
     
+    /**
+     * Custom encodings use this map to key the Unicode character
+     * to the single byte code.
+     */
+    protected IntHashtable specialMap;
+    
     static {
         BuiltinFonts14.put(COURIER, PdfName.COURIER);
         BuiltinFonts14.put(COURIER_BOLD, PdfName.COURIER_BOLD);
@@ -261,7 +283,7 @@ public abstract class BaseFont {
     /** Generates the PDF stream with the Type1 and Truetype fonts returning
      * a PdfStream.
      */
-    class StreamFont extends PdfStream {
+    static class StreamFont extends PdfStream {
         
         /** Generates the PDF stream with the Type1 and Truetype fonts returning
          * a PdfStream.
@@ -283,6 +305,12 @@ public abstract class BaseFont {
             }
         }
         
+        /**
+         * Generates the PDF stream for a font.
+         * @param contents the content of a stream
+         * @param subType the subtype of the font.
+         * @throws DocumentException
+         */
         public StreamFont(byte contents[], String subType) throws DocumentException {
             try {
                 bytes = contents;
@@ -303,8 +331,9 @@ public abstract class BaseFont {
     protected BaseFont() {
     }
     
-    /** Creates a new font. This font can be one of the 14 built in types,
-     * a Type1 font referred by an AFM file, a TrueType font (simple or collection) or a CJK font from the
+    /**
+     * Creates a new font. This font can be one of the 14 built in types,
+     * a Type1 font referred to by an AFM or PFM file, a TrueType font (simple or collection) or a CJK font from the
      * Adobe Asian Font Pack. TrueType fonts and CJK fonts can have an optional style modifier
      * appended to the name. These modifiers are: Bold, Italic and BoldItalic. An
      * example would be "STSong-Light,Bold". Note that this modifiers do not work if
@@ -314,6 +343,31 @@ public abstract class BaseFont {
      * The fonts are cached and if they already exist they are extracted from the cache,
      * not parsed again.
      * <P>
+     * Besides the common encodings described by name, custom encodings 
+     * can also be made. These encodings will only work for the single byte fonts
+     * Type1 and TrueType. The encoding string starts with a '#'
+     * followed by "simple" or "full". If "simple" there is a decimal for the first character position and then a list
+     * of hex values representing the Unicode codes that compose that encoding.<br>
+     * The "simple" encoding is recommended for TrueType fonts
+     * as the "full" encoding risks not matching the character with the right glyph
+     * if not done with care.<br>
+     * The "full" encoding is specially aimed at Type1 fonts where the glyphs have to be
+     * described by non standard names like the Tex math fonts. Each group of three elements
+     * compose a code position: the one byte code order in decimal or as 'x' (x cannot be the space), the name and the Unicode character
+     * used to access the glyph. The space must be assigned to character position 32 otherwise
+     * text justification will not work.
+     * <P>
+     * Example for a "simple" encoding that includes the Unicode
+     * character space, A, B and ecyrillic:
+     * <PRE>
+     * "# simple 32 0020 0041 0042 0454"
+     * </PRE>
+     * <P>
+     * Example for a "full" encoding for a Type1 Tex font:
+     * <PRE>
+     * "# full 'A' nottriangeqlleft 0041 'B' dividemultiply 0042 32 space 0020"
+     * </PRE>
+     * <P>
      * This method calls:<br>
      * <PRE>
      * createFont(name, encoding, embedded, true, null, null);
@@ -330,7 +384,7 @@ public abstract class BaseFont {
     }
     
     /** Creates a new font. This font can be one of the 14 built in types,
-     * a Type1 font referred by an AFM file, a TrueType font (simple or collection) or a CJK font from the
+     * a Type1 font referred to by an AFM or PFM file, a TrueType font (simple or collection) or a CJK font from the
      * Adobe Asian Font Pack. TrueType fonts and CJK fonts can have an optional style modifier
      * appended to the name. These modifiers are: Bold, Italic and BoldItalic. An
      * example would be "STSong-Light,Bold". Note that this modifiers do not work if
@@ -339,13 +393,38 @@ public abstract class BaseFont {
      * <P>
      * The fonts may or may not be cached depending on the flag <CODE>cached</CODE>.
      * If the <CODE>byte</CODE> arrays are present the font will be
-     * read from them instead of the name. The name is still required to identify
+     * read from them instead of the name. A name is still required to identify
      * the font type.
+     * <P>
+     * Besides the common encodings described by name, custom encodings 
+     * can also be made. These encodings will only work for the single byte fonts
+     * Type1 and TrueType. The encoding string starts with a '#'
+     * followed by "simple" or "full". If "simple" there is a decimal for the first character position and then a list
+     * of hex values representing the Unicode codes that compose that encoding.<br>
+     * The "simple" encoding is recommended for TrueType fonts
+     * as the "full" encoding risks not matching the character with the right glyph
+     * if not done with care.<br>
+     * The "full" encoding is specially aimed at Type1 fonts where the glyphs have to be
+     * described by non standard names like the Tex math fonts. Each group of three elements
+     * compose a code position: the one byte code order in decimal or as 'x' (x cannot be the space), the name and the Unicode character
+     * used to access the glyph. The space must be assigned to character position 32 otherwise
+     * text justification will not work.
+     * <P>
+     * Example for a "simple" encoding that includes the Unicode
+     * character space, A, B and ecyrillic:
+     * <PRE>
+     * "# simple 32 0020 0041 0042 0454"
+     * </PRE>
+     * <P>
+     * Example for a "full" encoding for a Type1 Tex font:
+     * <PRE>
+     * "# full 'A' nottriangeqlleft 0041 'B' dividemultiply 0042 32 space 0020"
+     * </PRE>
      * @param name the name of the font or it's location on file
      * @param encoding the encoding to be applied to this font
      * @param embedded true if the font is to be embedded in the PDF
      * @param cached true if the font comes from the cache or is added to
-     * the cache if new. false if the font is always created new
+     * the cache if new, false if the font is always created new
      * @param ttfAfm the true type font or the afm in a byte array
      * @param pfb the pfb in a byte array
      * @return returns a new font. This font may come from the cache but only if cached
@@ -372,7 +451,7 @@ public abstract class BaseFont {
             if (fontFound != null)
                 return fontFound;
         }
-        if (isBuiltinFonts14 || name.toLowerCase().endsWith(".afm")) {
+        if (isBuiltinFonts14 || name.toLowerCase().endsWith(".afm") || name.toLowerCase().endsWith(".pfm")) {
             fontBuilt = new Type1Font(name, encoding, embedded, ttfAfm, pfb);
             fontBuilt.fastWinansi = encoding.equals(CP1252);
         }
@@ -399,6 +478,16 @@ public abstract class BaseFont {
         return fontBuilt;
     }
     
+    /**
+     * Creates a font based on an existing document font. The created font font may not
+     * behave as expected, depending on the encoding or subset.
+     * @param fontRef the reference to the document font
+     * @return the font
+     */    
+    public static BaseFont createFont(PRIndirectReference fontRef) {
+        return new DocumentFont(fontRef);
+    }
+    
     /**
      * Gets the name without the modifiers Bold, Italic or BoldItalic.
      * @param name the full name of the font
@@ -432,12 +521,58 @@ public abstract class BaseFont {
     
     /**
      * Creates the <CODE>widths</CODE> and the <CODE>differences</CODE> arrays
-     * @throws UnsupportedEncodingException the encoding is not supported
      */
     protected void createEncoding() {
-        if (fontSpecific) {
-            for (int k = 0; k < 256; ++k)
+        if (encoding.startsWith("#")) {
+            specialMap = new IntHashtable();
+            StringTokenizer tok = new StringTokenizer(encoding.substring(1), " ,\t\n\r\f");
+            if (tok.nextToken().equals("full")) {
+                while (tok.hasMoreTokens()) {
+                    String order = tok.nextToken();
+                    String name = tok.nextToken();
+                    char uni = (char)Integer.parseInt(tok.nextToken(), 16);
+                    int orderK;
+                    if (order.startsWith("'"))
+                        orderK = (int)order.charAt(1);
+                    else
+                        orderK = Integer.parseInt(order);
+                    orderK %= 256;
+                    specialMap.put((int)uni, orderK);
+                    differences[orderK] = name;
+                    unicodeDifferences[orderK] = uni;
+                    widths[orderK] = getRawWidth((int)uni, name);
+                    charBBoxes[orderK] = getRawCharBBox((int)uni, name);
+                }
+            }
+            else {
+                int k = 0;
+                if (tok.hasMoreTokens())
+                    k = Integer.parseInt(tok.nextToken());
+                while (tok.hasMoreTokens() && k < 256) {
+                    String hex = tok.nextToken();
+                    int uni = Integer.parseInt(hex, 16) % 0x10000;
+                    String name = GlyphList.unicodeToName(uni);
+                    if (name != null) {
+                        specialMap.put(uni, k);
+                        differences[k] = name;
+                        unicodeDifferences[k] = (char)uni;
+                        widths[k] = getRawWidth(uni, name);
+                        charBBoxes[k] = getRawCharBBox(uni, name);
+                        ++k;
+                    }
+                }
+            }
+            for (int k = 0; k < 256; ++k) {
+                if (differences[k] == null) {
+                    differences[k] = notdef;
+                }
+            }
+        }
+        else if (fontSpecific) {
+            for (int k = 0; k < 256; ++k) {
                 widths[k] = getRawWidth(k, null);
+                charBBoxes[k] = getRawCharBBox(k, null);
+            }
         }
         else {
             String s;
@@ -459,6 +594,7 @@ public abstract class BaseFont {
                 differences[k] = name;
                 unicodeDifferences[k] = c;
                 widths[k] = getRawWidth((int)c, name);
+                charBBoxes[k] = getRawCharBBox((int)c, name);
             }
         }
     }
@@ -476,9 +612,18 @@ public abstract class BaseFont {
      * Gets the kerning between two Unicode chars.
      * @param char1 the first char
      * @param char2 the second char
-     * @return the kerning to be applied
+     * @return the kerning to be applied in normalized 1000 units
      */
     public abstract int getKerning(char char1, char char2);
+
+    /**
+     * Sets the kerning between two Unicode chars.
+     * @param char1 the first char
+     * @param char2 the second char
+     * @param kern the kerning to apply in normalized 1000 units
+     * @return <code>true</code> if the kerning was applied, <code>false</code> otherwise
+     */
+    public abstract boolean setKerning(char char1, char char2, int kern);
     
     /**
      * Gets the width of a <CODE>char</CODE> in normalized 1000 units.
@@ -520,6 +665,65 @@ public abstract class BaseFont {
         return total;
     }
     
+/**
+ * Gets the descent of a <CODE>String</CODE> in normalized 1000 units. The descent will always be
+ * less than or equal to zero even if all the characters have an higher descent.
+ * @param text the <CODE>String</CODE> to get the descent of
+ * @return the dexcent in normalized 1000 units
+ */
+    public int getDescent(String text) {
+        int min = 0;
+        char chars[] = text.toCharArray();
+        for (int k = 0; k < chars.length; ++k) {
+            int bbox[] = getCharBBox(chars[k]);
+            if (bbox != null && bbox[1] < min)
+                min = bbox[1];
+        }
+        return min;
+    }
+    
+/**
+ * Gets the ascent of a <CODE>String</CODE> in normalized 1000 units. The ascent will always be
+ * greater than or equal to zero even if all the characters have a lower ascent.
+ * @param text the <CODE>String</CODE> to get the ascent of
+ * @return the ascent in normalized 1000 units
+ */
+    public int getAscent(String text) {
+        int max = 0;
+        char chars[] = text.toCharArray();
+        for (int k = 0; k < chars.length; ++k) {
+            int bbox[] = getCharBBox(chars[k]);
+            if (bbox != null && bbox[3] > max)
+                max = bbox[3];
+        }
+        return max;
+    }
+
+/**
+ * Gets the descent of a <CODE>String</CODE> in points. The descent will always be
+ * less than or equal to zero even if all the characters have an higher descent.
+ * @param text the <CODE>String</CODE> to get the descent of
+ * @param fontSize the size of the font
+ * @return the dexcent in points
+ */
+    public float getDescentPoint(String text, float fontSize)
+    {
+        return (float)getDescent(text) * 0.001f * fontSize;
+    }
+    
+/**
+ * Gets the ascent of a <CODE>String</CODE> in points. The ascent will always be
+ * greater than or equal to zero even if all the characters have a lower ascent.
+ * @param text the <CODE>String</CODE> to get the ascent of
+ * @param fontSize the size of the font
+ * @return the ascent in points
+ */
+    public float getAscentPoint(String text, float fontSize)
+    {
+        return (float)getAscent(text) * 0.001f * fontSize;
+    }
+// ia>    
+    
     /**
      * Gets the width of a <CODE>String</CODE> in points taking kerning
      * into account.
@@ -569,6 +773,23 @@ public abstract class BaseFont {
     byte[] convertToBytes(String text) {
         if (directTextToByte)
             return PdfEncodings.convertToBytes(text, null);
+        if (specialMap != null) {
+            byte[] b = new byte[text.length()];
+            int ptr = 0;
+            int length = text.length();
+            for (int k = 0; k < length; ++k) {
+                char c = text.charAt(k);
+                if (specialMap.containsKey((int)c))
+                    b[ptr++] = (byte)specialMap.get((int)c);
+            }
+            if (ptr < length) {
+                byte[] b2 = new byte[ptr];
+                System.arraycopy(b, 0, b2, 0, ptr);
+                return b2;
+            }
+            else
+                return b;
+        }
         return PdfEncodings.convertToBytes(text, encoding);
     }
     
@@ -589,7 +810,8 @@ public abstract class BaseFont {
     }
     
     /** Gets the font parameter identified by <CODE>key</CODE>. Valid values
-     * for <CODE>key</CODE> are <CODE>ASCENT</CODE>, <CODE>CAPHEIGHT</CODE>, <CODE>DESCENT</CODE>,
+     * for <CODE>key</CODE> are <CODE>ASCENT</CODE>, <CODE>AWT_ASCENT</CODE>, <CODE>CAPHEIGHT</CODE>, 
+     * <CODE>DESCENT</CODE>, <CODE>AWT_DESCENT</CODE>,
      * <CODE>ITALICANGLE</CODE>, <CODE>BBOXLLX</CODE>, <CODE>BBOXLLY</CODE>, <CODE>BBOXURX</CODE>
      * and <CODE>BBOXURY</CODE>.
      * @param key the parameter to be extracted
@@ -643,6 +865,13 @@ public abstract class BaseFont {
      */
     public abstract String getPostscriptFontName();
     
+    /**
+     * Sets the font name that will appear in the pdf font dictionary.
+     * Use with care as it can easily make a font unreadable if not embedded.
+     * @param name the new font name
+     */    
+    public abstract void setPostscriptFontName(String name);
+    
     /** Gets the full name of the font. If it is a True Type font
      * each array element will have {Platform ID, Platform Encoding ID,
      * Language ID, font name}. The interpretation of this values can be
@@ -797,8 +1026,10 @@ public abstract class BaseFont {
     }
     
     /** Indicates if all the glyphs and widths for that particular
-     * encoding should be included in the document. Set to <CODE>false</CODE>
-     * to include all.
+     * encoding should be included in the document. When set to <CODE>true</CODE>
+     * only the glyphs used will be included in the font. When set to <CODE>false</CODE>
+     * and {@link #addSubsetRange(int[])} was not called the full font will be included
+     * otherwise just the characters ranges will be included.
      * @param subset new value of property subset
      */
     public void setSubset(boolean subset) {
@@ -900,4 +1131,127 @@ public abstract class BaseFont {
         widths[0xff & b[0]] = advance;
         return true;
     }
+    
+    private static void addFont(PRIndirectReference fontRef, IntHashtable hits, ArrayList fonts) {
+        PdfObject obj = PdfReader.getPdfObject(fontRef);
+        if (obj == null || !obj.isDictionary())
+            return;
+        PdfDictionary font = (PdfDictionary)obj;
+        PdfName subtype = (PdfName)PdfReader.getPdfObject(font.get(PdfName.SUBTYPE));
+        if (!PdfName.TYPE1.equals(subtype) && !PdfName.TRUETYPE.equals(subtype))
+            return;
+        PdfName name = (PdfName)PdfReader.getPdfObject(font.get(PdfName.BASEFONT));
+        fonts.add(new Object[]{PdfName.decodeName(name.toString()), fontRef});
+        hits.put(fontRef.getNumber(), 1);
+    }
+    
+    private static void recourseFonts(PdfDictionary page, IntHashtable hits, ArrayList fonts, int level) {
+        ++level;
+        if (level > 50) // in case we have an endless loop
+            return;
+        PdfDictionary resources = (PdfDictionary)PdfReader.getPdfObject(page.get(PdfName.RESOURCES));
+        if (resources == null)
+            return;
+        PdfDictionary font = (PdfDictionary)PdfReader.getPdfObject(resources.get(PdfName.FONT));
+        if (font != null) {
+            for (Iterator it = font.getKeys().iterator(); it.hasNext();) {
+                PdfObject ft = font.get((PdfName)it.next());        
+                if (ft == null || !ft.isIndirect())
+                    continue;
+                int hit = ((PRIndirectReference)ft).getNumber();
+                if (hits.containsKey(hit))
+                    continue;
+                addFont((PRIndirectReference)ft, hits, fonts);
+            }
+        }
+        PdfDictionary xobj = (PdfDictionary)PdfReader.getPdfObject(resources.get(PdfName.XOBJECT));
+        if (xobj != null) {
+            for (Iterator it = xobj.getKeys().iterator(); it.hasNext();) {
+                recourseFonts((PdfDictionary)PdfReader.getPdfObject(xobj.get((PdfName)it.next())), hits, fonts, level);
+            }
+        }
+    }
+    
+    /**
+     * Gets a list of all document fonts. Each element of the <CODE>ArrayList</CODE>
+     * contains a <CODE>Object[]{String,PRIndirectReference}</CODE> with the font name
+     * and the indirect reference to it.
+     * @param reader the document where the fonts are to be listed from
+     * @return the list of fonts and references
+     */    
+    public static ArrayList getDocumentFonts(PdfReader reader) {
+        IntHashtable hits = new IntHashtable();
+        ArrayList fonts = new ArrayList();
+        int npages = reader.getNumberOfPages();
+        for (int k = 1; k <= npages; ++k)
+            recourseFonts(reader.getPageN(k), hits, fonts, 1);
+        return fonts;
+    }
+    
+    /**
+     * Gets a list of the document fonts in a particular page. Each element of the <CODE>ArrayList</CODE>
+     * contains a <CODE>Object[]{String,PRIndirectReference}</CODE> with the font name
+     * and the indirect reference to it.
+     * @param reader the document where the fonts are to be listed from
+     * @param page the page to list the fonts from
+     * @return the list of fonts and references
+     */    
+    public static ArrayList getDocumentFonts(PdfReader reader, int page) {
+        IntHashtable hits = new IntHashtable();
+        ArrayList fonts = new ArrayList();
+        recourseFonts(reader.getPageN(page), hits, fonts, 1);
+        return fonts;
+    }
+    
+    /**
+     * Gets the smallest box enclosing the character contours. It will return
+     * <CODE>null</CODE> if the font has not the information or the character has no
+     * contours, as in the case of the space, for example. Characters with no contours may
+     * also return [0,0,0,0].
+     * @param c the character to get the contour bounding box from
+     * @return an array of four floats with the bounding box in the format [llx,lly,urx,ury] or
+     * <code>null</code>
+     */    
+    public int[] getCharBBox(char c) {
+        byte b[] = convertToBytes(new String(new char[]{c}));
+        if (b.length == 0)
+            return null;
+        else
+            return charBBoxes[b[0] & 0xff];
+    }
+    
+    protected abstract int[] getRawCharBBox(int c, String name);
+
+    /**
+     * iText expects Arabic Diactrics (tashkeel) to have zero advance but some fonts,
+     * most notably those that come with Windows, like times.ttf, have non-zero
+     * advance for those characters. This method makes those character to have zero
+     * width advance and work correctly in the iText Arabic shaping and reordering
+     * context.
+     */    
+    public void correctArabicAdvance() {
+        for (char c = '\u064b'; c <= '\u0658'; ++c)
+            setCharAdvance(c, 0);
+        setCharAdvance('\u0670', 0);
+        for (char c = '\u06d6'; c <= '\u06dc'; ++c)
+            setCharAdvance(c, 0);
+        for (char c = '\u06df'; c <= '\u06e4'; ++c)
+            setCharAdvance(c, 0);
+        for (char c = '\u06e7'; c <= '\u06e8'; ++c)
+            setCharAdvance(c, 0);
+        for (char c = '\u06ea'; c <= '\u06ed'; ++c)
+            setCharAdvance(c, 0);
+    }
+    
+    /**
+     * Adds a character range when subsetting. The range is an <CODE>int</CODE> array
+     * where the first element is the start range inclusive and the second element is the
+     * end range inclusive. Several ranges are allowed in the same array.
+     * @param range the character range
+     */
+    public void addSubsetRange(int[] range) {
+        if (subsetRanges == null)
+            subsetRanges = new ArrayList();
+        subsetRanges.add(range);
+    }
 }
