@@ -1,5 +1,5 @@
 /*
- * $Id: PdfAnnotation.java,v 1.43 2004/02/07 10:28:38 blowagie Exp $
+ * $Id: PdfAnnotation.java,v 1.67 2006/09/10 15:35:56 psoares33 Exp $
  * $Name:  $
  *
  * Copyright 1999, 2000, 2001, 2002 Bruno Lowagie
@@ -61,35 +61,67 @@ import java.io.*;
  */
 
 public class PdfAnnotation extends PdfDictionary {
-    
+    /** highlight attributename */
     public static final PdfName HIGHLIGHT_NONE = PdfName.N;
+    /** highlight attributename */
     public static final PdfName HIGHLIGHT_INVERT = PdfName.I;
+    /** highlight attributename */
     public static final PdfName HIGHLIGHT_OUTLINE = PdfName.O;
+    /** highlight attributename */
     public static final PdfName HIGHLIGHT_PUSH = PdfName.P;
+    /** highlight attributename */
     public static final PdfName HIGHLIGHT_TOGGLE = PdfName.T;
+    /** flagvalue */
     public static final int FLAGS_INVISIBLE = 1;
+    /** flagvalue */
     public static final int FLAGS_HIDDEN = 2;
+    /** flagvalue */
     public static final int FLAGS_PRINT = 4;
+    /** flagvalue */
     public static final int FLAGS_NOZOOM = 8;
+    /** flagvalue */
     public static final int FLAGS_NOROTATE = 16;
+    /** flagvalue */
     public static final int FLAGS_NOVIEW = 32;
+    /** flagvalue */
     public static final int FLAGS_READONLY = 64;
+    /** flagvalue */
+    public static final int FLAGS_LOCKED = 128;
+    /** flagvalue */
+    public static final int FLAGS_TOGGLENOVIEW = 256;
+    /** appearance attributename */
     public static final PdfName APPEARANCE_NORMAL = PdfName.N;
+    /** appearance attributename */
     public static final PdfName APPEARANCE_ROLLOVER = PdfName.R;
+    /** appearance attributename */
     public static final PdfName APPEARANCE_DOWN = PdfName.D;
+    /** attributevalue */
     public static final PdfName AA_ENTER = PdfName.E;
+    /** attributevalue */
     public static final PdfName AA_EXIT = PdfName.X;
+    /** attributevalue */
     public static final PdfName AA_DOWN = PdfName.D;
+    /** attributevalue */
     public static final PdfName AA_UP = PdfName.U;
+    /** attributevalue */
     public static final PdfName AA_FOCUS = PdfName.FO;
+    /** attributevalue */
     public static final PdfName AA_BLUR = PdfName.BL;
+    /** attributevalue */
     public static final PdfName AA_JS_KEY = PdfName.K;
+    /** attributevalue */
     public static final PdfName AA_JS_FORMAT = PdfName.F;
+    /** attributevalue */
     public static final PdfName AA_JS_CHANGE = PdfName.V;
+    /** attributevalue */
     public static final PdfName AA_JS_OTHER_CHANGE = PdfName.C;
+    /** attributevalue */
     public static final int MARKUP_HIGHLIGHT = 0;
+    /** attributevalue */
     public static final int MARKUP_UNDERLINE = 1;
+    /** attributevalue */
     public static final int MARKUP_STRIKEOUT = 2;
+    
     protected PdfWriter writer;
     protected PdfIndirectReference reference;
     protected HashMap templates;
@@ -103,7 +135,7 @@ public class PdfAnnotation extends PdfDictionary {
     private int placeInPage = -1;
     
     // constructors
-    protected PdfAnnotation(PdfWriter writer, Rectangle rect) {
+    public PdfAnnotation(PdfWriter writer, Rectangle rect) {
         this.writer = writer;
         if (rect != null)
             put(PdfName.RECT, new PdfRectangle(rect));
@@ -111,6 +143,13 @@ public class PdfAnnotation extends PdfDictionary {
     
 /**
  * Constructs a new <CODE>PdfAnnotation</CODE> of subtype text.
+ * @param writer
+ * @param llx
+ * @param lly
+ * @param urx
+ * @param ury
+ * @param title
+ * @param content
  */
     
     PdfAnnotation(PdfWriter writer, float llx, float lly, float urx, float ury, PdfString title, PdfString content) {
@@ -123,6 +162,12 @@ public class PdfAnnotation extends PdfDictionary {
     
 /**
  * Constructs a new <CODE>PdfAnnotation</CODE> of subtype link (Action).
+ * @param writer
+ * @param llx
+ * @param lly
+ * @param urx
+ * @param ury
+ * @param action
  */
     
     public PdfAnnotation(PdfWriter writer, float llx, float lly, float urx, float ury, PdfAction action) {
@@ -137,43 +182,51 @@ public class PdfAnnotation extends PdfDictionary {
     /**
      * Creates a screen PdfAnnotation
      * @param writer
-     * @param llx
-     * @param lly
-     * @param urx
-     * @param ury
-     * @param clipPath
+     * @param rect
+     * @param clipTitle
+     * @param fs
      * @param mimeType
      * @param playOnDisplay
-     * @return
+     * @return a screen PdfAnnotation
+     * @throws IOException
      */
     public static PdfAnnotation createScreen(PdfWriter writer, Rectangle rect, String clipTitle, PdfFileSpecification fs,
                                              String mimeType, boolean playOnDisplay) throws IOException {
         PdfAnnotation ann = new PdfAnnotation(writer, rect);
         ann.put(PdfName.SUBTYPE, PdfName.SCREEN);
         ann.put (PdfName.F, new PdfNumber(FLAGS_PRINT));
-        ann.put(PdfName.BORDER, new PdfBorderArray(0, 0, 0));
-        ann.put(PdfName.C, new PdfColor(0x00, 0x00, 0xFF));
         ann.put(PdfName.TYPE, PdfName.ANNOT);
         ann.setPage();
         PdfIndirectReference ref = ann.getIndirectReference();
         PdfAction action = PdfAction.rendition(clipTitle,fs,mimeType, ref);
+        PdfIndirectReference actionRef = writer.addToBody(action).getIndirectReference();
         // for play on display add trigger event
         if (playOnDisplay)
         {
             PdfDictionary aa = new PdfDictionary();
-            aa.put(new PdfName("PV"), action);
+            aa.put(new PdfName("PV"), actionRef);
             ann.put(PdfName.AA, aa);
         }
+        ann.put(PdfName.A, actionRef);
         return ann;
     }
 
-    PdfIndirectReference getIndirectReference() {
+    public PdfIndirectReference getIndirectReference() {
         if (reference == null) {
             reference = writer.getPdfIndirectReference();
         }
         return reference;
     }
     
+    /**
+     * @param writer
+     * @param rect
+     * @param title
+     * @param contents
+     * @param open
+     * @param icon
+     * @return a PdfAnnotation
+     */
     public static PdfAnnotation createText(PdfWriter writer, Rectangle rect, String title, String contents, boolean open, String icon) {
         PdfAnnotation annot = new PdfAnnotation(writer, rect);
         annot.put(PdfName.SUBTYPE, PdfName.TEXT);
@@ -189,6 +242,13 @@ public class PdfAnnotation extends PdfDictionary {
         return annot;
     }
     
+    /**
+     * Creates a link.
+     * @param writer
+     * @param rect
+     * @param highlight
+     * @return A PdfAnnotation
+     */
     protected static PdfAnnotation createLink(PdfWriter writer, Rectangle rect, PdfName highlight) {
         PdfAnnotation annot = new PdfAnnotation(writer, rect);
         annot.put(PdfName.SUBTYPE, PdfName.LINK);
@@ -197,18 +257,43 @@ public class PdfAnnotation extends PdfDictionary {
         return annot;
     }
     
+    /**
+     * Creates an Annotation with an Action.
+     * @param writer
+     * @param rect
+     * @param highlight
+     * @param action
+     * @return A PdfAnnotation
+     */
     public static PdfAnnotation createLink(PdfWriter writer, Rectangle rect, PdfName highlight, PdfAction action) {
         PdfAnnotation annot = createLink(writer, rect, highlight);
         annot.putEx(PdfName.A, action);
         return annot;
     }
 
+    /**
+     * Creates an Annotation with an local destination.
+     * @param writer
+     * @param rect
+     * @param highlight
+     * @param namedDestination
+     * @return A PdfAnnotation
+     */
     public static PdfAnnotation createLink(PdfWriter writer, Rectangle rect, PdfName highlight, String namedDestination) {
         PdfAnnotation annot = createLink(writer, rect, highlight);
         annot.put(PdfName.DEST, new PdfString(namedDestination));
         return annot;
     }
 
+    /**
+     * Creates an Annotation with a PdfDestination.
+     * @param writer
+     * @param rect
+     * @param highlight
+     * @param page
+     * @param dest
+     * @return A PdfAnnotation
+     */
     public static PdfAnnotation createLink(PdfWriter writer, Rectangle rect, PdfName highlight, int page, PdfDestination dest) {
         PdfAnnotation annot = createLink(writer, rect, highlight);
         PdfIndirectReference ref = writer.getPageReference(page);
@@ -217,6 +302,14 @@ public class PdfAnnotation extends PdfDictionary {
         return annot;
     }
     
+    /**
+     * Add some free text to the document.
+     * @param writer
+     * @param rect
+     * @param contents
+     * @param defaultAppearance
+     * @return A PdfAnnotation
+     */
     public static PdfAnnotation createFreeText(PdfWriter writer, Rectangle rect, String contents, PdfContentByte defaultAppearance) {
         PdfAnnotation annot = new PdfAnnotation(writer, rect);
         annot.put(PdfName.SUBTYPE, PdfName.FREETEXT);
@@ -225,6 +318,17 @@ public class PdfAnnotation extends PdfDictionary {
         return annot;
     }
 
+    /**
+     * Adds a line to the document. Move over the line and a tooltip is shown.
+     * @param writer
+     * @param rect
+     * @param contents
+     * @param x1
+     * @param y1
+     * @param x2
+     * @param y2
+     * @return A PdfAnnotation
+     */
     public static PdfAnnotation createLine(PdfWriter writer, Rectangle rect, String contents, float x1, float y1, float x2, float y2) {
         PdfAnnotation annot = new PdfAnnotation(writer, rect);
         annot.put(PdfName.SUBTYPE, PdfName.LINE);
@@ -237,7 +341,15 @@ public class PdfAnnotation extends PdfDictionary {
         return annot;
     }
 
-    public static PdfAnnotation createSquareCirlcle(PdfWriter writer, Rectangle rect, String contents, boolean square) {
+    /**
+     * Adds a circle or a square that shows a tooltip when you pass over it.
+     * @param writer
+     * @param rect
+     * @param contents The tooltip
+     * @param square true if you want a square, false if you want a circle
+     * @return A PdfAnnotation
+     */
+    public static PdfAnnotation createSquareCircle(PdfWriter writer, Rectangle rect, String contents, boolean square) {
         PdfAnnotation annot = new PdfAnnotation(writer, rect);
         if (square)
             annot.put(PdfName.SUBTYPE, PdfName.SQUARE);
@@ -267,6 +379,14 @@ public class PdfAnnotation extends PdfDictionary {
         return annot;
     }
 
+    /**
+     * Adds a Stamp to your document. Move over the stamp and a tooltip is shown
+     * @param writer
+     * @param rect
+     * @param contents
+     * @param name
+     * @return A PdfAnnotation
+     */
     public static PdfAnnotation createStamp(PdfWriter writer, Rectangle rect, String contents, String name) {
         PdfAnnotation annot = new PdfAnnotation(writer, rect);
         annot.put(PdfName.SUBTYPE, PdfName.STAMP);
@@ -298,7 +418,7 @@ public class PdfAnnotation extends PdfDictionary {
      * @param fileStore an array with the file. If it's <CODE>null</CODE>
      * the file will be read from the disk
      * @param file the path to the file. It will only be used if
-     * <CODE>fileStore</CODE> in not <CODE>null</CODE>
+     * <CODE>fileStore</CODE> is not <CODE>null</CODE>
      * @param fileDisplay the actual file name stored in the pdf
      * @throws IOException on error
      * @return the annotation
@@ -307,18 +427,36 @@ public class PdfAnnotation extends PdfDictionary {
         return createFileAttachment(writer, rect, contents, PdfFileSpecification.fileEmbedded(writer, file, fileDisplay, fileStore));
     }
 
+    /** Creates a file attachment annotation
+     * @param writer
+     * @param rect
+     * @param contents
+     * @param fs
+     * @return the annotation
+     * @throws IOException
+     */
     public static PdfAnnotation createFileAttachment(PdfWriter writer, Rectangle rect, String contents, PdfFileSpecification fs) throws IOException {
         PdfAnnotation annot = new PdfAnnotation(writer, rect);
         annot.put(PdfName.SUBTYPE, PdfName.FILEATTACHMENT);
-        annot.put(PdfName.CONTENTS, new PdfString(contents, PdfObject.TEXT_UNICODE));
+        if (contents != null)
+            annot.put(PdfName.CONTENTS, new PdfString(contents, PdfObject.TEXT_UNICODE));
         annot.put(PdfName.FS, fs.getReference());
         return annot;
     }
     
+    /**
+     * Adds a popup to your document.
+     * @param writer
+     * @param rect
+     * @param contents
+     * @param open
+     * @return A PdfAnnotation
+     */
     public static PdfAnnotation createPopup(PdfWriter writer, Rectangle rect, String contents, boolean open) {
         PdfAnnotation annot = new PdfAnnotation(writer, rect);
         annot.put(PdfName.SUBTYPE, PdfName.POPUP);
-        annot.put(PdfName.CONTENTS, new PdfString(contents, PdfObject.TEXT_UNICODE));
+        if (contents != null)
+            annot.put(PdfName.CONTENTS, new PdfString(contents, PdfObject.TEXT_UNICODE));
         if (open)
             annot.put(PdfName.OPEN, PdfBoolean.PDFTRUE);
         return annot;
@@ -342,14 +480,27 @@ public class PdfAnnotation extends PdfDictionary {
     }
     
     public void setBorder(PdfBorderArray border) {
-        putDel(PdfName.BORDER, border);
+        put(PdfName.BORDER, border);
     }
 
     public void setBorderStyle(PdfBorderDictionary border) {
-        putDel(PdfName.BS, border);
+        put(PdfName.BS, border);
     }
     
-    public void setAppearance(PdfName ap, PdfAppearance template) {
+    /**
+     * Sets the annotation's highlighting mode. The values can be
+     * <CODE>HIGHLIGHT_NONE</CODE>, <CODE>HIGHLIGHT_INVERT</CODE>,
+     * <CODE>HIGHLIGHT_OUTLINE</CODE> and <CODE>HIGHLIGHT_PUSH</CODE>;
+     * @param highlight the annotation's highlighting mode
+     */    
+    public void setHighlighting(PdfName highlight) {
+        if (highlight.equals(HIGHLIGHT_INVERT))
+            remove(PdfName.H);
+        else
+            put(PdfName.H, highlight);
+    }
+    
+    public void setAppearance(PdfName ap, PdfTemplate template) {
         PdfDictionary dic = (PdfDictionary)get(PdfName.AP);
         if (dic == null)
             dic = new PdfDictionary();
@@ -362,7 +513,7 @@ public class PdfAnnotation extends PdfDictionary {
         templates.put(template, null);
     }
 
-    public void setAppearance(PdfName ap, String state, PdfAppearance template) {
+    public void setAppearance(PdfName ap, String state, PdfTemplate template) {
         PdfDictionary dicAp = (PdfDictionary)get(PdfName.AP);
         if (dicAp == null)
             dicAp = new PdfDictionary();
@@ -392,7 +543,7 @@ public class PdfAnnotation extends PdfDictionary {
     }
     
     public void setColor(Color color) {
-        putDel(PdfName.C, new PdfColor(color));
+        put(PdfName.C, new PdfColor(color));
     }
     
     public void setTitle(String title) {
@@ -409,7 +560,7 @@ public class PdfAnnotation extends PdfDictionary {
     }
     
     public void setAction(PdfAction action) {
-        putDel(PdfName.A, action);
+        put(PdfName.A, action);
     }
     
     public void setAdditionalActions(PdfName key, PdfAction action) {
@@ -470,11 +621,122 @@ public class PdfAnnotation extends PdfDictionary {
     }
     
     /** Places the annotation in a specified page that must be greater
-     * or equal to the current one. The first page is 1.
+     * or equal to the current one. With <code>PdfStamper</code> the page
+     * can be any. The first page is 1.
      * @param placeInPage New value of property placeInPage.
      */
     public void setPlaceInPage(int placeInPage) {
         this.placeInPage = placeInPage;
     }
     
+    public void setRotate(int v) {
+        put(PdfName.ROTATE, new PdfNumber(v));
+    }
+    
+    PdfDictionary getMK() {
+        PdfDictionary mk = (PdfDictionary)get(PdfName.MK);
+        if (mk == null) {
+            mk = new PdfDictionary();
+            put(PdfName.MK, mk);
+        }
+        return mk;
+    }
+    
+    public void setMKRotation(int rotation) {
+        getMK().put(PdfName.R, new PdfNumber(rotation));
+    }
+    
+    public static PdfArray getMKColor(Color color) {
+        PdfArray array = new PdfArray();
+        int type = ExtendedColor.getType(color);
+        switch (type) {
+            case ExtendedColor.TYPE_GRAY: {
+                array.add(new PdfNumber(((GrayColor)color).getGray()));
+                break;
+            }
+            case ExtendedColor.TYPE_CMYK: {
+                CMYKColor cmyk = (CMYKColor)color;
+                array.add(new PdfNumber(cmyk.getCyan()));
+                array.add(new PdfNumber(cmyk.getMagenta()));
+                array.add(new PdfNumber(cmyk.getYellow()));
+                array.add(new PdfNumber(cmyk.getBlack()));
+                break;
+            }
+            case ExtendedColor.TYPE_SEPARATION:
+            case ExtendedColor.TYPE_PATTERN:
+            case ExtendedColor.TYPE_SHADING:
+                throw new RuntimeException("Separations, patterns and shadings are not allowed in MK dictionary.");
+            default:
+                array.add(new PdfNumber(color.getRed() / 255f));
+                array.add(new PdfNumber(color.getGreen() / 255f));
+                array.add(new PdfNumber(color.getBlue() / 255f));
+        }
+        return array;
+    }
+    
+    public void setMKBorderColor(Color color) {
+        if (color == null)
+            getMK().remove(PdfName.BC);
+        else
+            getMK().put(PdfName.BC, getMKColor(color));
+    }
+    
+    public void setMKBackgroundColor(Color color) {
+        if (color == null)
+            getMK().remove(PdfName.BG);
+        else
+            getMK().put(PdfName.BG, getMKColor(color));
+    }
+    
+    public void setMKNormalCaption(String caption) {
+        getMK().put(PdfName.CA, new PdfString(caption, PdfObject.TEXT_UNICODE));
+    }
+    
+    public void setMKRolloverCaption(String caption) {
+        getMK().put(PdfName.RC, new PdfString(caption, PdfObject.TEXT_UNICODE));
+    }
+    
+    public void setMKAlternateCaption(String caption) {
+        getMK().put(PdfName.AC, new PdfString(caption, PdfObject.TEXT_UNICODE));
+    }
+    
+    public void setMKNormalIcon(PdfTemplate template) {
+        getMK().put(PdfName.I, template.getIndirectReference());
+    }
+    
+    public void setMKRolloverIcon(PdfTemplate template) {
+        getMK().put(PdfName.RI, template.getIndirectReference());
+    }
+    
+    public void setMKAlternateIcon(PdfTemplate template) {
+        getMK().put(PdfName.IX, template.getIndirectReference());
+    }
+    
+    public void setMKIconFit(PdfName scale, PdfName scalingType, float leftoverLeft, float leftoverBottom, boolean fitInBounds) {
+        PdfDictionary dic = new PdfDictionary();
+        if (!scale.equals(PdfName.A))
+            dic.put(PdfName.SW, scale);
+        if (!scalingType.equals(PdfName.P))
+            dic.put(PdfName.S, scalingType);
+        if (leftoverLeft != 0.5f || leftoverBottom != 0.5f) {
+            PdfArray array = new PdfArray(new PdfNumber(leftoverLeft));
+            array.add(new PdfNumber(leftoverBottom));
+            dic.put(PdfName.A, array);
+        }
+        if (fitInBounds)
+            dic.put(PdfName.FB, PdfBoolean.PDFTRUE);
+        getMK().put(PdfName.IF, dic);
+    }
+    
+    public void setMKTextPosition(int tp) {
+        getMK().put(PdfName.TP, new PdfNumber(tp));
+    }
+    
+    /**
+     * Sets the layer this annotation belongs to.
+     * @param layer the layer this annotation belongs to
+     */    
+    public void setLayer(PdfOCG layer) {
+        put(PdfName.OC, layer.getRef());
+    }
 }
\ No newline at end of file
