@@ -1,5 +1,5 @@
 /*
- * $Id: PdfCopy.java,v 1.18 2004/02/07 10:28:40 blowagie Exp $
+ * $Id: PdfCopy.java,v 1.43 2006/09/10 15:35:56 psoares33 Exp $
  * $Name:  $
  *
  * Copyright 1999, 2000, 2001, 2002 Bruno Lowagie
@@ -71,7 +71,7 @@ public class PdfCopy extends PdfWriter {
      * This class holds information about indirect references, since they are
      * renumbered by iText.
      */
-    class IndirectReferences {
+    static class IndirectReferences {
         PdfIndirectReference theRef;
         boolean hasCopied;
         IndirectReferences(PdfIndirectReference ref) {
@@ -94,7 +94,7 @@ public class PdfCopy extends PdfWriter {
     /**
      * A key to allow us to hash indirect references
      */
-    protected class RefKey {
+    protected static class RefKey {
         int num;
         int gen;
         RefKey(int num, int gen) {
@@ -148,6 +148,7 @@ public class PdfCopy extends PdfWriter {
         if (currentPdfReaderInstance != null) {
             if (currentPdfReaderInstance.getReader() != reader) {
                 try {
+                    currentPdfReaderInstance.getReader().close();
                     currentPdfReaderInstance.getReaderFile().close();
                 }
                 catch (IOException ioe) {
@@ -190,8 +191,8 @@ public class PdfCopy extends PdfWriter {
             indirects.put(key, iRef);
         }
         iRef.setCopied();
-        PdfObject obj = copyObject((PdfObject)PdfReader.getPdfObject(in));
-        PdfIndirectObject theObj = body.add(obj, theRef);
+        PdfObject obj = copyObject(PdfReader.getPdfObjectRelease(in));
+        addToBody(obj, theRef);
         return theRef;
     }
     
@@ -206,11 +207,13 @@ public class PdfCopy extends PdfWriter {
         
         for (Iterator it = in.getKeys().iterator(); it.hasNext();) {
             PdfName key = (PdfName)it.next();
-            PdfObject value = (PdfObject) in.get(key);
+            PdfObject value = in.get(key);
             //	    System.out.println("Copy " + key);
-            if (key.equals(PdfName.PARENT) &&
-            type != null && ((PdfName)type).equals(PdfName.PAGE)) {
-                out.put(PdfName.PARENT, topPageParent);
+            if (type != null && PdfName.PAGE.equals(type)) {
+                if (key.equals(PdfName.PARENT))
+                    out.put(PdfName.PARENT, topPageParent);
+                else if (!key.equals(PdfName.B))
+                    out.put(key, copyObject(value));
             }
             else
                 out.put(key, copyObject(value));
@@ -226,7 +229,7 @@ public class PdfCopy extends PdfWriter {
         
         for (Iterator it = in.getKeys().iterator(); it.hasNext();) {
             PdfName key = (PdfName) it.next();
-            PdfObject value = (PdfObject)in.get(key);
+            PdfObject value = in.get(key);
             out.put(key, copyObject(value));
         }
         
@@ -252,6 +255,8 @@ public class PdfCopy extends PdfWriter {
      * Translate a PR-object to a Pdf-object
      */
     protected PdfObject copyObject(PdfObject in) throws IOException,BadPdfFormatException {
+        if (in == null)
+            return PdfNull.PDFNULL;
         switch (in.type) {
             case PdfObject.DICTIONARY:
                 //	        System.out.println("Dictionary: " + in.toString());
@@ -305,11 +310,13 @@ public class PdfCopy extends PdfWriter {
             PdfDictionary catalog = reader.getCatalog();
             PRIndirectReference ref = (PRIndirectReference)catalog.get(PdfName.PAGES);
             indirects.put(new RefKey(ref), new IndirectReferences(topPageParent));
-            ref = (PRIndirectReference)catalog.get(PdfName.ACROFORM);
-            if (ref != null) {
-                if (acroForm == null) acroForm = body.getPdfIndirectReference();
-                indirects.put(new RefKey(ref), new IndirectReferences(acroForm));
-            }
+            ref = null;
+            PdfObject o = catalog.get(PdfName.ACROFORM);
+            if (o == null || o.type() != PdfObject.INDIRECT)
+                return;
+            ref = (PRIndirectReference)o;
+            if (acroForm == null) acroForm = body.getPdfIndirectReference();
+            indirects.put(new RefKey(ref), new IndirectReferences(acroForm));
         }
     }
     /**
@@ -322,6 +329,7 @@ public class PdfCopy extends PdfWriter {
         
         PdfDictionary thePage = reader.getPageN(pageNum);
         PRIndirectReference origRef = reader.getPageOrigRef(pageNum);
+        reader.releasePage(pageNum);
         RefKey key = new RefKey(origRef);
         PdfIndirectReference pageRef;
         IndirectReferences iRef = (IndirectReferences)indirects.get(key);
@@ -340,7 +348,7 @@ public class PdfCopy extends PdfWriter {
             iRef.setCopied();
             PdfDictionary newPage = copyDictionary(thePage);
             newPage.put(PdfName.PARENT, topPageParent);
-            PdfIndirectObject pageObj = body.add(newPage, pageRef);
+            addToBody(newPage, pageRef);
         }
         root.addPage(pageRef);
         pageNumbersToRefs.add(pageRef);
@@ -362,7 +370,11 @@ public class PdfCopy extends PdfWriter {
         setFromReader(reader);
         
         PdfDictionary catalog = reader.getCatalog();
-        PRIndirectReference hisRef = (PRIndirectReference)catalog.get(PdfName.ACROFORM);
+        PRIndirectReference hisRef = null;
+        PdfObject o = catalog.get(PdfName.ACROFORM);
+        if (o != null && o.type() == PdfObject.INDIRECT)
+            hisRef = (PRIndirectReference)o;
+        if (hisRef == null) return; // bugfix by John Englar
         RefKey key = new RefKey(hisRef);
         PdfIndirectReference myRef;
         IndirectReferences iRef = (IndirectReferences)indirects.get(key);
@@ -377,7 +389,7 @@ public class PdfCopy extends PdfWriter {
         if (! iRef.getCopied()) {
             iRef.setCopied();
             PdfDictionary theForm = copyDictionary((PdfDictionary)PdfReader.getPdfObject(hisRef));
-            PdfIndirectObject myObj = body.add(theForm, myRef);
+            addToBody(theForm, myRef);
         }
     }
     
@@ -387,8 +399,6 @@ public class PdfCopy extends PdfWriter {
      */
     protected PdfDictionary getCatalog(PdfIndirectReference rootObj) {
         try {
-            if (newBookmarks != null && newBookmarks.size() > 0)
-                setViewerPreferences(PageModeUseOutlines);
             PdfDictionary theCat = ((PdfDocument)document).getCatalog(rootObj);
             if (acroForm != null) theCat.put(PdfName.ACROFORM, acroForm);
             if (newBookmarks == null || newBookmarks.size() == 0)
@@ -401,7 +411,6 @@ public class PdfCopy extends PdfWriter {
             top.put(PdfName.COUNT, new PdfNumber(((Integer)kids[2]).intValue()));
             addToBody(top, topRef);
             theCat.put(PdfName.OUTLINES, topRef);
-            
             return theCat;
         }
         catch (IOException e) {
@@ -409,6 +418,11 @@ public class PdfCopy extends PdfWriter {
         }
     }
     
+    /**
+     * Sets the bookmarks. The list structure is defined in
+     * <CODE>SimpleBookmark#</CODE>.
+     * @param outlines the bookmarks or <CODE>null</CODE> to remove any
+     */    
     public void setOutlines(List outlines) {
         newBookmarks = outlines;
     }
@@ -430,6 +444,7 @@ public class PdfCopy extends PdfWriter {
             super.close();
             if (ri != null) {
                 try {
+                    ri.getReader().close();
                     ri.getReaderFile().close();
                 }
                 catch (IOException ioe) {
@@ -438,8 +453,24 @@ public class PdfCopy extends PdfWriter {
             }
         }
     }
-    PdfIndirectReference add(PdfImage pdfImage) throws PdfException  { return null; }
+    PdfIndirectReference add(PdfImage pdfImage, PdfIndirectReference fixedRef) throws PdfException  { return null; }
     public PdfIndirectReference add(PdfOutline outline) { return null; }
     public void addAnnotation(PdfAnnotation annot) {  }
     PdfIndirectReference add(PdfPage page, PdfContents contents) throws PdfException { return null; }
+
+    public void freeReader(PdfReader reader) throws IOException {
+        indirectMap.remove(reader);
+        if (currentPdfReaderInstance != null) {
+            if (currentPdfReaderInstance.getReader() == reader) {
+                try {
+                    currentPdfReaderInstance.getReader().close();
+                    currentPdfReaderInstance.getReaderFile().close();
+                }
+                catch (IOException ioe) {
+                    // empty on purpose
+                }
+                currentPdfReaderInstance = null;
+            }
+        }
+    }
 }
\ No newline at end of file
