@@ -1,4 +1,6 @@
 /*
+ * $Id: PdfGraphics2D.java,v 1.54 2006/09/05 11:22:07 psoares33 Exp $
+ *
  * Copyright 2002 by Jim Moore <jim@scolamoore.com>.
  *
  * The contents of this file are subject to the Mozilla Public License Version 1.1
@@ -47,6 +49,7 @@
 
 package com.lowagie.text.pdf;
 
+import java.awt.AlphaComposite;
 import java.awt.BasicStroke;
 import java.awt.Color;
 import java.awt.Component;
@@ -69,7 +72,6 @@ import java.awt.Stroke;
 import java.awt.Transparency;
 import java.awt.font.FontRenderContext;
 import java.awt.font.GlyphVector;
-import java.awt.font.TextLayout;
 import java.awt.geom.AffineTransform;
 import java.awt.geom.Arc2D;
 import java.awt.geom.Area;
@@ -88,12 +90,12 @@ import java.awt.image.renderable.RenderableImage;
 import java.awt.RenderingHints.Key;
 import java.awt.geom.NoninvertibleTransformException;
 import java.awt.geom.Point2D;
-import java.text.CharacterIterator;
 import java.text.AttributedCharacterIterator;
 import java.util.Map;
 import java.util.HashMap;
 import java.util.Hashtable;
 import java.util.ArrayList;
+import java.io.ByteArrayOutputStream;
 //
 import java.util.Set;
 import java.util.Iterator;
@@ -133,13 +135,11 @@ public class PdfGraphics2D extends Graphics2D {
     
     private FontMapper fontMapper;
     
-    private PdfFontMetrics fontMetrics;
-    
     private ArrayList kids;
     
     private boolean kid = false;
     
-    private Graphics dg2 = new BufferedImage(2, 2, BufferedImage.TYPE_INT_RGB).getGraphics();
+    private Graphics2D dg2 = new BufferedImage(2, 2, BufferedImage.TYPE_INT_RGB).createGraphics();
     
     private boolean onlyShapes = false;
     
@@ -159,40 +159,47 @@ public class PdfGraphics2D extends Graphics2D {
     
     public static int AFM_DIVISOR = 1000; // used to calculate coordinates
 
+    private boolean convertImagesToJPEG = false;
+    private float jpegQuality = .95f;
+
+	// Added by Alexej Suchov
+	private float alpha;
+
+	// Added by Alexej Suchov
+	private Composite composite;
+
+	// Added by Alexej Suchov
+	private Paint realPaint;
+
     private PdfGraphics2D() {
+        dg2.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS, RenderingHints.VALUE_FRACTIONALMETRICS_ON);
+        setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS, RenderingHints.VALUE_FRACTIONALMETRICS_ON);
     }
     
     /**
      * Constructor for PDFGraphics2D.
      *
      */
-    PdfGraphics2D(PdfContentByte cb, float width, float height, FontMapper fontMapper) {
+    PdfGraphics2D(PdfContentByte cb, float width, float height, FontMapper fontMapper, boolean onlyShapes, boolean convertImagesToJPEG, float quality) {
         super();
+        dg2.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS, RenderingHints.VALUE_FRACTIONALMETRICS_ON);
+        setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS, RenderingHints.VALUE_FRACTIONALMETRICS_ON);
+        try {
+            Class.forName("com.sun.image.codec.jpeg.JPEGCodec");
+        }
+        catch (Throwable t) {
+            convertImagesToJPEG = false;
+        }
+        this.convertImagesToJPEG = convertImagesToJPEG;
+        this.jpegQuality = quality;
+        this.onlyShapes = onlyShapes;
         this.transform = new AffineTransform();
         this.baseFonts = new HashMap();
-        this.fontMapper = fontMapper;
-        this.kids = new ArrayList();
-        if (this.fontMapper == null)
-            this.fontMapper = new DefaultFontMapper();
-        paint = Color.black;
-        background = Color.white;
-        setFont(new Font("sanserif", Font.PLAIN, 12));
-        this.cb = cb;
-        cb.saveState();
-        this.width = width;
-        this.height = height;
-        clip = new Area(new Rectangle2D.Float(0, 0, width, height));
-        clip(clip);
-        originalStroke = stroke = oldStroke = strokeOne;
-        setStrokeDiff(stroke, null);
-        cb.saveState();
-    }
-    
-    PdfGraphics2D(PdfContentByte cb, float width, float height) {
-        super();
-        this.onlyShapes = true;
-        this.transform = new AffineTransform();
-        this.kids = new ArrayList();
+        if (!onlyShapes) {
+            this.fontMapper = fontMapper;
+            if (this.fontMapper == null)
+                this.fontMapper = new DefaultFontMapper();
+        }
         paint = Color.black;
         background = Color.white;
         setFont(new Font("sanserif", Font.PLAIN, 12));
@@ -225,8 +232,11 @@ public class PdfGraphics2D extends Graphics2D {
      * @see Graphics2D#drawImage(BufferedImage, BufferedImageOp, int, int)
      */
     public void drawImage(BufferedImage img, BufferedImageOp op, int x, int y) {
-        BufferedImage result = op.createCompatibleDestImage(img, img.getColorModel());
-        result = op.filter(img, result);
+        BufferedImage result = img;
+        if (op != null) {
+            result = op.createCompatibleDestImage(img, img.getColorModel());
+            result = op.filter(img, result);
+        }
         drawImage(result, x, y, null);
     }
     
@@ -272,10 +282,10 @@ public class PdfGraphics2D extends Graphics2D {
     }
     
     /**
-     * @calculates position and/or stroke thickness depending on the font size
+     * Calculates position and/or stroke thickness depending on the font size
      * @param d value to be converted
      * @param i font size
-     * @return
+     * @return position and/or stroke thickness depending on the font size
      */
     public static double asPoints(double d, int i) {
         return (d * (double)i) / (double)AFM_DIVISOR;
@@ -289,7 +299,10 @@ public class PdfGraphics2D extends Graphics2D {
         underline = false;
         Set set = iter.getAttributes().keySet();
         for(Iterator iterator = set.iterator(); iterator.hasNext();) {
-            TextAttribute textattribute = (TextAttribute)iterator.next();
+            AttributedCharacterIterator.Attribute attribute = (AttributedCharacterIterator.Attribute)iterator.next();
+            if (!(attribute instanceof TextAttribute))
+                continue;
+            TextAttribute textattribute = (TextAttribute)attribute;
             if(textattribute.equals(TextAttribute.FONT)) {
                 Font font = (Font)iter.getAttributes().get(textattribute);
                 setFont(font);
@@ -298,33 +311,37 @@ public class PdfGraphics2D extends Graphics2D {
                 if(iter.getAttributes().get(textattribute) == TextAttribute.UNDERLINE_ON)
                     underline = true;
             }
-            else if(textattribute.equals(TextAttribute.SUPERSCRIPT)) {
-                /*
-                iter.getAttributes().get(textattribute);
-                Integer _tmp = TextAttribute.SUPERSCRIPT_SUPER;
-                subscript = true;
-                 */
-            }
             else if(textattribute.equals(TextAttribute.SIZE)) {
                 Object obj = iter.getAttributes().get(textattribute);
-                Font font1 = null;
                 if(obj instanceof Integer) {
                     int i = ((Integer)obj).intValue();
-                    font1 = getFont().deriveFont(getFont().getStyle(), i);
+                    setFont(getFont().deriveFont(getFont().getStyle(), i));
                 }
                 else if(obj instanceof Float) {
                     float f = ((Float)obj).floatValue();
-                    font1 = getFont().deriveFont(getFont().getStyle(), f);
+                    setFont(getFont().deriveFont(getFont().getStyle(), f));
                 }
-                else {
-                    //System.out.println("Unknown type for attribute SIZE");
-                    return;
-                }
-                setFont(font1);
             }
-            else {
-                String s = "only FONT/SIZE/UNDERLINE/SUPERSCRIPT supported";
-                throw new RuntimeException(s);
+            else if(textattribute.equals(TextAttribute.FOREGROUND)) {
+                setColor((Color) iter.getAttributes().get(textattribute));
+            }
+            else if(textattribute.equals(TextAttribute.FAMILY)) {
+              Font font = getFont();
+              Map fontAttributes = font.getAttributes();
+              fontAttributes.put(TextAttribute.FAMILY, iter.getAttributes().get(textattribute));
+              setFont(font.deriveFont(fontAttributes));
+            }
+            else if(textattribute.equals(TextAttribute.POSTURE)) {
+              Font font = getFont();
+              Map fontAttributes = font.getAttributes();
+              fontAttributes.put(TextAttribute.POSTURE, iter.getAttributes().get(textattribute));
+              setFont(font.deriveFont(fontAttributes)); 
+            }
+            else if(textattribute.equals(TextAttribute.WEIGHT)) {
+              Font font = getFont();
+              Map fontAttributes = font.getAttributes();
+              fontAttributes.put(TextAttribute.WEIGHT, iter.getAttributes().get(textattribute));
+              setFont(font.deriveFont(fontAttributes)); 
             }
         }
     }
@@ -333,10 +350,13 @@ public class PdfGraphics2D extends Graphics2D {
      * @see Graphics2D#drawString(String, float, float)
      */
     public void drawString(String s, float x, float y) {
+        if (s.length() == 0)
+            return;
         setFillPaint();
         if (onlyShapes) {
-            TextLayout tl = new TextLayout(s, this.font, new FontRenderContext(new AffineTransform(), false, true));
-            tl.draw(this, x, y);
+            drawGlyphVector(this.font.layoutGlyphVector(getFontRenderContext(), s.toCharArray(), 0, s.length(), java.awt.Font.LAYOUT_LEFT_TO_RIGHT), x, y);
+//            Use the following line to compile in JDK 1.3    
+//            drawGlyphVector(this.font.createGlyphVector(getFontRenderContext(), s), x, y);
         }
         else {
             AffineTransform at = getTransform();
@@ -352,25 +372,36 @@ public class PdfGraphics2D extends Graphics2D {
             cb.beginText();
             cb.setFontAndSize(baseFont, fontSize);
             cb.setTextMatrix((float)mx[0], (float)mx[1], (float)mx[2], (float)mx[3], (float)mx[4], (float)mx[5]);
+            double width = 0;
+            if (font.getSize2D() > 0) {
+                float scale = 1000 / font.getSize2D();
+                width = font.deriveFont(AffineTransform.getScaleInstance(scale, scale)).getStringBounds(s, getFontRenderContext()).getWidth() / scale;
+            }
+            if (s.length() > 1) {
+                float adv = ((float)width - baseFont.getWidthPoint(s, fontSize)) / (s.length() - 1);
+                cb.setCharacterSpacing(adv);
+            }
             cb.showText(s);
+            if (s.length() > 1) {
+                cb.setCharacterSpacing(0);
+            }
             cb.endText();
             setTransform(at);
             if(underline)
             {
                 // These two are supposed to be taken from the .AFM file
-                int UnderlinePosition = -100;
+                //int UnderlinePosition = -100;
                 int UnderlineThickness = 50;
                 //
-                FontMetrics fm = getFontMetrics();
-                double width = fm.stringWidth(s);
                 double d = asPoints((double)UnderlineThickness, (int)fontSize);
                 setStroke(new BasicStroke((float)d));
-                y = (float)((double)(y) + asPoints((double)(-UnderlineThickness), (int)fontSize));
+                y = (float)((double)(y) + asPoints((double)(UnderlineThickness), (int)fontSize));
                 Line2D line = new Line2D.Double((double)x, (double)y, (double)(width+x), (double)y);
                 draw(line);
             }
         }
     }
+
     /**
      * @see Graphics#drawString(AttributedCharacterIterator, int, int)
      */
@@ -443,24 +474,61 @@ public class PdfGraphics2D extends Graphics2D {
      * @see Graphics2D#getDeviceConfiguration()
      */
     public GraphicsConfiguration getDeviceConfiguration() {
-        return ((Graphics2D)dg2).getDeviceConfiguration();
+        return dg2.getDeviceConfiguration();
     }
     
     /**
      * @see Graphics2D#setComposite(Composite)
+	 * @author Alexej Suchov
      */
     public void setComposite(Composite comp) {
         
+		if (comp instanceof AlphaComposite) {
+
+			AlphaComposite composite = (AlphaComposite) comp;
+
+			if (composite.getRule() == 3) {
+
+				alpha = composite.getAlpha();
+				this.composite = composite;
+
+				if (realPaint != null && (realPaint instanceof Color)) {
+
+					Color c = (Color) realPaint;
+					paint = new Color(c.getRed(), c.getGreen(), c.getBlue(),
+							(int) ((float) c.getAlpha() * alpha));
+				}
+				return;
+			}
+		}
+
+		this.composite = comp;
+		alpha = 1.0F;
+
     }
     
     /**
      * @see Graphics2D#setPaint(Paint)
+	 * 
+	 * @author Alexej Suchov
      */
     public void setPaint(Paint paint) {
         if (paint == null)
             return;
         this.paint = paint;
-//        setPaint(paint, false, 0, 0);
+		realPaint = paint;
+
+		if ((composite instanceof AlphaComposite) && (paint instanceof Color)) {
+			
+			AlphaComposite co = (AlphaComposite) composite;
+			
+			if (co.getRule() == 3) {
+				Color c = (Color) paint;
+				this.paint = new Color(c.getRed(), c.getGreen(), c.getBlue(), (int) ((float) c.getAlpha() * alpha));
+				realPaint = paint;
+			}
+		}
+
     }
 
     private Stroke transformStroke(Stroke stroke) {
@@ -563,14 +631,17 @@ public class PdfGraphics2D extends Graphics2D {
     
     
     /**
-     * @see Graphics2D#setRenderingHint(Key, Object)
+     * Sets a rendering hint
+     * @param arg0
+     * @param arg1
      */
     public void setRenderingHint(Key arg0, Object arg1) {
         rhints.put(arg0, arg1);
     }
     
     /**
-     * @see Graphics2D#getRenderingHint(Key)
+     * @param arg0 a key
+     * @return the rendering hint
      */
     public Object getRenderingHint(Key arg0) {
         return rhints.get(arg0);
@@ -666,10 +737,15 @@ public class PdfGraphics2D extends Graphics2D {
     
     /**
      * @see Graphics2D#getPaint()
+	 * @author Alexej Suchov
      */
     public Paint getPaint() {
-        return paint;
-    }
+        if (realPaint != null) {
+            return realPaint;
+        } else {
+            return paint;
+        }
+	}
     
     /**
      * @see Graphics2D#getComposite()
@@ -704,7 +780,9 @@ public class PdfGraphics2D extends Graphics2D {
      * @see Graphics2D#getFontRenderContext()
      */
     public FontRenderContext getFontRenderContext() {
-        return new FontRenderContext(null, true, true);
+        boolean antialias = RenderingHints.VALUE_TEXT_ANTIALIAS_ON.equals(getRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING));
+        boolean fractions = RenderingHints.VALUE_FRACTIONALMETRICS_ON.equals(getRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS));
+        return new FontRenderContext(new AffineTransform(), antialias, fractions);
     }
     
     /**
@@ -716,12 +794,13 @@ public class PdfGraphics2D extends Graphics2D {
         g2.transform = new AffineTransform(this.transform);
         g2.baseFonts = this.baseFonts;
         g2.fontMapper = this.fontMapper;
-        g2.kids = this.kids;
         g2.paint = this.paint;
         g2.fillGState = this.fillGState;
         g2.strokeGState = this.strokeGState;
         g2.background = this.background;
         g2.mediaTracker = this.mediaTracker;
+        g2.convertImagesToJPEG = this.convertImagesToJPEG;
+        g2.jpegQuality = this.jpegQuality;
         g2.setFont(this.font);
         g2.cb = this.cb.getDuplicate();
         g2.cb.saveState();
@@ -739,9 +818,10 @@ public class PdfGraphics2D extends Graphics2D {
         if (g2.clip != null)
             g2.followPath(g2.clip, CLIP);
         g2.kid = true;
-        synchronized (kids) {
-            kids.add(g2);
-        }
+        if (this.kids == null)
+            this.kids = new ArrayList();
+        this.kids.add(new Integer(cb.getInternalBuffer().size()));
+        this.kids.add(g2);
         return g2;
     }
     
@@ -800,7 +880,6 @@ public class PdfGraphics2D extends Graphics2D {
         }
         if (f == font)
             return;
-        fontMetrics = null;
         font = f;
         fontSize = f.getSize2D();
         baseFont = getCachedBaseFont(f);
@@ -821,15 +900,7 @@ public class PdfGraphics2D extends Graphics2D {
      * @see Graphics#getFontMetrics(Font)
      */
     public FontMetrics getFontMetrics(Font f) {
-        if (onlyShapes) {
-            return dg2.getFontMetrics(f);
-        }
-        if (f == font) {
-            if (fontMetrics == null)
-                fontMetrics = new PdfFontMetrics(f, getCachedBaseFont(f));
-            return fontMetrics;
-        }
-        return new PdfFontMetrics(f, getCachedBaseFont(f));
+        return dg2.getFontMetrics(f);
     }
     
     /**
@@ -1089,6 +1160,7 @@ public class PdfGraphics2D extends Graphics2D {
         Graphics g = mask.getGraphics();
         g.fillRect(sx1,sy1, (int)swidth, (int)sheight);
         drawImage(img, mask, tx, null, observer);
+        g.dispose();
         return true;
     }
     
@@ -1102,15 +1174,38 @@ public class PdfGraphics2D extends Graphics2D {
             disposeCalled = true;
             cb.restoreState();
             cb.restoreState();
-            for (int k = 0; k < kids.size(); ++k) {
-                PdfGraphics2D g2 = (PdfGraphics2D)kids.get(k);
-                g2.cb.restoreState();
-                g2.cb.restoreState();
-                cb.add(g2.cb);
+            dg2.dispose();
+            dg2 = null;
+            if (kids != null) {
+                ByteBuffer buf = new ByteBuffer();
+                internalDispose(buf);
+                ByteBuffer buf2 = cb.getInternalBuffer();
+                buf2.reset();
+                buf2.append(buf);
             }
         }
     }
     
+    private void internalDispose(ByteBuffer buf) {
+        int last = 0;
+        int pos = 0;
+        ByteBuffer buf2 = cb.getInternalBuffer();
+        if (kids != null) {
+            for (int k = 0; k < kids.size(); k += 2) {
+                pos = ((Integer)kids.get(k)).intValue();
+                PdfGraphics2D g2 = (PdfGraphics2D)kids.get(k + 1);
+                g2.cb.restoreState();
+                g2.cb.restoreState();
+                buf.append(buf2.getBuffer(), last, pos - last);
+                g2.dg2.dispose();
+                g2.dg2 = null;
+                g2.internalDispose(buf);
+                last = pos;
+            }
+        }
+        buf.append(buf2.getBuffer(), last, buf2.size() - last);
+    }
+    
     ///////////////////////////////////////////////
     //
     //
@@ -1136,12 +1231,12 @@ public class PdfGraphics2D extends Graphics2D {
         else if (drawType==FILL)
             setFillPaint();
         PathIterator points;
+        int traces = 0;
         if (drawType == CLIP)
             points = s.getPathIterator(IDENTITY);
         else
             points = s.getPathIterator(transform);
         float[] coords = new float[6];
-        int traces = 0;
         while(!points.isDone()) {
             ++traces;
             int segtype = points.currentSegment(coords);
@@ -1150,26 +1245,25 @@ public class PdfGraphics2D extends Graphics2D {
                 case PathIterator.SEG_CLOSE:
                     cb.closePath();
                     break;
-                    
+
                 case PathIterator.SEG_CUBICTO:
                     cb.curveTo(coords[0], coords[1], coords[2], coords[3], coords[4], coords[5]);
                     break;
-                    
+
                 case PathIterator.SEG_LINETO:
                     cb.lineTo(coords[0], coords[1]);
                     break;
-                    
+
                 case PathIterator.SEG_MOVETO:
                     cb.moveTo(coords[0], coords[1]);
                     break;
-                    
+
                 case PathIterator.SEG_QUADTO:
                     cb.curveTo(coords[0], coords[1], coords[2], coords[3]);
                     break;
             }
             points.next();
         }
-        
         switch (drawType) {
         case FILL:
             if (traces > 0) {
@@ -1228,9 +1322,37 @@ public class PdfGraphics2D extends Graphics2D {
         
         double[] mx = new double[6];
         inverse.getMatrix(mx);
+        if (currentFillGState != 255) {
+            PdfGState gs = fillGState[255];
+            if (gs == null) {
+                gs = new PdfGState();
+                gs.setFillOpacity(1);
+                fillGState[255] = gs;
+            }
+            cb.setGState(gs);
+        }
         
         try {
-            com.lowagie.text.Image image = com.lowagie.text.Image.getInstance(img, bgColor);
+            com.lowagie.text.Image image = null;
+            if(!convertImagesToJPEG){
+                image = com.lowagie.text.Image.getInstance(img, bgColor);
+            }
+            else{
+                BufferedImage scaled = new BufferedImage(img.getWidth(null), img.getHeight(null), BufferedImage.TYPE_INT_RGB);
+                Graphics2D g3 = scaled.createGraphics();
+                g3.drawImage(img, 0, 0, img.getWidth(null), img.getHeight(null), null);
+                g3.dispose();
+                
+                ByteArrayOutputStream baos = new ByteArrayOutputStream();
+                com.sun.image.codec.jpeg.JPEGImageEncoder encoder = com.sun.image.codec.jpeg.JPEGCodec.createJPEGEncoder(baos);
+                com.sun.image.codec.jpeg.JPEGEncodeParam param = com.sun.image.codec.jpeg.JPEGCodec.getDefaultJPEGEncodeParam(scaled);
+                param.setQuality(jpegQuality, true);
+                encoder.encode(scaled, param);
+                scaled.flush();
+                scaled = null;
+                image = com.lowagie.text.Image.getInstance(baos.toByteArray());
+                
+            }
             if (mask!=null) {
                 com.lowagie.text.Image msk = com.lowagie.text.Image.getInstance(mask, null, true);
                 msk.makeMask();
@@ -1241,7 +1363,10 @@ public class PdfGraphics2D extends Graphics2D {
         } catch (Exception ex) {
             throw new IllegalArgumentException();
         }
-        
+        if (currentFillGState != 255) {
+            PdfGState gs = fillGState[currentFillGState];
+            cb.setGState(gs);
+        }        
         return true;
     }
     
@@ -1319,6 +1444,7 @@ public class PdfGraphics2D extends Graphics2D {
                 com.lowagie.text.Image image = com.lowagie.text.Image.getInstance(img, null);
                 PdfPatternPainter pattern = cb.createPattern(image.width(), image.height());
                 AffineTransform inverse = this.normalizeMatrix();
+                inverse.translate(rect.getX(), rect.getY());
                 inverse.scale(rect.getWidth() / image.width(), -rect.getHeight() / image.height());
                 double[] mx = new double[6];
                 inverse.getMatrix(mx);
@@ -1357,6 +1483,8 @@ public class PdfGraphics2D extends Graphics2D {
                     tx.translate(-xoffset,-yoffset);
                     g.drawImage(img,tx,null);
                 }
+                g.dispose();
+                g = null;
                 com.lowagie.text.Image image = com.lowagie.text.Image.getInstance(img, null);
                 PdfPatternPainter pattern = cb.createPattern(width, height);
                 image.setAbsolutePosition(0,0);
@@ -1386,194 +1514,7 @@ public class PdfGraphics2D extends Graphics2D {
         }
         mediaTracker.removeImage(image);
     }
-    
-    ///////////////////////////////////////////////
-    //
-    //
-    //		PdfFontMetrics class
-    //
-    //
-    
-    
-    class PdfFontMetrics extends FontMetrics {
-        private BaseFont bf;
-        private float fontSize;
-        private double ascent = -1;
-        private double descent = -1;
-        private double leading = -1;
-        private double maxAdvance = -1;
-        private int widths[];
-        private double scaleX;
-        private double scaleY;
         
-        private PdfFontMetrics(Font f, BaseFont bf) {
-            super(f);
-            this.bf = bf;
-            this.fontSize = f.getSize2D();
-            AffineTransform af = f.getTransform();
-            scaleX = af.getScaleX();
-            scaleY = af.getScaleY();
-        }
-        
-        public int getAscent() {
-            if (ascent < 0)
-                ascent = bf.getFontDescriptor(BaseFont.AWT_ASCENT, fontSize) * scaleY;
-            return (int)(0.9999 + ascent * 1.1);
-        }
-        
-        public int getDescent() {
-            if (descent < 0)
-                descent =  -bf.getFontDescriptor(BaseFont.AWT_DESCENT, fontSize) * scaleY;
-            return (int)(0.9999 + descent);
-        }
-        
-        public int getLeading() {
-            if (leading < 0)
-                leading = bf.getFontDescriptor(BaseFont.AWT_LEADING, fontSize) * scaleY;
-            return (int)(0.9999 + leading);
-        }
-        
-        public int getMaxAdvance() {
-            if (maxAdvance < 0)
-                maxAdvance = bf.getFontDescriptor(BaseFont.AWT_MAXADVANCE, fontSize) * scaleX;
-            return (int)(0.9999 + maxAdvance);
-        }
-        
-        public int[] getWidths() {
-            if (widths == null) {
-                widths = new int[256];
-                for (char ch = 0 ; ch < 256 ; ch++) {
-                    widths[ch] = charWidth(ch);
-                }
-            }
-            return widths;
-        }
-        
-        public int charWidth(char c) {
-            return (int)(bf.getWidthPoint(c, fontSize) * scaleX);
-        }
-        
-        public int stringWidth(String s) {
-            return (int)(bf.getWidthPoint(s, fontSize) * scaleX);
-        }
-        /**
-         * Returns the bounds of the specified <code>String</code> in the
-         * specified <code>Graphics</code> context.  The bounds is used
-         * to layout the <code>String</code>.
-         * @param str the specified <code>String</code>
-         * @param context the specified <code>Graphics</code> context
-         * @return a {@link Rectangle2D} that is the bounding box of the
-         * specified <code>String</code> in the specified
-         * <code>Graphics</code> context.
-         * @see java.awt.Font#getStringBounds(String, FontRenderContext)
-         */
-        public Rectangle2D getStringBounds( String str, Graphics context) {
-            char[] array = str.toCharArray();
-            return getStringBounds(array, 0, array.length, context);
-        }
-        
-        /**
-         * Returns the bounds of the specified <code>String</code> in the
-         * specified <code>Graphics</code> context.  The bounds is used
-         * to layout the <code>String</code>.
-         * @param str the specified <code>String</code>
-         * @param beginIndex the offset of the beginning of <code>str</code>
-         * @param limit the length of <code>str</code>
-         * @param context the specified <code>Graphics</code> context
-         * @return a <code>Rectangle2D</code> that is the bounding box of the
-         * specified <code>String</code> in the specified
-         * <code>Graphics</code> context.
-         * @see java.awt.Font#getStringBounds(String, int, int, FontRenderContext)
-         */
-        public Rectangle2D getStringBounds( String str, int beginIndex, int limit,
-            Graphics context) {
-            String substr = str.substring(beginIndex, limit);
-            return getStringBounds(substr, context);
-        }
-        
-        /**
-         * Returns the bounds of the specified array of characters
-         * in the specified <code>Graphics</code> context.
-         * The bounds is used to layout the <code>String</code>
-         * created with the specified array of characters,
-         * <code>beginIndex</code> and <code>limit</code>.
-         * @param chars an array of characters
-         * @param beginIndex the initial offset of the array of
-         * characters
-         * @param limit the length of the array of characters
-         * @param context the specified <code>Graphics</code> context
-         * @return a <code>Rectangle2D</code> that is the bounding box of the
-         * specified character array in the specified
-         * <code>Graphics</code> context.
-         * @see java.awt.Font#getStringBounds(char[], int, int, FontRenderContext)
-         */
-        
-        public Rectangle2D getStringBounds(char chars[], int beginIndex, int limit, Graphics context) {
-            
-            if (beginIndex < 0) {
-                throw new IndexOutOfBoundsException("beginIndex: " + beginIndex);
-            }
-            if (limit > chars.length) {
-                throw new IndexOutOfBoundsException("limit: " + limit);
-            }
-            if (beginIndex > limit) {
-                throw new IndexOutOfBoundsException("range length: " + (limit - beginIndex));
-            }
-            String str = new String(chars, beginIndex, limit - beginIndex);
-            return new Rectangle2D.Float(0, -getAscent(), (float)(bf.getWidthPoint(str, fontSize) * scaleX), getHeight());
-        }
-        
-        /**
-         * Returns the bounds of the characters indexed in the specified
-         * <code>CharacterIterator</code> in the
-         * specified <code>Graphics</code> context.
-         * @param ci the specified <code>CharacterIterator</code>
-         * @param beginIndex the initial offset in <code>ci</code>
-         * @param limit the end index of <code>ci</code>
-         * @param context the specified <code>Graphics</code> context
-         * @return a <code>Rectangle2D</code> that is the bounding box of the
-         * characters indexed in the specified <code>CharacterIterator</code>
-         * in the specified <code>Graphics</code> context.
-         * @see java.awt.Font#getStringBounds(CharacterIterator, int, int, FontRenderContext)
-         */
-        public Rectangle2D getStringBounds(CharacterIterator ci, int beginIndex, int limit, Graphics context) {
-            int start = ci.getBeginIndex();
-            int end = ci.getEndIndex();
-            
-            if (beginIndex < start) {
-                throw new IndexOutOfBoundsException("beginIndex: " + beginIndex);
-            }
-            if (limit > end) {
-                throw new IndexOutOfBoundsException("limit: " + limit);
-            }
-            if (beginIndex > limit) {
-                throw new IndexOutOfBoundsException("range length: " + (limit - beginIndex));
-            }
-            
-            char[]  arr = new char[limit - beginIndex];
-            
-            ci.setIndex(beginIndex);
-            for(int idx = 0; idx < arr.length; idx++) {
-                arr[idx] = ci.current();
-                ci.next();
-            }
-            
-            return getStringBounds(arr,0,arr.length,context);
-        }
-        
-        /**
-         * Returns the bounds for the character with the maximum bounds
-         * in the specified <code>Graphics</code> context.
-         * @param context the specified <code>Graphics</code> context
-         * @return a <code>Rectangle2D</code> that is the
-         * bounding box for the character with the maximum bounds.
-         * @see java.awt.Font#getMaxCharBounds(FontRenderContext)
-         */
-        public Rectangle2D getMaxCharBounds(Graphics context) {
-            return getStringBounds("M", context);
-        }
-    }
-    
     static private class fakeComponent extends Component {
     }
 }
