@@ -1,8 +1,8 @@
 /*
- * $Id: PdfName.java,v 1.52 2004/02/07 10:28:43 blowagie Exp $
+ * $Id: PdfName.java,v 1.83 2006/06/04 22:23:38 psoares33 Exp $
  * $Name:  $
  *
- * Copyright 1999, 2000, 2001, 2002 Bruno Lowagie
+ * Copyright 1999-2006 Bruno Lowagie
  *
  * The contents of this file are subject to the Mozilla Public License Version 1.1
  * (the "License"); you may not use this file except in compliance with the License.
@@ -84,8 +84,20 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name */
     public static final PdfName ACTION = new PdfName("Action");
     /** A name */
+    public static final PdfName ADBE_PKCS7_DETACHED = new PdfName("adbe.pkcs7.detached");
+    /** A name */
+    public static final PdfName ADBE_PKCS7_SHA1 = new PdfName("adbe.pkcs7.sha1");
+    /** A name */
+    public static final PdfName ADBE_X509_RSA_SHA1 = new PdfName("adbe.x509.rsa_sha1");
+    /** A name */
+    public static final PdfName ADOBE_PPKLITE = new PdfName("Adobe.PPKLite");
+    /** A name */
+    public static final PdfName ADOBE_PPKMS = new PdfName("Adobe.PPKMS");
+    /** A name */
     public static final PdfName AIS = new PdfName("AIS");
     /** A name */
+    public static final PdfName ALLPAGES = new PdfName("AllPages");
+    /** A name */
     public static final PdfName ALTERNATE = new PdfName("Alternate");
     /** A name */
     public static final PdfName ANNOT = new PdfName("Annot");
@@ -96,6 +108,8 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name */
     public static final PdfName AP = new PdfName("AP");
     /** A name */
+    public static final PdfName ARTBOX = new PdfName("ArtBox");
+    /** A name */
     public static final PdfName ASCENT = new PdfName("Ascent");
     /** A name */
     public static final PdfName AS = new PdfName("AS");
@@ -118,6 +132,8 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name */
     public static final PdfName BG = new PdfName("BG");
     /** A name */
+    public static final PdfName BIGFIVE = new PdfName("BigFive");
+    /** A name */
     public static final PdfName BITSPERCOMPONENT = new PdfName("BitsPerComponent");
     /** A name */
     public static final PdfName BITSPERSAMPLE = new PdfName("BitsPerSample");
@@ -128,6 +144,8 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name */
     public static final PdfName BLACKPOINT = new PdfName("BlackPoint");
     /** A name */
+    public static final PdfName BLEEDBOX = new PdfName("BleedBox");
+    /** A name */
     public static final PdfName BLINDS = new PdfName("Blinds");
     /** A name */
     public static final PdfName BM = new PdfName("BM");
@@ -162,12 +180,18 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name */
     public static final PdfName CATALOG = new PdfName("Catalog");
     /** A name */
+    public static final PdfName CATEGORY = new PdfName("Category");
+    /** A name */
     public static final PdfName CCITTFAXDECODE = new PdfName("CCITTFaxDecode");
     /** A name */
     public static final PdfName CENTERWINDOW = new PdfName("CenterWindow");
     /** A name */
+    public static final PdfName CERT = new PdfName("Cert");
+    /** A name */
     public static final PdfName CH = new PdfName("Ch");
     /** A name */
+    public static final PdfName CHARPROCS = new PdfName("CharProcs");
+    /** A name */
     public static final PdfName CIDFONTTYPE0 = new PdfName("CIDFontType0");
     /** A name */
     public static final PdfName CIDFONTTYPE2 = new PdfName("CIDFontType2");
@@ -186,6 +210,8 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name */
     public static final PdfName COLUMNS = new PdfName("Columns");
     /** A name */
+    public static final PdfName CONTACTINFO = new PdfName("ContactInfo");
+    /** A name */
     public static final PdfName CONTENT = new PdfName("Content");
     /** A name */
     public static final PdfName CONTENTS = new PdfName("Contents");
@@ -206,6 +232,8 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name */
     public static final PdfName CREATOR = new PdfName("Creator");
     /** A name */
+    public static final PdfName CREATORINFO = new PdfName("CreatorInfo");
+    /** A name */
     public static final PdfName CROPBOX = new PdfName("CropBox");
     /** A name */
     public static final PdfName CS = new PdfName("CS");
@@ -218,16 +246,26 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name */
     public static final PdfName DCTDECODE = new PdfName("DCTDecode");
     /** A name */
-    public static final PdfName DESCENDANTFONTS = new PdfName("DescendantFonts");
-    /** A name */
-    public static final PdfName DESCENT = new PdfName("Descent");
-    /** A name */
     public static final PdfName DECODE = new PdfName("Decode");
     /** A name */
     public static final PdfName DECODEPARMS = new PdfName("DecodeParms");
     /** A name */
+    public static final PdfName DEFAULTCMYK = new PdfName("DefaultCMYK");
+    /** A name */
+    public static final PdfName DEFAULTGRAY = new PdfName("DefaultGray");
+    /** A name */
+    public static final PdfName DEFAULTRGB = new PdfName("DefaultRGB");
+    /** A name */
+    public static final PdfName DESC = new PdfName("Desc");
+    /** A name */
+    public static final PdfName DESCENDANTFONTS = new PdfName("DescendantFonts");
+    /** A name */
+    public static final PdfName DESCENT = new PdfName("Descent");
+    /** A name */
     public static final PdfName DEST = new PdfName("Dest");
     /** A name */
+    public static final PdfName DESTOUTPUTPROFILE = new PdfName("DestOutputProfile");
+    /** A name */
     public static final PdfName DESTS = new PdfName("Dests");
     /** A name */
     public static final PdfName DEVICEGRAY = new PdfName("DeviceGray");
@@ -244,6 +282,8 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name */
     public static final PdfName DIRECTION = new PdfName("Direction");
     /** A name */
+    public static final PdfName DISPLAYDOCTITLE = new PdfName("DisplayDocTitle");
+    /** A name */
     public static final PdfName DM = new PdfName("Dm");
     /** A name */
     public static final PdfName DOMAIN = new PdfName("Domain");
@@ -268,6 +308,8 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name */
     public static final PdfName EMBEDDEDFILE = new PdfName("EmbeddedFile");
     /** A name */
+    public static final PdfName EMBEDDEDFILES = new PdfName("EmbeddedFiles");
+    /** A name */
     public static final PdfName ENCODE = new PdfName("Encode");
     /** A name */
     public static final PdfName ENCODEDBYTEALIGN = new PdfName("EncodedByteAlign");
@@ -284,8 +326,16 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name */
     public static final PdfName EXTGSTATE = new PdfName("ExtGState");
     /** A name */
+    public static final PdfName EXPORT = new PdfName("Export");
+    /** A name */
+    public static final PdfName EXPORTSTATE = new PdfName("ExportState");
+    /** A name */
+    public static final PdfName EVENT = new PdfName("Event");
+    /** A name */
     public static final PdfName F = new PdfName("F");
     /** A name */
+    public static final PdfName FB = new PdfName("FB");
+    /** A name */
     public static final PdfName FDECODEPARMS = new PdfName("FDecodeParms");
     /** A name */
     public static final PdfName FDF = new PdfName("FDF");
@@ -342,6 +392,8 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name */
     public static final PdfName FONTFILE3 = new PdfName("FontFile3");
     /** A name */
+    public static final PdfName FONTMATRIX = new PdfName("FontMatrix");
+    /** A name */
     public static final PdfName FONTNAME = new PdfName("FontName");
     /** A name */
     public static final PdfName FORM = new PdfName("Form");
@@ -350,6 +402,8 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name */
     public static final PdfName FREETEXT = new PdfName("FreeText");
     /** A name */
+    public static final PdfName FRM = new PdfName("FRM");
+    /** A name */
     public static final PdfName FS = new PdfName("FS");
     /** A name */
     public static final PdfName FT = new PdfName("FT");
@@ -364,6 +418,8 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name of an attribute. */
     public static final PdfName GAMMA = new PdfName("Gamma");
     /** A name of an attribute. */
+    public static final PdfName GBK = new PdfName("GBK");
+    /** A name of an attribute. */
     public static final PdfName GLITTER = new PdfName("Glitter");
     /** A name of an attribute. */
     public static final PdfName GOTO = new PdfName("GoTo");
@@ -372,6 +428,10 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name of an attribute. */
     public static final PdfName GROUP = new PdfName("Group");
     /** A name of an attribute. */
+    public static final PdfName GTS_PDFX = new PdfName("GTS_PDFX");
+    /** A name of an attribute. */
+    public static final PdfName GTS_PDFXVERSION = new PdfName("GTS_PDFXVersion");
+    /** A name of an attribute. */
     public static final PdfName H = new PdfName("H");
     /** A name of an attribute. */
     public static final PdfName HEIGHT = new PdfName("Height");
@@ -416,6 +476,8 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name */
     public static final PdfName IMAGEMASK = new PdfName("ImageMask");
     /** A name */
+    public static final PdfName INDEX = new PdfName("Index");
+    /** A name */
     public static final PdfName INDEXED = new PdfName("Indexed");
     /** A name */
     public static final PdfName INFO = new PdfName("Info");
@@ -432,6 +494,8 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name */
     public static final PdfName ISMAP = new PdfName("IsMap");
     /** A name */
+    public static final PdfName IRT = new PdfName("IRT");
+    /** A name */
     public static final PdfName ITALICANGLE = new PdfName("ItalicAngle");
     /** A name */
     public static final PdfName IX = new PdfName("IX");
@@ -450,6 +514,10 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name */
     public static final PdfName L2R = new PdfName("L2R");
     /** A name */
+    public static final PdfName LANG = new PdfName("Lang");
+    /** A name */
+    public static final PdfName LANGUAGE = new PdfName("Language");
+    /** A name */
     public static final PdfName LAST = new PdfName("Last");
     /** A name */
     public static final PdfName LASTCHAR = new PdfName("LastChar");
@@ -468,6 +536,8 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name */
     public static final PdfName LINK = new PdfName("Link");
     /** A name */
+    public static final PdfName LISTMODE = new PdfName("ListMode");
+    /** A name */
     public static final PdfName LOCATION = new PdfName("Location");
     /** A name */
     public static final PdfName LOCK = new PdfName("Lock");
@@ -482,14 +552,26 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name of an encoding */
     public static final PdfName MAC_ROMAN_ENCODING = new PdfName("MacRomanEncoding");
     /** A name */
+    public static final PdfName MARKED = new PdfName("Marked");
+    /** A name */
+    public static final PdfName MARKINFO = new PdfName("MarkInfo");
+    /** A name */
     public static final PdfName MASK = new PdfName("Mask");
     /** A name */
+    public static final PdfName MAX = new PdfName("max");
+    /** A name */
     public static final PdfName MAXLEN = new PdfName("MaxLen");
     /** A name */
     public static final PdfName MEDIABOX = new PdfName("MediaBox");
     /** A name */
+    public static final PdfName MCID = new PdfName("MCID");
+    /** A name */
+    public static final PdfName MCR = new PdfName("MCR");
+    /** A name */
     public static final PdfName METADATA = new PdfName("Metadata");
     /** A name */
+    public static final PdfName MIN = new PdfName("min");
+    /** A name */
     public static final PdfName MK = new PdfName("MK");
     /** A name */
     public static final PdfName MMTYPE1 = new PdfName("MMType1");
@@ -498,6 +580,16 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name */
     public static final PdfName N = new PdfName("N");
     /** A name */
+    public static final PdfName N0 = new PdfName("n0");
+    /** A name */
+    public static final PdfName N1 = new PdfName("n1");
+    /** A name */
+    public static final PdfName N2 = new PdfName("n2");
+    /** A name */
+    public static final PdfName N3 = new PdfName("n3");
+    /** A name */
+    public static final PdfName N4 = new PdfName("n4");
+    /** A name */
     public static final PdfName NAME = new PdfName("Name");
     /** A name */
     public static final PdfName NAMED = new PdfName("Named");
@@ -514,15 +606,31 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name */
     public static final PdfName NM = new PdfName("NM");
     /** A name */
+    public static final PdfName NONE = new PdfName("None");
+    /** A name */
     public static final PdfName NONFULLSCREENPAGEMODE = new PdfName("NonFullScreenPageMode");
     /** A name */
     public static final PdfName NUMS = new PdfName("Nums");
     /** A name */
     public static final PdfName O = new PdfName("O");
     /** A name */
+    public static final PdfName OBJSTM = new PdfName("ObjStm");
+    /** A name */
     public static final PdfName OC = new PdfName("OC");
     /** A name */
-    public static final PdfName OFF = new PdfName("Off");
+    public static final PdfName OCG = new PdfName("OCG");
+    /** A name */
+    public static final PdfName OCGS = new PdfName("OCGs");
+    /** A name */
+    public static final PdfName OCMD = new PdfName("OCMD");
+    /** A name */
+    public static final PdfName OCPROPERTIES = new PdfName("OCProperties");
+    /** A name */
+    public static final PdfName Off = new PdfName("Off");
+    /** A name */
+    public static final PdfName OFF = new PdfName("OFF");
+    /** A name */
+    public static final PdfName ON = new PdfName("ON");
     /** A name */
     public static final PdfName ONECOLUMN = new PdfName("OneColumn");
     /** A name */
@@ -544,6 +652,14 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name */
     public static final PdfName OUTLINES = new PdfName("Outlines");
     /** A name */
+    public static final PdfName OUTPUTCONDITION = new PdfName("OutputCondition");
+    /** A name */
+    public static final PdfName OUTPUTCONDITIONIDENTIFIER = new PdfName("OutputConditionIdentifier");
+    /** A name */
+    public static final PdfName OUTPUTINTENT = new PdfName("OutputIntent");
+    /** A name */
+    public static final PdfName OUTPUTINTENTS = new PdfName("OutputIntents");
+    /** A name */
     public static final PdfName P = new PdfName("P");
     /** A name */
     public static final PdfName PAGE = new PdfName("Page");
@@ -560,8 +676,12 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name */
     public static final PdfName PANOSE = new PdfName("Panose");
     /** A name */
+    public static final PdfName PARAMS = new PdfName("Params");
+    /** A name */
     public static final PdfName PARENT = new PdfName("Parent");
     /** A name */
+    public static final PdfName PARENTTREE = new PdfName("ParentTree");
+    /** A name */
     public static final PdfName PATTERN = new PdfName("Pattern");
     /** A name */
     public static final PdfName PATTERNTYPE = new PdfName("PatternType");
@@ -570,20 +690,34 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name */
     public static final PdfName PERCEPTUAL = new PdfName("Perceptual");
     /** A name */
+    public static final PdfName PG = new PdfName("Pg");
+    /** A name */
     public static final PdfName POPUP = new PdfName("Popup");
     /** A name */
     public static final PdfName PREDICTOR = new PdfName("Predictor");
     /** A name */
+    public static final PdfName PREFERRED = new PdfName("Preferred");
+    /** A name */
+    public static final PdfName PRESERVERB = new PdfName("PreserveRB");
+    /** A name */
     public static final PdfName PREV = new PdfName("Prev");
     /** A name */
     public static final PdfName PREVPAGE = new PdfName("PrevPage");
     /** A name */
+    public static final PdfName PRINT = new PdfName("Print");
+    /** A name */
+    public static final PdfName PRINTSCALING = new PdfName("PrintScaling");
+    /** A name */
+    public static final PdfName PRINTSTATE = new PdfName("PrintState");
+    /** A name */
     public static final PdfName PROCSET = new PdfName("ProcSet");
     /** A name */
     public static final PdfName PRODUCER = new PdfName("Producer");
     /** A name */
     public static final PdfName PROPERTIES = new PdfName("Properties");
     /** A name */
+    public static final PdfName PS = new PdfName("PS");
+    /** A name */
     public static final PdfName Q = new PdfName("Q");
     /** A name */
     public static final PdfName QUADPOINTS = new PdfName("QuadPoints");
@@ -596,12 +730,16 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name */
     public static final PdfName RC = new PdfName("RC");
     /** A name */
+    public static final PdfName RBGROUPS = new PdfName("RBGroups");
+    /** A name */
     public static final PdfName REASON = new PdfName("Reason");
     /** A name */
     public static final PdfName RECT = new PdfName("Rect");
     /** A name */
     public static final PdfName REGISTRY = new PdfName("Registry");
     /** A name */
+    public static final PdfName REGISTRYNAME = new PdfName("RegistryName");
+    /** A name */
     public static final PdfName RELATIVECALORIMETRIC = new PdfName("RelativeColorimetric");
     /** A name */
     public static final PdfName RENDITION = new PdfName("Rendition");
@@ -612,6 +750,8 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name */
     public static final PdfName RI = new PdfName("RI");
     /** A name */
+    public static final PdfName ROLEMAP = new PdfName("RoleMap");
+    /** A name */
     public static final PdfName ROOT = new PdfName("Root");
     /** A name */
     public static final PdfName ROTATE = new PdfName("Rotate");
@@ -630,10 +770,14 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name */
     public static final PdfName SEPARATION = new PdfName("Separation");
     /** A name */
+    public static final PdfName SETOCGSTATE = new PdfName("SetOCGState");
+    /** A name */
     public static final PdfName SHADING = new PdfName("Shading");
     /** A name */
     public static final PdfName SHADINGTYPE = new PdfName("ShadingType");
     /** A name */
+    public static final PdfName SHIFT_JIS = new PdfName("Shift-JIS");
+    /** A name */
     public static final PdfName SIG = new PdfName("Sig");
     /** A name */
     public static final PdfName SIGFLAGS = new PdfName("SigFlags");
@@ -654,10 +798,16 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name */
     public static final PdfName STANDARD = new PdfName("Standard");
     /** A name */
+    public static final PdfName STATE = new PdfName("State");
+    /** A name */
     public static final PdfName STRIKEOUT = new PdfName("StrikeOut");
     /** A name */
     public static final PdfName STRUCTPARENT = new PdfName("StructParent");
     /** A name */
+    public static final PdfName STRUCTPARENTS = new PdfName("StructParents");
+    /** A name */
+    public static final PdfName STRUCTTREEROOT = new PdfName("StructTreeRoot");
+    /** A name */
     public static final PdfName STYLE = new PdfName("Style");
     /** A name */
     public static final PdfName STEMV = new PdfName("StemV");
@@ -704,6 +854,8 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name */
     public static final PdfName TM = new PdfName("TM");
     /** A name */
+    public static final PdfName TOGGLE = new PdfName("Toggle");
+    /** A name */
     public static final PdfName TOUNICODE = new PdfName("ToUnicode");
     /** A name */
     public static final PdfName TP = new PdfName("TP");
@@ -714,6 +866,8 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name */
     public static final PdfName TRAPPED = new PdfName("Trapped");
     /** A name */
+    public static final PdfName TRIMBOX = new PdfName("TrimBox");
+    /** A name */
     public static final PdfName TRUETYPE = new PdfName("TrueType");
     /** A name */
     public static final PdfName TU = new PdfName("TU");
@@ -722,6 +876,10 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name */
     public static final PdfName TWOCOLUMNRIGHT = new PdfName("TwoColumnRight");
     /** A name */
+    public static final PdfName TWOPAGELEFT = new PdfName("TwoPageLeft");
+    /** A name */
+    public static final PdfName TWOPAGERIGHT = new PdfName("TwoPageRight");
+    /** A name */
     public static final PdfName TX = new PdfName("Tx");
     /** A name */
     public static final PdfName TYPE = new PdfName("Type");
@@ -730,28 +888,52 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name */
     public static final PdfName TYPE1 = new PdfName("Type1");
     /** A name of an attribute. */
+    public static final PdfName TYPE3 = new PdfName("Type3");
+    /** A name of an attribute. */
     public static final PdfName U = new PdfName("U");
     /** A name of an attribute. */
+    public static final PdfName UHC = new PdfName("UHC");
+    /** A name of an attribute. */
     public static final PdfName UNDERLINE = new PdfName("Underline");
     /** A name */
     public static final PdfName URI = new PdfName("URI");
     /** A name */
     public static final PdfName URL = new PdfName("URL");
     /** A name */
+    public static final PdfName USAGE = new PdfName("Usage");
+    /** A name */
+    public static final PdfName USEATTACHMENTS = new PdfName("UseAttachments");
+    /** A name */
     public static final PdfName USENONE = new PdfName("UseNone");
     /** A name */
+    public static final PdfName USEOC = new PdfName("UseOC");
+    /** A name */
     public static final PdfName USEOUTLINES = new PdfName("UseOutlines");
     /** A name */
+    public static final PdfName USER = new PdfName("User");
+    /** A name */
+    public static final PdfName USERUNIT = new PdfName("UserUnit");
+    /** A name */
     public static final PdfName USETHUMBS = new PdfName("UseThumbs");
     /** A name */
     public static final PdfName V = new PdfName("V");
     /** A name */
+    public static final PdfName VERISIGN_PPKVS = new PdfName("VeriSign.PPKVS");
+    /** A name */
+    public static final PdfName VIEW = new PdfName("View");
+    /** A name */
     public static final PdfName VIEWERPREFERENCES = new PdfName("ViewerPreferences");
+    /** A name */
+    public static final PdfName VIEWSTATE = new PdfName("ViewState");
+    /** A name */
+    public static final PdfName VISIBLEPAGES = new PdfName("VisiblePages");
     /** A name of an attribute. */
     public static final PdfName W = new PdfName("W");
     /** A name of an attribute. */
     public static final PdfName W2 = new PdfName("W2");
     /** A name of an attribute. */
+    public static final PdfName WC = new PdfName("WC");
+    /** A name of an attribute. */
     public static final PdfName WIDGET = new PdfName("Widget");
     /** A name of an attribute. */
     public static final PdfName WIDTH = new PdfName("Width");
@@ -772,32 +954,52 @@ public class PdfName extends PdfObject implements Comparable{
     /** A name */
     public static final PdfName X = new PdfName("X");
     /** A name */
+    public static final PdfName XFA = new PdfName("XFA");
+    /** A name */
+    public static final PdfName XML = new PdfName("XML");
+    /** A name */
     public static final PdfName XOBJECT = new PdfName("XObject");
     /** A name */
     public static final PdfName XSTEP = new PdfName("XStep");
     /** A name */
+    public static final PdfName XREF = new PdfName("XRef");
+    /** A name */
+    public static final PdfName XREFSTM = new PdfName("XRefStm");
+    /** A name */
     public static final PdfName XYZ = new PdfName("XYZ");
     /** A name */
     public static final PdfName YSTEP = new PdfName("YStep");
     /** A name of a base 14 type 1 font */
     public static final PdfName ZAPFDINGBATS = new PdfName("ZapfDingbats");
+    /** A name */
+    public static final PdfName ZOOM = new PdfName("Zoom");
     
     private int hash = 0;
     
     // constructors
     
+
     /**
-     * Constructs a <CODE>PdfName</CODE>-object.
-     *
-     * @param		name		the new Name.
+     * Constructs a new <CODE>PdfName</CODE>. The name length will be checked.
+     * @param name the new name
+     */
+    public PdfName(String name) {
+        this(name, true);
+    }
+    
+    /**
+     * Constructs a new <CODE>PdfName</CODE>.
+     * @param name the new name
+     * @param lengthCheck if <CODE>true</CODE> check the lenght validity, if <CODE>false</CODE> the name can
+     * have any length
      */
     
-    public PdfName(String name) {
+    public PdfName(String name, boolean lengthCheck) {
         super(PdfObject.NAME);
         // The minimum number of characters in a name is 0, the maximum is 127 (the '/' not included)
         int length = name.length();
-        if (length > 127) {
-            throw new IllegalArgumentException("The name is too long (" + bytes.length + " characters).");
+        if (lengthCheck && length > 127) {
+            throw new IllegalArgumentException("The name '" + name + "' is too long (" + length + " characters).");
         }
         // The name has to be checked for illegal characters
         // every special character has to be substituted
@@ -840,6 +1042,10 @@ public class PdfName extends PdfObject implements Comparable{
         bytes = pdfName.toByteArray();
     }
     
+    /**
+     * Constructs a PdfName.
+     * @param bytes the byte representation of the name
+     */
     public PdfName(byte bytes[]) {
         super(PdfObject.NAME, bytes);
     }
@@ -849,12 +1055,9 @@ public class PdfName extends PdfObject implements Comparable{
      * Compares this object with the specified object for order.  Returns a
      * negative integer, zero, or a positive integer as this object is less
      * than, equal to, or greater than the specified object.<p>
-     *
-     *
-     * @param   object the Object to be compared.
-     * @return  a negative integer, zero, or a positive integer as this object
-     *		is less than, equal to, or greater than the specified object.
-     *
+     * @param object the Object to be compared.
+     * @return a negative integer, zero, or a positive integer as this object
+     * 		is less than, equal to, or greater than the specified object.
      * @throws ClassCastException if the specified object's type prevents it
      *         from being compared to this Object.
      */
