@@ -1,5 +1,5 @@
 /*
- * $Id: RandomAccessFileOrArray.java,v 1.28 2004/02/07 10:28:46 blowagie Exp $
+ * $Id: RandomAccessFileOrArray.java,v 1.50 2006/09/13 15:13:43 psoares33 Exp $
  * $Name:  $
  *
  * Copyright 2001, 2002 Paulo Soares
@@ -52,9 +52,9 @@ package com.lowagie.text.pdf;
 
 import java.io.DataInputStream;
 import java.io.DataInput;
+import java.io.FileInputStream;
 import java.io.IOException;
 import java.io.EOFException;
-import java.io.RandomAccessFile;
 import java.io.File;
 import java.io.InputStream;
 import java.io.ByteArrayOutputStream;
@@ -66,7 +66,7 @@ import java.net.URL;
  */
 public class RandomAccessFileOrArray implements DataInput {
     
-    RandomAccessFile rf;
+    MappedRandomAccessFile rf;
     String filename;
     byte arrayIn[];
     int arrayInPtr;
@@ -75,62 +75,78 @@ public class RandomAccessFileOrArray implements DataInput {
     
     /** Holds value of property startOffset. */
     private int startOffset = 0;
-    
+
     public RandomAccessFileOrArray(String filename) throws IOException {
+    	this(filename, false);
+    }
+    
+    public RandomAccessFileOrArray(String filename, boolean forceRead) throws IOException {
         File file = new File(filename);
         if (!file.canRead()) {
-            InputStream is = BaseFont.getResourceStream(filename);
-            if (is == null)
-                throw new IOException(filename + " not found as file or resource.");
-            try {
-                byte b[] = new byte[4096];
-                ByteArrayOutputStream out = new ByteArrayOutputStream();
-                for (;;) {
-                    int read = is.read(b);
-                    if (read < 1)
-                        break;
-                    out.write(b, 0, read);
-                }
-                this.arrayIn = out.toByteArray();
-                return;
-            }
-            finally {
+            if (filename.startsWith("file:/") || filename.startsWith("http://") || filename.startsWith("https://") || filename.startsWith("jar:")) {
+                InputStream is = new URL(filename).openStream();
                 try {
-                    is.close();
+                    this.arrayIn = InputStreamToArray(is);
+                    return;
                 }
-                catch (IOException ioe) {
-                    // empty on purpose
+                finally {
+                    try {is.close();}catch(IOException ioe){}
+                }
+            }
+            else {
+                InputStream is = BaseFont.getResourceStream(filename);
+                if (is == null)
+                    throw new IOException(filename + " not found as file or resource.");
+                try {
+                    this.arrayIn = InputStreamToArray(is);
+                    return;
+                }
+                finally {
+                    try {is.close();}catch(IOException ioe){}
                 }
             }
         }
+        else if (forceRead) {
+            InputStream s = null;
+            try {
+                s = new FileInputStream(file);
+                this.arrayIn = InputStreamToArray(s);
+            }
+            finally {
+                try {s.close();}catch(Exception e){}
+            }
+        	return;
+        }
         this.filename = filename;
-        rf = new RandomAccessFile(filename, "r");
+        rf = new MappedRandomAccessFile(filename, "r");
     }
 
     public RandomAccessFileOrArray(URL url) throws IOException {
         InputStream is = url.openStream();
         try {
-            byte b[] = new byte[4096];
-            ByteArrayOutputStream out = new ByteArrayOutputStream();
-            for (;;) {
-                int read = is.read(b);
-                if (read < 1)
-                    break;
-                out.write(b, 0, read);
-            }
-            this.arrayIn = out.toByteArray();
-            return;
+            this.arrayIn = InputStreamToArray(is);
         }
         finally {
-            try {
-                is.close();
-            }
-            catch (IOException ioe) {
-                // empty on purpose
-            }
+            try {is.close();}catch(IOException ioe){}
         }
     }
 
+    public RandomAccessFileOrArray(InputStream is) throws IOException {
+        this.arrayIn = InputStreamToArray(is);
+    }
+    
+    public static byte[] InputStreamToArray(InputStream is) throws IOException {
+        byte b[] = new byte[8192];
+        ByteArrayOutputStream out = new ByteArrayOutputStream();
+        while (true) {
+            int read = is.read(b);
+            if (read < 1)
+                break;
+            out.write(b, 0, read);
+        }
+        return out.toByteArray();
+    }
+
     public RandomAccessFileOrArray(byte arrayIn[]) {
         this.arrayIn = arrayIn;
     }
@@ -244,16 +260,9 @@ public class RandomAccessFileOrArray implements DataInput {
     }
     
     public void reOpen() throws IOException {
-        isBack = false;
-        if (filename != null) {
-            close();
-            rf = new RandomAccessFile(filename, "r");
-            if (startOffset != 0)
-                rf.seek(startOffset);
-        }
-        else {
-            arrayInPtr = startOffset;
-        }
+        if (filename != null && rf == null)
+            rf = new MappedRandomAccessFile(filename, "r");
+        seek(0);
     }
     
     protected void insureOpen() throws IOException {
@@ -262,6 +271,10 @@ public class RandomAccessFileOrArray implements DataInput {
         }
     }
     
+    public boolean isOpen() {
+        return (filename == null || rf != null);
+    }
+    
     public void close() throws IOException {
         isBack = false;
         if (rf != null) {
@@ -271,8 +284,10 @@ public class RandomAccessFileOrArray implements DataInput {
     }
     
     public int length() throws IOException {
-        if (arrayIn == null)
+        if (arrayIn == null) {
+            insureOpen();
             return (int)rf.length() - startOffset;
+        }
         else
             return arrayIn.length - startOffset;
     }
@@ -293,9 +308,11 @@ public class RandomAccessFileOrArray implements DataInput {
     }
     
     public int getFilePointer() throws IOException {
+        insureOpen();
         int n = isBack ? 1 : 0;
-        if (arrayIn == null)
+        if (arrayIn == null) {
             return (int)rf.getFilePointer() - n - startOffset;
+        }
         else
             return arrayInPtr - n - startOffset;
     }
