@@ -1,8 +1,8 @@
 /*
- * $Id: TrueTypeFont.java,v 1.46 2004/02/07 10:28:46 blowagie Exp $
+ * $Id: TrueTypeFont.java,v 1.59 2006/08/27 16:25:16 psoares33 Exp $
  * $Name:  $
  *
- * Copyright 2001, 2002 Paulo Soares
+ * Copyright 2001-2006 Paulo Soares
  *
  * The contents of this file are subject to the Mozilla Public License Version 1.1
  * (the "License"); you may not use this file except in compliance with the License.
@@ -53,6 +53,8 @@ package com.lowagie.text.pdf;
 import java.io.*;
 import java.util.HashMap;
 import java.util.ArrayList;
+import java.util.Iterator;
+import java.util.Map;
 import com.lowagie.text.DocumentException;
 import com.lowagie.text.ExceptionConverter;
 /** Reads a Truetype font
@@ -172,6 +174,8 @@ class TrueTypeFont extends BaseFont {
      * 'hmtx' normalized to 1000 units.
      */
     protected int GlyphWidths[];
+    
+    protected int bboxes[][];
     /** The map containing the code information for the table 'cmap', encoding 1.0.
      * The key is the code and the value is an <CODE>int[2]</CODE> where position 0
      * is the glyph number and position 1 is the glyph width normalized to 1000
@@ -186,13 +190,13 @@ class TrueTypeFont extends BaseFont {
      * units.
      */
     protected HashMap cmap31;
-    /** The map containig the kerning information. It represents the content of
+    /** The map containing the kerning information. It represents the content of
      * table 'kern'. The key is an <CODE>Integer</CODE> where the top 16 bits
-     * are the Unicode for the first character and the lower 16 bits are the
-     * Unicode for the second character. The value is the amount of kerning in
+     * are the glyph number for the first character and the lower 16 bits are the
+     * glyph number for the second character. The value is the amount of kerning in
      * normalized 1000 units as an <CODE>Integer</CODE>. This value is usually negative.
      */
-    protected HashMap kerning = new HashMap();
+    protected IntHashtable kerning = new IntHashtable();
     /**
      * The font name.
      * This name is usually extracted from the table 'name' with
@@ -221,7 +225,7 @@ class TrueTypeFont extends BaseFont {
     
     /** The components of table 'head'.
      */
-    protected class FontHeader {
+    protected static class FontHeader {
         /** A variable. */
         int flags;
         /** A variable. */
@@ -240,7 +244,7 @@ class TrueTypeFont extends BaseFont {
     
     /** The components of table 'hhea'.
      */
-    protected class HorizontalHeader {
+    protected static class HorizontalHeader {
         /** A variable. */
         short Ascender;
         /** A variable. */
@@ -265,7 +269,7 @@ class TrueTypeFont extends BaseFont {
     
     /** The components of table 'OS/2'.
      */
-    protected class WindowsMetrics {
+    protected static class WindowsMetrics {
         /** A variable. */
         short xAvgCharWidth;
         /** A variable. */
@@ -363,7 +367,8 @@ class TrueTypeFont extends BaseFont {
         }
         else
             throw new DocumentException(fileName + style + " is not a TTF, OTF or TTC font file.");
-        PdfEncodings.convertToBytes(" ", enc); // check if the encoding exists
+        if (!encoding.startsWith("#"))
+            PdfEncodings.convertToBytes(" ", enc); // check if the encoding exists
         createEncoding();
     }
     
@@ -446,6 +451,8 @@ class TrueTypeFont extends BaseFont {
         os_2.usLastCharIndex = rf.readUnsignedShort();
         os_2.sTypoAscender = rf.readShort();
         os_2.sTypoDescender = rf.readShort();
+        if (os_2.sTypoDescender > 0)
+            os_2.sTypoDescender = (short)(-os_2.sTypoDescender);
         os_2.sTypoLineGap = rf.readShort();
         os_2.usWinAscent = rf.readUnsignedShort();
         os_2.usWinDescent = rf.readUnsignedShort();
@@ -610,6 +617,8 @@ class TrueTypeFont extends BaseFont {
                 readGlyphWidths();
                 readCMaps();
                 readKerning();
+                readBbox();
+                GlyphWidths = null;
             }
         }
         finally {
@@ -676,12 +685,54 @@ class TrueTypeFont extends BaseFont {
      * @param glyph the glyph to get the width of
      * @return the width of the glyph in normalized 1000 units
      */
-    public int getGlyphWidth(int glyph) {
+    protected int getGlyphWidth(int glyph) {
         if (glyph >= GlyphWidths.length)
             glyph = GlyphWidths.length - 1;
         return GlyphWidths[glyph];
     }
     
+    private void readBbox() throws DocumentException, IOException {
+        int tableLocation[];
+        tableLocation = (int[])tables.get("head");
+        if (tableLocation == null)
+            throw new DocumentException("Table 'head' does not exist in " + fileName + style);
+        rf.seek(tableLocation[0] + TrueTypeFontSubSet.HEAD_LOCA_FORMAT_OFFSET);
+        boolean locaShortTable = (rf.readUnsignedShort() == 0);
+        tableLocation = (int[])tables.get("loca");
+        if (tableLocation == null)
+            return;
+        rf.seek(tableLocation[0]);
+        int locaTable[];
+        if (locaShortTable) {
+            int entries = tableLocation[1] / 2;
+            locaTable = new int[entries];
+            for (int k = 0; k < entries; ++k)
+                locaTable[k] = rf.readUnsignedShort() * 2;
+        }
+        else {
+            int entries = tableLocation[1] / 4;
+            locaTable = new int[entries];
+            for (int k = 0; k < entries; ++k)
+                locaTable[k] = rf.readInt();
+        }
+        tableLocation = (int[])tables.get("glyf");
+        if (tableLocation == null)
+            throw new DocumentException("Table 'glyf' does not exist in " + fileName + style);
+        int tableGlyphOffset = tableLocation[0];
+        bboxes = new int[locaTable.length - 1][];
+        for (int glyph = 0; glyph < locaTable.length - 1; ++glyph) {
+            int start = locaTable[glyph];
+            if (start != locaTable[glyph + 1]) {
+                rf.seek(tableGlyphOffset + start + 2);
+                bboxes[glyph] = new int[]{
+                    (rf.readShort() * 1000) / head.unitsPerEm,
+                    (rf.readShort() * 1000) / head.unitsPerEm,
+                    (rf.readShort() * 1000) / head.unitsPerEm,
+                    (rf.readShort() * 1000) / head.unitsPerEm};
+            }
+        }
+    }
+    
     /** Reads the several maps from the table 'cmap'. The maps of interest are 1.0 for symbolic
      *  fonts and 3.1 for all others. A symbolic font is defined as having the map 3.0.
      * @throws DocumentException the font is invalid
@@ -768,7 +819,6 @@ class TrueTypeFont extends BaseFont {
      * @throws IOException the font file could not be read
      */
     HashMap readFormat4() throws IOException {
-        int mask = (fontSpecific ? 0xff : 0xffff);
         HashMap h = new HashMap();
         int table_lenght = rf.readUnsignedShort();
         rf.skipBytes(2);
@@ -810,7 +860,7 @@ class TrueTypeFont extends BaseFont {
                 int r[] = new int[2];
                 r[0] = glyph;
                 r[1] = getGlyphWidth(r[0]);
-                h.put(new Integer(j & mask), r);
+                h.put(new Integer(fontSpecific ? ((j & 0xff00) == 0xf000 ? j & 0xff : j) : j), r);
             }
         }
         return h;
@@ -858,8 +908,8 @@ class TrueTypeFont extends BaseFont {
                 int nPairs = rf.readUnsignedShort();
                 rf.skipBytes(6);
                 for (int j = 0; j < nPairs; ++j) {
-                    Integer pair = new Integer(rf.readInt());
-                    Integer value = new Integer(((int)rf.readShort() * 1000) / head.unitsPerEm);
+                    int pair = rf.readInt();
+                    int value = ((int)rf.readShort() * 1000) / head.unitsPerEm;
                     kerning.put(pair, value);
                 }
             }
@@ -880,11 +930,7 @@ class TrueTypeFont extends BaseFont {
         if (metrics == null)
             return 0;
         int c2 = metrics[0];
-        Integer v = (Integer)kerning.get(new Integer((c1 << 16) + c2));
-        if (v == null)
-            return 0;
-        else
-            return v.intValue();
+        return kerning.get((c1 << 16) + c2);
     }
     
     /** Gets the width from the font according to the unicode char <CODE>c</CODE>.
@@ -895,7 +941,7 @@ class TrueTypeFont extends BaseFont {
      */
     int getRawWidth(int c, String name) {
         HashMap map = null;
-        if (name == null)
+        if (name == null || cmap31 == null)
             map = cmap10;
         else
             map = cmap31;
@@ -925,9 +971,9 @@ class TrueTypeFont extends BaseFont {
         (int)head.yMax * 1000 / head.unitsPerEm));
         if (cff) {
             if (encoding.startsWith("Identity-"))
-                dic.put(PdfName.FONTNAME, new PdfName(fontName+"-"+encoding));
+                dic.put(PdfName.FONTNAME, new PdfName(subsetPrefix + fontName+"-"+encoding));
             else
-                dic.put(PdfName.FONTNAME, new PdfName(fontName + style));
+                dic.put(PdfName.FONTNAME, new PdfName(subsetPrefix + fontName + style));
         }
         else
             dic.put(PdfName.FONTNAME, new PdfName(subsetPrefix + fontName + style));
@@ -1015,13 +1061,79 @@ class TrueTypeFont extends BaseFont {
         return dic;
     }
     
-    private byte[] getFullFont() throws IOException {
-        RandomAccessFileOrArray rf2 = new RandomAccessFileOrArray(rf);
-        rf2.reOpen();
-        byte b[] = new byte[rf2.length()];
-        rf2.readFully(b);
-        rf2.close();
-        return b;
+    protected byte[] getFullFont() throws IOException {
+        RandomAccessFileOrArray rf2 = null;
+        try {
+            rf2 = new RandomAccessFileOrArray(rf);
+            rf2.reOpen();
+            byte b[] = new byte[rf2.length()];
+            rf2.readFully(b);
+            return b;
+        } 
+        finally {
+            try {rf2.close();} catch (Exception e) {}
+        }
+    }
+    
+    protected static int[] compactRanges(ArrayList ranges) {
+        ArrayList simp = new ArrayList();
+        for (int k = 0; k < ranges.size(); ++k) {
+            int[] r = (int[])ranges.get(k);
+            for (int j = 0; j < r.length; j += 2) {
+                simp.add(new int[]{Math.max(0, Math.min(r[j], r[j + 1])), Math.min(0xffff, Math.max(r[j], r[j + 1]))});
+            }
+        }
+        for (int k1 = 0; k1 < simp.size() - 1; ++k1) {
+            for (int k2 = k1 + 1; k2 < simp.size(); ++k2) {
+                int[] r1 = (int[])simp.get(k1);
+                int[] r2 = (int[])simp.get(k2);
+                if ((r1[0] >= r2[0] && r1[0] <= r2[1]) || (r1[1] >= r2[0] && r1[0] <= r2[1])) {
+                    r1[0] = Math.min(r1[0], r2[0]);
+                    r1[1] = Math.max(r1[1], r2[1]);
+                    simp.remove(k2);
+                    --k2;
+                }
+            }
+        }
+        int[] s = new int[simp.size() * 2];
+        for (int k = 0; k < simp.size(); ++k) {
+            int[] r = (int[])simp.get(k);
+            s[k * 2] = r[0];
+            s[k * 2 + 1] = r[1];
+        }
+        return s;
+    }
+    
+    protected void addRangeUni(HashMap longTag, boolean includeMetrics, boolean subsetp) {
+        if (!subsetp && (subsetRanges != null || directoryOffset > 0)) {
+            int[] rg = (subsetRanges == null && directoryOffset > 0) ? new int[]{0, 0xffff} : compactRanges(subsetRanges);
+            HashMap usemap;
+            if (!fontSpecific && cmap31 != null) 
+                usemap = cmap31;
+            else if (fontSpecific && cmap10 != null) 
+                usemap = cmap10;
+            else if (cmap31 != null) 
+                usemap = cmap31;
+            else 
+                usemap = cmap10;
+            for (Iterator it = usemap.entrySet().iterator(); it.hasNext();) {
+                Map.Entry e = (Map.Entry)it.next();
+                int[] v = (int[])e.getValue();
+                Integer gi = new Integer(v[0]);
+                if (longTag.containsKey(gi))
+                    continue;
+                int c = ((Integer)e.getKey()).intValue();
+                boolean skip = true;
+                for (int k = 0; k < rg.length; k += 2) {
+                    if (c >= rg[k] && c <= rg[k + 1]) {
+                        skip = false;
+                        break;
+                    }
+                }
+                if (!skip)
+                    longTag.put(gi, includeMetrics ? new int[]{v[0], v[1], c} : null);
+            }
+        }
     }
     
     /** Outputs to the writer the font dictionaries and streams.
@@ -1035,7 +1147,9 @@ class TrueTypeFont extends BaseFont {
         int firstChar = ((Integer)params[0]).intValue();
         int lastChar = ((Integer)params[1]).intValue();
         byte shortTag[] = (byte[])params[2];
-        if (!subset) {
+        boolean subsetp = ((Boolean)params[3]).booleanValue() && subset;
+        
+        if (!subsetp) {
             firstChar = 0;
             lastChar = shortTag.length - 1;
             for (int k = 0; k < shortTag.length; ++k)
@@ -1067,22 +1181,36 @@ class TrueTypeFont extends BaseFont {
                 ind_font = obj.getIndirectReference();
             }
             else {
-                if (subset)
+                if (subsetp)
                     subsetPrefix = createSubsetPrefix();
                 HashMap glyphs = new HashMap();
                 for (int k = firstChar; k <= lastChar; ++k) {
                     if (shortTag[k] != 0) {
-                        int metrics[];
-                        if (fontSpecific)
-                            metrics = getMetricsTT(k);
-                        else
-                            metrics = getMetricsTT(unicodeDifferences[k]);
+                        int[] metrics = null;
+                        if (specialMap != null) {
+                            int[] cd = GlyphList.nameToUnicode(differences[k]);
+                            if (cd != null)
+                                metrics = getMetricsTT(cd[0]);
+                        }
+                        else {
+                            if (fontSpecific)
+                                metrics = getMetricsTT(k);
+                            else
+                                metrics = getMetricsTT(unicodeDifferences[k]);
+                        }
                         if (metrics != null)
                             glyphs.put(new Integer(metrics[0]), null);
                     }
                 }
-                TrueTypeFontSubSet sb = new TrueTypeFontSubSet(fileName, new RandomAccessFileOrArray(rf), glyphs, directoryOffset, true);
-                byte b[] = sb.process();
+                addRangeUni(glyphs, false, subsetp);
+                byte[] b = null;
+                if (subsetp || directoryOffset != 0 || subsetRanges != null) {
+                    TrueTypeFontSubSet sb = new TrueTypeFontSubSet(fileName, new RandomAccessFileOrArray(rf), glyphs, directoryOffset, true, !subsetp);
+                    b = sb.process();
+                }
+                else {
+                    b = getFullFont();
+                }
                 int lengths[] = new int[]{b.length};
                 pobj = new StreamFont(b, lengths);
                 obj = writer.addToBody(pobj);
@@ -1144,6 +1272,10 @@ class TrueTypeFont extends BaseFont {
             return (int[])cmap31.get(new Integer(c));
         if (fontSpecific && cmap10 != null) 
             return (int[])cmap10.get(new Integer(c));
+        if (cmap31 != null) 
+            return (int[])cmap31.get(new Integer(c));
+        if (cmap10 != null) 
+            return (int[])cmap10.get(new Integer(c));
         return null;
     }
 
@@ -1207,5 +1339,47 @@ class TrueTypeFont extends BaseFont {
     public boolean hasKernPairs() {
         return kerning.size() > 0;
     }    
-}
-
+    
+    /**
+     * Sets the font name that will appear in the pdf font dictionary.
+     * Use with care as it can easily make a font unreadable if not embedded.
+     * @param name the new font name
+     */    
+    public void setPostscriptFontName(String name) {
+        fontName = name;
+    }
+    
+    /**
+     * Sets the kerning between two Unicode chars.
+     * @param char1 the first char
+     * @param char2 the second char
+     * @param kern the kerning to apply in normalized 1000 units
+     * @return <code>true</code> if the kerning was applied, <code>false</code> otherwise
+     */
+    public boolean setKerning(char char1, char char2, int kern) {
+        int metrics[] = getMetricsTT(char1);
+        if (metrics == null)
+            return false;
+        int c1 = metrics[0];
+        metrics = getMetricsTT(char2);
+        if (metrics == null)
+            return false;
+        int c2 = metrics[0];
+        kerning.put((c1 << 16) + c2, kern);
+        return true;
+    }
+    
+    protected int[] getRawCharBBox(int c, String name) {
+        HashMap map = null;
+        if (name == null || cmap31 == null)
+            map = cmap10;
+        else
+            map = cmap31;
+        if (map == null)
+            return null;
+        int metric[] = (int[])map.get(new Integer(c));
+        if (metric == null || bboxes == null)
+            return null;
+        return bboxes[metric[0]];
+    }
+}
\ No newline at end of file
