@@ -1,8 +1,8 @@
 /*
- * $Id: Type1Font.java,v 1.50 2004/02/07 10:28:46 blowagie Exp $
+ * $Id: Type1Font.java,v 1.66 2006/08/27 16:25:17 psoares33 Exp $
  * $Name:  $
  *
- * Copyright 2001, 2002 Paulo Soares
+ * Copyright 2001-2006 Paulo Soares
  *
  * The contents of this file are subject to the Mozilla Public License Version 1.1
  * (the "License"); you may not use this file except in compliance with the License.
@@ -50,10 +50,8 @@
 
 package com.lowagie.text.pdf;
 
-import com.lowagie.text.ExceptionConverter;
 import com.lowagie.text.DocumentException;
 import java.util.HashMap;
-import java.util.ArrayList;
 import java.util.StringTokenizer;
 import com.lowagie.text.pdf.fonts.FontsResourceAnchor;
 import java.io.*;
@@ -135,10 +133,11 @@ class Type1Font extends BaseFont
     private int StdVW = 80;
     
 /** Represents the section CharMetrics in the AFM file. Each
- *  element of this array contains a <CODE>Object[3]</CODE> with an
- *  Integer, Integer and String. This is the code, width and name.
+ *  value of this array contains a <CODE>Object[4]</CODE> with an
+ *  Integer, Integer, String and int[]. This is the code, width, name and char bbox.
+ *  The key is the name of the char and also an Integer with the char number.
  */
-    private ArrayList CharMetrics = new ArrayList();
+    private HashMap CharMetrics = new HashMap();
 /** Represents the section KernPairs in the AFM file. The key is
  *  the name of the first character and the value is a <CODE>Object[]</CODE>
  *  with 2 elements for each kern pair. Position 0 is the name of
@@ -244,14 +243,38 @@ class Type1Font extends BaseFont
                 }
             }
         }
+        else if (afmFile.toLowerCase().endsWith(".pfm")) {
+            try {
+                ByteArrayOutputStream ba = new ByteArrayOutputStream();
+                if (ttfAfm == null)
+                    rf = new RandomAccessFileOrArray(afmFile);
+                else
+                    rf = new RandomAccessFileOrArray(ttfAfm);
+                Pfm2afm.convert(rf, ba);
+                rf.close();
+                rf = new RandomAccessFileOrArray(ba.toByteArray());
+                process(rf);
+            }
+            finally {
+                if (rf != null) {
+                    try {
+                        rf.close();
+                    }
+                    catch (Exception e) {
+                        // empty on purpose
+                    }
+                }
+            }
+        }
         else
-            throw new DocumentException(afmFile + " is not an AFM font file.");
+            throw new DocumentException(afmFile + " is not an AFM or PFM font file.");
         try {
             EncodingScheme = EncodingScheme.trim();
             if (EncodingScheme.equals("AdobeStandardEncoding") || EncodingScheme.equals("StandardEncoding")) {
                 fontSpecific = false;
             }
-            PdfEncodings.convertToBytes(" ", enc); // check if the encoding exists
+            if (!encoding.startsWith("#"))
+                PdfEncodings.convertToBytes(" ", enc); // check if the encoding exists
             createEncoding();
         }
         catch (RuntimeException re) {
@@ -269,29 +292,18 @@ class Type1Font extends BaseFont
  * @param name the glyph name
  * @return the width of the char
  */
-    int getRawWidth(int c, String name)
-    {
-        try {
-            if (name == null) { // font specific
-                for (int k = 0; k < CharMetrics.size(); ++k) {
-                    Object metrics[] = (Object[])CharMetrics.get(k);
-                    if (((Integer)(metrics[0])).intValue() == c)
-                        return ((Integer)(metrics[1])).intValue();
-                }
-            }
-            else {
-                if (name.equals(".notdef"))
-                    return 0;
-                for (int k = 0; k < CharMetrics.size(); ++k) {
-                    Object metrics[] = (Object[])CharMetrics.get(k);
-                    if (name.equals(metrics[2]))
-                        return ((Integer)(metrics[1])).intValue();
-                }
-            }
+    int getRawWidth(int c, String name) {
+        Object metrics[];
+        if (name == null) { // font specific
+            metrics = (Object[])CharMetrics.get(new Integer(c));
         }
-        catch (Exception e) {
-            throw new ExceptionConverter(e);
+        else {
+            if (name.equals(".notdef"))
+                return 0;
+            metrics = (Object[])CharMetrics.get(name);
         }
+        if (metrics != null)
+            return ((Integer)(metrics[1])).intValue();
         return 0;
     }
     
@@ -332,7 +344,7 @@ class Type1Font extends BaseFont
         boolean isMetrics = false;
         while ((line = rf.readLine()) != null)
         {
-            StringTokenizer tok = new StringTokenizer(line);
+            StringTokenizer tok = new StringTokenizer(line, " ,\n\r\t\f");
             if (!tok.hasMoreTokens())
                 continue;
             String ident = tok.nextToken();
@@ -397,6 +409,8 @@ class Type1Font extends BaseFont
             Integer C = new Integer(-1);
             Integer WX = new Integer(250);
             String N = "";
+            int B[] = null;
+
             tok = new StringTokenizer(line, ";");
             while (tok.hasMoreTokens())
             {
@@ -410,8 +424,17 @@ class Type1Font extends BaseFont
                     WX = new Integer(Float.valueOf(tokc.nextToken()).intValue());
                 else if (ident.equals("N"))
                     N = tokc.nextToken();
+                else if (ident.equals("B")) {
+                    B = new int[]{Integer.parseInt(tokc.nextToken()), 
+                                         Integer.parseInt(tokc.nextToken()),
+                                         Integer.parseInt(tokc.nextToken()),
+                                         Integer.parseInt(tokc.nextToken())};
+                }
             }
-            CharMetrics.add(new Object[]{C, WX, N});
+            Object metrics[] = new Object[]{C, WX, N, B};
+            if (C.intValue() >= 0)
+                CharMetrics.put(C, metrics);
+            CharMetrics.put(N, metrics);
         }
         if (isMetrics)
             throw new DocumentException("Missing EndCharMetrics in " + fileName);
@@ -569,7 +592,7 @@ class Type1Font extends BaseFont
         dic.put(PdfName.SUBTYPE, PdfName.TYPE1);
         dic.put(PdfName.BASEFONT, new PdfName(FontName));
         boolean stdEncoding = encoding.equals("Cp1252") || encoding.equals("MacRoman");
-        if (!fontSpecific) {
+        if (!fontSpecific || specialMap != null) {
             for (int k = firstChar; k <= lastChar; ++k) {
                 if (!differences[k].equals(notdef)) {
                     firstChar = k;
@@ -597,7 +620,7 @@ class Type1Font extends BaseFont
                 dic.put(PdfName.ENCODING, enc);
             }
         }
-        if (forceWidthsOutput || !(builtinFont && (fontSpecific || stdEncoding))) {
+        if (specialMap != null || forceWidthsOutput || !(builtinFont && (fontSpecific || stdEncoding))) {
             dic.put(PdfName.FIRSTCHAR, new PdfNumber(firstChar));
             dic.put(PdfName.LASTCHAR, new PdfNumber(lastChar));
             PdfArray wd = new PdfArray();
@@ -625,7 +648,8 @@ class Type1Font extends BaseFont
         int firstChar = ((Integer)params[0]).intValue();
         int lastChar = ((Integer)params[1]).intValue();
         byte shortTag[] = (byte[])params[2];
-        if (!subset) {
+        boolean subsetp = ((Boolean)params[3]).booleanValue() && subset;
+        if (!subsetp) {
             firstChar = 0;
             lastChar = shortTag.length - 1;
             for (int k = 0; k < shortTag.length; ++k)
@@ -722,4 +746,63 @@ class Type1Font extends BaseFont
         return KernPairs.size() > 0;
     }
     
-}
+    /**
+     * Sets the font name that will appear in the pdf font dictionary.
+     * Use with care as it can easily make a font unreadable if not embedded.
+     * @param name the new font name
+     */    
+    public void setPostscriptFontName(String name) {
+        FontName = name;
+    }
+    
+    /**
+     * Sets the kerning between two Unicode chars.
+     * @param char1 the first char
+     * @param char2 the second char
+     * @param kern the kerning to apply in normalized 1000 units
+     * @return <code>true</code> if the kerning was applied, <code>false</code> otherwise
+     */
+    public boolean setKerning(char char1, char char2, int kern) {
+        String first = GlyphList.unicodeToName((int)char1);
+        if (first == null)
+            return false;
+        String second = GlyphList.unicodeToName((int)char2);
+        if (second == null)
+            return false;
+        Object obj[] = (Object[])KernPairs.get(first);
+        if (obj == null) {
+            obj = new Object[]{second, new Integer(kern)};
+            KernPairs.put(first, obj);
+            return true;
+        }
+        for (int k = 0; k < obj.length; k += 2) {
+            if (second.equals(obj[k])) {
+                obj[k + 1] = new Integer(kern);
+                return true;
+            }
+        }
+        int size = obj.length;
+        Object obj2[] = new Object[size + 2];
+        System.arraycopy(obj, 0, obj2, 0, size);
+        obj2[size] = second;
+        obj2[size + 1] = new Integer(kern);
+        KernPairs.put(first, obj2);
+        return true;
+    }
+    
+    protected int[] getRawCharBBox(int c, String name) {
+        Object metrics[];
+        if (name == null) { // font specific
+            metrics = (Object[])CharMetrics.get(new Integer(c));
+        }
+        else {
+            if (name.equals(".notdef"))
+                return null;
+            metrics = (Object[])CharMetrics.get(name);
+        }
+        if (metrics != null)
+            return ((int[])(metrics[3]));
+        return null;
+    }
+    
+}
\ No newline at end of file
