@@ -1,5 +1,5 @@
 /*
- * Copyright 2003 by Paulo Soares.
+ * Copyright 2003-2005 by Paulo Soares.
  *
  * The contents of this file are subject to the Mozilla Public License Version 1.1
  * (the "License"); you may not use this file except in compliance with the License.
@@ -47,6 +47,7 @@
 package com.lowagie.text.pdf.codec;
 import com.lowagie.text.pdf.*;
 import com.lowagie.text.Image;
+import com.lowagie.text.Jpeg;
 import com.lowagie.text.ExceptionConverter;
 import java.io.*;
 import java.util.zip.*;
@@ -122,6 +123,17 @@ public class TiffImage {
                 default:
                     return getTiffImageColor(dir, s);
             }
+            float rotation = 0;
+            if (dir.isTagPresent(TIFFConstants.TIFFTAG_ORIENTATION)) {
+                int rot = (int)dir.getFieldAsLong(TIFFConstants.TIFFTAG_ORIENTATION);
+                if (rot == TIFFConstants.ORIENTATION_BOTRIGHT || rot == TIFFConstants.ORIENTATION_BOTLEFT)
+                    rotation = (float)Math.PI;
+                else if (rot == TIFFConstants.ORIENTATION_LEFTTOP || rot == TIFFConstants.ORIENTATION_LEFTBOT)
+                    rotation = (float)(Math.PI / 2.0);
+                else if (rot == TIFFConstants.ORIENTATION_RIGHTTOP || rot == TIFFConstants.ORIENTATION_RIGHTBOT)
+                    rotation = -(float)(Math.PI / 2.0);
+            }
+
             Image img = null;
             long tiffT4Options = 0;
             long tiffT6Options = 0;
@@ -145,10 +157,12 @@ public class TiffImage {
             long tstrip = 0xFFFFFFFFL;
             if (dir.isTagPresent(TIFFConstants.TIFFTAG_ROWSPERSTRIP))
                 tstrip = dir.getFieldAsLong(TIFFConstants.TIFFTAG_ROWSPERSTRIP);
-            int rowsStrip = (int)Math.min((long)h, tstrip);
-            TIFFField field = dir.getField(TIFFConstants.TIFFTAG_STRIPOFFSETS);
+            int rowsStrip = (int)Math.min(h, tstrip);
             long offset[] = getArrayLongShort(dir, TIFFConstants.TIFFTAG_STRIPOFFSETS);
             long size[] = getArrayLongShort(dir, TIFFConstants.TIFFTAG_STRIPBYTECOUNTS);
+            if (size == null && h == rowsStrip) { // some TIFF producers are really lousy, so...
+                size = new long[]{s.length() - (int)offset[0]};
+            }
             boolean reverse = false;
             TIFFField fillOrderField =  dir.getField(TIFFConstants.TIFFTAG_FILLORDER);
             if (fillOrderField != null)
@@ -174,7 +188,7 @@ public class TiffImage {
                     if (t4OptionsField != null) {
                         tiffT4Options = t4OptionsField.getAsLong(0);
                     if ((tiffT4Options & TIFFConstants.GROUP3OPT_2DENCODING) != 0)
-                        compression = Image.CCITTG3_2D;
+                        imagecomp = Image.CCITTG3_2D;
                     if ((tiffT4Options & TIFFConstants.GROUP3OPT_FILLBITS) != 0)
                         params |= Image.CCITT_ENCODEDBYTEALIGN;
                     }
@@ -190,24 +204,24 @@ public class TiffImage {
                 byte im[] = new byte[(int)size[0]];
                 s.seek(offset[0]);
                 s.readFully(im);
-                img = Image.getInstance((int) w, (int) h, reverse, imagecomp, params, im);
+                img = Image.getInstance(w, h, reverse, imagecomp, params, im);
                 img.setInverted(true);
             }
             else {
                 int rowsLeft = h;
-                CCITTG4Encoder g4 = new CCITTG4Encoder((int)w);
+                CCITTG4Encoder g4 = new CCITTG4Encoder(w);
                 for (int k = 0; k < offset.length; ++k) {
                     byte im[] = new byte[(int)size[k]];
                     s.seek(offset[k]);
                     s.readFully(im);
                     int height = Math.min(rowsStrip, rowsLeft);
-                    TIFFFaxDecoder decoder = new TIFFFaxDecoder(fillOrder, (int)w, height);
+                    TIFFFaxDecoder decoder = new TIFFFaxDecoder(fillOrder, w, height);
                     byte outBuf[] = new byte[(w + 7) / 8 * height];
                     switch (compression) {
                         case TIFFConstants.COMPRESSION_CCITTRLEW:
                         case TIFFConstants.COMPRESSION_CCITTRLE:
                             decoder.decode1D(outBuf, im, 0, height);
-                            g4.encodeT6Lines(outBuf, 0, height);
+                            g4.fax4Encode(outBuf,height);
                             break;
                         case TIFFConstants.COMPRESSION_CCITTFAX3:
                             try {
@@ -223,25 +237,34 @@ public class TiffImage {
                                     throw e;
                                 }
                             }
-                            g4.encodeT6Lines(outBuf, 0, height);
+                            g4.fax4Encode(outBuf, height);
                             break;
                         case TIFFConstants.COMPRESSION_CCITTFAX4:
                             decoder.decodeT6(outBuf, im, 0, height, tiffT6Options);
-                            g4.encodeT6Lines(outBuf, 0, height);
+                            g4.fax4Encode(outBuf, height);
                             break;
                     }
                     rowsLeft -= rowsStrip;
                 }
                 byte g4pic[] = g4.close();
-                img = Image.getInstance((int) w, (int) h, false, Image.CCITTG4, params & Image.CCITT_BLACKIS1, g4pic);
+                img = Image.getInstance(w, h, false, Image.CCITTG4, params & Image.CCITT_BLACKIS1, g4pic);
             }
             img.setDpi(dpiX, dpiY);
             img.setXYRatio(XYRatio);
             if (dir.isTagPresent(TIFFConstants.TIFFTAG_ICCPROFILE)) {
-                TIFFField fd = dir.getField(TIFFConstants.TIFFTAG_ICCPROFILE);
-                img.tagICC(ICC_Profile.getInstance(fd.getAsBytes()));
+                try {
+                    TIFFField fd = dir.getField(TIFFConstants.TIFFTAG_ICCPROFILE);
+                    ICC_Profile icc_prof = ICC_Profile.getInstance(fd.getAsBytes());
+                    if (icc_prof.getNumComponents() == 1)
+                        img.tagICC(icc_prof);
+                }
+                catch (Exception e) {
+                    //empty
+                }
             }
             img.setOriginalType(Image.ORIGINAL_TIFF);
+            if (rotation != 0)
+                img.setInitialRotation(rotation);
             return img;
         }
         catch (Exception e) {
@@ -259,6 +282,8 @@ public class TiffImage {
                 case TIFFConstants.COMPRESSION_LZW:
                 case TIFFConstants.COMPRESSION_PACKBITS:
                 case TIFFConstants.COMPRESSION_DEFLATE:
+                case TIFFConstants.COMPRESSION_OJPEG:
+                case TIFFConstants.COMPRESSION_JPEG:
                     break;
                 default:
                     throw new IllegalArgumentException("The compression " + compression + " is not supported.");
@@ -272,7 +297,18 @@ public class TiffImage {
                 case TIFFConstants.PHOTOMETRIC_PALETTE:
                     break;
                 default:
-                    throw new IllegalArgumentException("The photometric " + photometric + " is not supported.");
+                    if (compression != TIFFConstants.COMPRESSION_OJPEG && compression != TIFFConstants.COMPRESSION_JPEG)
+                        throw new IllegalArgumentException("The photometric " + photometric + " is not supported.");
+            }
+            float rotation = 0;
+            if (dir.isTagPresent(TIFFConstants.TIFFTAG_ORIENTATION)) {
+                int rot = (int)dir.getFieldAsLong(TIFFConstants.TIFFTAG_ORIENTATION);
+                if (rot == TIFFConstants.ORIENTATION_BOTRIGHT || rot == TIFFConstants.ORIENTATION_BOTLEFT)
+                    rotation = (float)Math.PI;
+                else if (rot == TIFFConstants.ORIENTATION_LEFTTOP || rot == TIFFConstants.ORIENTATION_LEFTBOT)
+                    rotation = (float)(Math.PI / 2.0);
+                else if (rot == TIFFConstants.ORIENTATION_RIGHTTOP || rot == TIFFConstants.ORIENTATION_RIGHTBOT)
+                    rotation = -(float)(Math.PI / 2.0);
             }
             if (dir.isTagPresent(TIFFConstants.TIFFTAG_PLANARCONFIG)
                 && dir.getFieldAsLong(TIFFConstants.TIFFTAG_PLANARCONFIG) == TIFFConstants.PLANARCONFIG_SEPARATE)
@@ -304,10 +340,15 @@ public class TiffImage {
             if (dir.isTagPresent(TIFFConstants.TIFFTAG_RESOLUTIONUNIT))
                 resolutionUnit = (int)dir.getFieldAsLong(TIFFConstants.TIFFTAG_RESOLUTIONUNIT);
             dpiX = getDpi(dir.getField(TIFFConstants.TIFFTAG_XRESOLUTION), resolutionUnit);
-            dpiX = getDpi(dir.getField(TIFFConstants.TIFFTAG_YRESOLUTION), resolutionUnit);
-            int rowsStrip = (int)dir.getFieldAsLong(TIFFConstants.TIFFTAG_ROWSPERSTRIP);
+            dpiY = getDpi(dir.getField(TIFFConstants.TIFFTAG_YRESOLUTION), resolutionUnit);
+            int rowsStrip = h;
+            if (dir.isTagPresent(TIFFConstants.TIFFTAG_ROWSPERSTRIP)) //another hack for broken tiffs
+                rowsStrip = (int)dir.getFieldAsLong(TIFFConstants.TIFFTAG_ROWSPERSTRIP);
             long offset[] = getArrayLongShort(dir, TIFFConstants.TIFFTAG_STRIPOFFSETS);
             long size[] = getArrayLongShort(dir, TIFFConstants.TIFFTAG_STRIPBYTECOUNTS);
+            if (size == null && h == rowsStrip) { // some TIFF producers are really lousy, so...
+                size = new long[]{s.length() - (int)offset[0]};
+            }
             if (compression == TIFFConstants.COMPRESSION_LZW) {
                 TIFFField predictorField = dir.getField(TIFFConstants.TIFFTAG_PREDICTOR);
                 if (predictorField != null) {
@@ -331,75 +372,114 @@ public class TiffImage {
             }
             else {
                 stream = new ByteArrayOutputStream();
-                zip = new DeflaterOutputStream(stream);
+                if (compression != TIFFConstants.COMPRESSION_OJPEG && compression != TIFFConstants.COMPRESSION_JPEG)
+                    zip = new DeflaterOutputStream(stream);
             }
-            for (int k = 0; k < offset.length; ++k) {
-                byte im[] = new byte[(int)size[k]];
-                s.seek(offset[k]);
-                s.readFully(im);
-                int height = Math.min(rowsStrip, rowsLeft);
-                byte outBuf[] = null;
-                if (compression != TIFFConstants.COMPRESSION_NONE)
-                    outBuf = new byte[(w * bitsPerSample * samplePerPixel + 7) / 8 * height];
-                switch (compression) {
-                    case TIFFConstants.COMPRESSION_DEFLATE:
-                        inflate(im, outBuf);
-                        break;
-                    case TIFFConstants.COMPRESSION_NONE:
-                        outBuf = im;
-                        break;
-                    case TIFFConstants.COMPRESSION_PACKBITS:
-                        decodePackbits(im,  outBuf);
-                        break;
-                    case TIFFConstants.COMPRESSION_LZW:
-                        lzwDecoder.decode(im, outBuf, height);
-                        break;
+            if (compression == TIFFConstants.COMPRESSION_OJPEG) {
+                if ((!dir.isTagPresent(TIFFConstants.TIFFTAG_JPEGIFOFFSET))
+                || (!dir.isTagPresent(TIFFConstants.TIFFTAG_JPEGIFBYTECOUNT))) {
+                    throw new RuntimeException("Missing tag(s) for OJPEG compression.");
+                }
+                int jpegOffset = (int)dir.getFieldAsLong(TIFFConstants.TIFFTAG_JPEGIFOFFSET);
+                int jpegLength = (int)dir.getFieldAsLong(TIFFConstants.TIFFTAG_JPEGIFBYTECOUNT) +
+                        (int)size[0];
+
+                byte[] jpeg = new byte[Math.min(jpegLength, s.length() - jpegOffset)];
+
+                int posFilePointer = s.getFilePointer();
+                posFilePointer += jpegOffset;
+                s.seek(posFilePointer);
+                s.readFully(jpeg);
+                img = new Jpeg(jpeg);
+            } 
+            else if (compression == TIFFConstants.COMPRESSION_JPEG) {
+                if (size.length > 1)
+                    throw new IOException("Compression JPEG is only supported with a single strip. This image has " + size.length + " strips.");
+                byte[] jpeg = new byte[(int)size[0]];
+                s.seek(offset[0]);
+                s.readFully(jpeg);
+                img = new Jpeg(jpeg);
+            } 
+            else {
+                for (int k = 0; k < offset.length; ++k) {
+                    byte im[] = new byte[(int)size[k]];
+                    s.seek(offset[k]);
+                    s.readFully(im);
+                    int height = Math.min(rowsStrip, rowsLeft);
+                    byte outBuf[] = null;
+                    if (compression != TIFFConstants.COMPRESSION_NONE)
+                        outBuf = new byte[(w * bitsPerSample * samplePerPixel + 7) / 8 * height];
+                    switch (compression) {
+                        case TIFFConstants.COMPRESSION_DEFLATE:
+                            inflate(im, outBuf);
+                            break;
+                        case TIFFConstants.COMPRESSION_NONE:
+                            outBuf = im;
+                            break;
+                        case TIFFConstants.COMPRESSION_PACKBITS:
+                            decodePackbits(im,  outBuf);
+                            break;
+                        case TIFFConstants.COMPRESSION_LZW:
+                            lzwDecoder.decode(im, outBuf, height);
+                            break;
+                    }
+                    if (bitsPerSample == 1 && samplePerPixel == 1) {
+                        g4.fax4Encode(outBuf, height);
+                    }
+                    else {
+                        zip.write(outBuf);
+                    }
+                    rowsLeft -= rowsStrip;
                 }
                 if (bitsPerSample == 1 && samplePerPixel == 1) {
-                    g4.encodeT6Lines(outBuf, 0, height);
+                    img = Image.getInstance(w, h, false, Image.CCITTG4, 
+                        photometric == TIFFConstants.PHOTOMETRIC_MINISBLACK ? Image.CCITT_BLACKIS1 : 0, g4.close());
                 }
                 else {
-                    zip.write(outBuf);
+                    zip.close();
+                    img = Image.getInstance(w, h, samplePerPixel, bitsPerSample, stream.toByteArray());
+                    img.setDeflated(true);
                 }
-                rowsLeft -= rowsStrip;
-            }
-            if (bitsPerSample == 1 && samplePerPixel == 1) {
-                img = Image.getInstance(w, h, false, Image.CCITTG4, 
-                    photometric == TIFFConstants.PHOTOMETRIC_MINISBLACK ? Image.CCITT_BLACKIS1 : 0, g4.close());
-            }
-            else {
-                zip.close();
-                img = Image.getInstance(w, h, samplePerPixel, bitsPerSample, stream.toByteArray());
-                img.setDeflated(true);
             }
             img.setDpi(dpiX, dpiY);
-            if (dir.isTagPresent(TIFFConstants.TIFFTAG_ICCPROFILE)) {
-                TIFFField fd = dir.getField(TIFFConstants.TIFFTAG_ICCPROFILE);
-                img.tagICC(ICC_Profile.getInstance(fd.getAsBytes()));
-            }
-            if (dir.isTagPresent(TIFFConstants.TIFFTAG_COLORMAP)) {
-                TIFFField fd = dir.getField(TIFFConstants.TIFFTAG_COLORMAP);
-                char rgb[] = fd.getAsChars();
-                byte palette[] = new byte[rgb.length];
-                int gColor = rgb.length / 3;
-                int bColor = gColor * 2;
-                for (int k = 0; k < gColor; ++k) {
-                    palette[k * 3] = (byte)(rgb[k] >>> 8);
-                    palette[k * 3 + 1] = (byte)(rgb[k + gColor] >>> 8);
-                    palette[k * 3 + 2] = (byte)(rgb[k + bColor] >>> 8);
+            if (compression != TIFFConstants.COMPRESSION_OJPEG && compression != TIFFConstants.COMPRESSION_JPEG) {
+                if (dir.isTagPresent(TIFFConstants.TIFFTAG_ICCPROFILE)) {
+                    try {
+                        TIFFField fd = dir.getField(TIFFConstants.TIFFTAG_ICCPROFILE);
+                        ICC_Profile icc_prof = ICC_Profile.getInstance(fd.getAsBytes());
+                        if (samplePerPixel == icc_prof.getNumComponents())
+                            img.tagICC(icc_prof);
+                    }
+                    catch (Exception e) {
+                        //empty
+                    }
                 }
-                PdfArray indexed = new PdfArray();
-                indexed.add(PdfName.INDEXED);
-                indexed.add(PdfName.DEVICERGB);
-                indexed.add(new PdfNumber(gColor - 1));
-                indexed.add(new PdfString(palette));
-                PdfDictionary additional = new PdfDictionary();
-                additional.put(PdfName.COLORSPACE, indexed);
-                img.setAdditional(additional);
+                if (dir.isTagPresent(TIFFConstants.TIFFTAG_COLORMAP)) {
+                    TIFFField fd = dir.getField(TIFFConstants.TIFFTAG_COLORMAP);
+                    char rgb[] = fd.getAsChars();
+                    byte palette[] = new byte[rgb.length];
+                    int gColor = rgb.length / 3;
+                    int bColor = gColor * 2;
+                    for (int k = 0; k < gColor; ++k) {
+                        palette[k * 3] = (byte)(rgb[k] >>> 8);
+                        palette[k * 3 + 1] = (byte)(rgb[k + gColor] >>> 8);
+                        palette[k * 3 + 2] = (byte)(rgb[k + bColor] >>> 8);
+                    }
+                    PdfArray indexed = new PdfArray();
+                    indexed.add(PdfName.INDEXED);
+                    indexed.add(PdfName.DEVICERGB);
+                    indexed.add(new PdfNumber(gColor - 1));
+                    indexed.add(new PdfString(palette));
+                    PdfDictionary additional = new PdfDictionary();
+                    additional.put(PdfName.COLORSPACE, indexed);
+                    img.setAdditional(additional);
+                }
+                img.setOriginalType(Image.ORIGINAL_TIFF);
             }
             if (photometric == TIFFConstants.PHOTOMETRIC_MINISWHITE)
                 img.setInverted(true);
-            img.setOriginalType(Image.ORIGINAL_TIFF);
+            if (rotation != 0)
+                img.setInitialRotation(rotation);
             return img;
         }
         catch (Exception e) {
