@@ -1,5 +1,5 @@
 /**
- * $Id: RtfCell.java,v 1.23 2004/02/07 10:29:03 blowagie Exp $
+ * $Id: RtfCell.java,v 1.38 2006/09/12 13:12:53 blowagie Exp $
  * $Name:  $
  *
  * Copyright 2001, 2002 by Mark Hall
@@ -60,9 +60,13 @@ import java.awt.Color;
  * A Helper Class for the <CODE>RtfWriter</CODE>.
  * <P>
  * Do not use it directly
+ * 
+ * ONLY FOR USE WITH THE RtfWriter NOT with the RtfWriter2.
  *
  * Parts of this Class were contributed by Steffen Stundzig. Many thanks for the
  * improvements.
+ * Updates by Benoit Wiart
+ * @deprecated Please move to the RtfWriter2 and associated classes.
  */
 public class RtfCell {
     /** Constants for merging Cells */
@@ -178,6 +182,8 @@ public class RtfCell {
      * @param cellWidth The default width of a cell
      * @param x The column index of this <code>RtfCell</code>
      * @param y The row index of this <code>RtfCell</code>
+     * @param cellpadding the cellpadding
+     * @return the position of the right side of the cell
      */
     public int importCell(Cell cell, int cellLeft, int cellWidth, int x, int y, int cellpadding) {
         this.cellpadding = cellpadding;
@@ -224,6 +230,8 @@ public class RtfCell {
      *
      * @param os The <code>OutputStream</code> to which to write the properties
      * of the <code>RtfCell</code> to.
+     * @return true if writing the cell settings succeeded
+     * @throws DocumentException
      */
     public boolean writeCellSettings(ByteArrayOutputStream os) throws DocumentException {
         try {
@@ -359,13 +367,10 @@ public class RtfCell {
             }
             os.write(RtfWriter.escape);
             os.write(cellBackgroundColor);
-            if ((store.backgroundColor() == null) && (store.grayFill() == 0)) {
+            if (store.backgroundColor() == null) {
                 writeInt(os, writer.addColor(new Color(255, 255, 255)));
-            } else if (store.backgroundColor() != null) {
-                writeInt(os, writer.addColor(store.backgroundColor()));
             } else {
-                int shadeColor = (int) (store.grayFill() * 255);
-                writeInt(os, writer.addColor(new Color(shadeColor, shadeColor, shadeColor)));
+                writeInt(os, writer.addColor(store.backgroundColor()));
             }
             os.write((byte) '\n');
             os.write(RtfWriter.escape);
@@ -413,29 +418,53 @@ public class RtfCell {
      *
      * @param os The <code>OutputStream</code> to which to write the content of
      * the <code>RtfCell</code> to.
+     * @return true if writing the cell content succeeded
+     * @throws DocumentException
      */
     public boolean writeCellContent(ByteArrayOutputStream os) throws DocumentException {
         try {
             if (mergeType == MERGE_HORIZ_PREV || mergeType == MERGE_BOTH_PREV) {
                 return true;
             }
-
+            
             if (!emptyCell) {
                 Iterator cellIterator = store.getElements();
+                Paragraph container = null;
                 while (cellIterator.hasNext()) {
                     Element element = (Element) cellIterator.next();
-
-                    // if horizontal alignment is undefined overwrite
-                    // with that of enclosing cell
-                    if (element instanceof Paragraph && ((Paragraph) element).alignment() == Element.ALIGN_UNDEFINED) {
-                        ((Paragraph) element).setAlignment(store.horizontalAlignment());
-                    }
-                    writer.addElement(element, os);
-                    if (element.type() == Element.PARAGRAPH && cellIterator.hasNext()) {
-                        os.write(RtfWriter.escape);
-                        os.write(RtfWriter.paragraph);
+                    // should we wrap it in a paragraph
+                    if(!(element instanceof Paragraph)) {
+                        if(container != null) {
+                            container.add(element);
+                        } else {
+                            container = new Paragraph();
+                            container.setAlignment(store.horizontalAlignment());
+                            container.add(element);
+                        }
+                    } else {
+                        if(container != null) {
+                            writer.addElement(container, os);
+                            container =null;
+                            container =null;
+                        }
+                        
+                        
+                        // if horizontal alignment is undefined overwrite
+                        // with that of enclosing cell
+                        if (element instanceof Paragraph && ((Paragraph) element).alignment() == Element.ALIGN_UNDEFINED) {
+                            ((Paragraph) element).setAlignment(store.horizontalAlignment());
+                        }
+                        writer.addElement(element, os);
+                        if (element.type() == Element.PARAGRAPH && cellIterator.hasNext()) {
+                            os.write(RtfWriter.escape);
+                            os.write(RtfWriter.paragraph);
+                        }
                     }
                 }
+                if(container != null) {
+                    writer.addElement(container, os);
+                    container =null;
+                }
             } else {
                 os.write(RtfWriter.escape);
                 os.write(RtfWriter.paragraphDefaults);
@@ -482,28 +511,37 @@ public class RtfCell {
         return cellWidth;
     }
 
-
+    /**
+     * sets the width of the cell
+     * @param value a width
+     */
     public void setCellWidth(int value) {
         cellWidth = value;
     }
 
     /**
      * Get the position of the right border of this <code>RtfCell</code>.
+     * @return position of the right border
      */
     public int getCellRight() {
         return cellRight;
     }
 
 
+    /**
+     * Sets the right position of the cell
+     * @param value a cell position
+     */
     public void setCellRight(int value) {
         cellRight = value;
     }
 
-    /*
+    /**
      * Write an Integer to the Outputstream.
      *
      * @param out The <code>OutputStream</code> to be written to.
      * @param i The int to be written.
+     * @throws IOException
      */
     private void writeInt(ByteArrayOutputStream out, int i) throws IOException {
         out.write(Integer.toString(i).getBytes());
