@@ -1,5 +1,5 @@
 /*
- * $Id: RtfWriter.java,v 1.49 2004/02/07 10:29:03 blowagie Exp $
+ * $Id: RtfWriter.java,v 1.71 2006/09/12 13:12:53 blowagie Exp $
  * $Name:  $
  *
  * Copyright 2001, 2002 by Mark Hall
@@ -61,9 +61,13 @@ import java.util.Date;
 import java.awt.Color;
 import java.text.SimpleDateFormat;
 import java.text.ParsePosition;
-import com.lowagie.text.pdf.wmf.MetaDo;
+import com.lowagie.text.pdf.codec.wmf.MetaDo;
 
 /**
+ * If you are creating a new project using the rtf part of iText, please
+ * consider using the new RtfWriter2. The RtfWriter is in bug-fix-only mode,
+ * will be deprecated end of 2005 and removed end of 2007.
+ * 
  * A <CODE>DocWriter</CODE> class for Rich Text Files (RTF).
  * <P>
  * A <CODE>RtfWriter</CODE> can be added as a <CODE>DocListener</CODE>
@@ -105,10 +109,11 @@ import com.lowagie.text.pdf.wmf.MetaDo;
  * </ul>
  * <br />
  *
- * @author <a href="mailto:mhall@myrealbox.com">Mark.Hall@myrealbox.com</a>
- * @author Steffen Stundzig
- * @author <a href="ericmattes@yahoo.com">Eric Mattes</a>
- * @author <a href="raul.wegmann@uam.es">Raul Wegmann</a>
+ * @author Mark Hall (mhall@edu.uni-klu.ac.at)
+ * @author Steffen Stundzig (Steffen.Stundzig@smb-tec.com)
+ * @author Eric Mattes (ericmattes@yahoo.com)
+ * @author Raul Wegmann (raul.wegmann@uam.es)
+ * @deprecated The RtfWriter is deprecated and will be removed from the iText library end of 2007
  */
 public class RtfWriter extends DocWriter implements DocListener {
     /**
@@ -721,6 +726,7 @@ public class RtfWriter extends DocWriter implements DocListener {
 
     /**
      * Adds the footer to the bottom of the <CODE>Document</CODE>.
+     * @param footer
      */
     public void setFooter(HeaderFooter footer) {
         this.footer = footer;
@@ -729,6 +735,7 @@ public class RtfWriter extends DocWriter implements DocListener {
 
     /**
      * Adds the header to the top of the <CODE>Document</CODE>.
+     * @param header
      */
     public void setHeader(HeaderFooter header) {
         this.header = header;
@@ -826,11 +833,15 @@ public class RtfWriter extends DocWriter implements DocListener {
 
     /**
      * Signals that an <CODE>Element</CODE> was added to the <CODE>Document</CODE>.
-     *
+     * 
+     * @param element A high level object to add
      * @return    <CODE>true</CODE> if the element was added, <CODE>false</CODE> if not.
      * @throws    DocumentException   if a document isn't open yet, or has been closed
      */
     public boolean add(Element element) throws DocumentException {
+        if (pause) {
+            return false;
+        }
         return addElement(element, content);
     }
 
@@ -839,6 +850,8 @@ public class RtfWriter extends DocWriter implements DocListener {
 
     /**
      * Adds an <CODE>Element</CODE> to the <CODE>Document</CODE>.
+     * @param element the high level element to add
+     * @param out the outputstream to which the RTF data is sent
      * @return    <CODE>true</CODE> if the element was added, <CODE>false</CODE> if not.
      * @throws    DocumentException   if a document isn't open yet, or has been closed
      */
@@ -865,7 +878,12 @@ public class RtfWriter extends DocWriter implements DocListener {
                     writeList((com.lowagie.text.List) element, out);
                     break;
                 case Element.TABLE:
-                    writeTable((Table) element, out);
+                	try {
+                		writeTable((Table) element, out);
+                	}
+                	catch(ClassCastException cce) {
+                		writeTable(((SimpleTable)element).createTable(), out);
+                	}
                     break;
                 case Element.ANNOTATION:
                     writeAnnotation((Annotation) element, out);
@@ -909,6 +927,7 @@ public class RtfWriter extends DocWriter implements DocListener {
      * @param out The <code>ByteArrayOutputStream</code> to write to
      *
      * @throws IOException
+     * @throws DocumentException
      */
     private void writeSection(Section sectionElement, ByteArrayOutputStream out) throws IOException, DocumentException {
         if (sectionElement.type() == Element.CHAPTER) {
@@ -1062,6 +1081,7 @@ public class RtfWriter extends DocWriter implements DocListener {
      * @param out The <code>ByteArrayOutputStream</code> to write to
      *
      * @throws IOException
+     * @throws DocumentException
      */
     private void writeChunk(Chunk chunk, ByteArrayOutputStream out) throws IOException, DocumentException {
         if (chunk instanceof RtfField) {
@@ -1071,7 +1091,7 @@ public class RtfWriter extends DocWriter implements DocListener {
                 writeImage(chunk.getImage(), out);
             } else {
                 writeInitialFontSignature(out, chunk);
-                out.write(filterSpecialChar(chunk.content()).getBytes());
+                out.write(filterSpecialChar(chunk.content(), false).getBytes());
                 writeFinishingFontSignature(out, chunk);
             }
         }
@@ -1178,6 +1198,7 @@ public class RtfWriter extends DocWriter implements DocListener {
      * @param out The <code>ByteArrayOutputStream</code> to write to
      *
      * @throws IOException
+     * @throws DocumentException
      */
     private void writeListElement(ListItem listItem, ByteArrayOutputStream out) throws IOException, DocumentException {
         Iterator chunks = listItem.getChunks().iterator();
@@ -1427,7 +1448,7 @@ public class RtfWriter extends DocWriter implements DocListener {
         inTable = true;
         table.complete();
         RtfTable rtfTable = new RtfTable(this);
-        rtfTable.importTable(table, pageWidth);
+        rtfTable.importTable(table, pageWidth - marginLeft - marginRight);
         rtfTable.writeTable(out);
         inTable = false;
     }
@@ -1525,8 +1546,12 @@ public class RtfWriter extends DocWriter implements DocListener {
             } else {
                 imgIn = new ByteArrayInputStream(image.getOriginalData());
             }
-            if (type == Image.ORIGINAL_WMF) //remove the placeable header
-                imgIn.skip(22);
+            if (type == Image.ORIGINAL_WMF) { //remove the placeable header
+                long skipLength = 22;
+            	while(skipLength > 0) {
+            	    skipLength = skipLength - imgIn.skip(skipLength);
+            	}
+            }
         }
         int buffer = -1;
         int count = 0;
@@ -1696,7 +1721,6 @@ public class RtfWriter extends DocWriter implements DocListener {
     private boolean writeDocument() {
         try {
             writeDocumentIntro();
-            os.write((byte) '\n');
             writeFontList();
             os.write((byte) '\n');
             writeColorList();
@@ -1721,6 +1745,7 @@ public class RtfWriter extends DocWriter implements DocListener {
     }
 
     /** Write the Rich Text file settings
+     * @throws IOException
      */
     private void writeDocumentIntro() throws IOException {
         os.write(openGroup);
@@ -1731,6 +1756,7 @@ public class RtfWriter extends DocWriter implements DocListener {
         os.write(escape);
         os.write(ansiCodepage);
         writeInt(os, 1252);
+        os.write((byte)'\n');
         os.write(escape);
         os.write(defaultFont);
         writeInt(os, 0);
@@ -1738,6 +1764,7 @@ public class RtfWriter extends DocWriter implements DocListener {
 
     /**
      * Write the font list to the final <code>ByteArrayOutputStream</code>
+     * @throws IOException
      */
     private void writeFontList() throws IOException {
         Font fnt;
@@ -1799,7 +1826,7 @@ public class RtfWriter extends DocWriter implements DocListener {
                     os.write(fontCharset);
                     writeInt(os, 0);
                     os.write(delimiter);
-                    os.write(fnt.getFamilyname().getBytes());
+                    os.write(filterSpecialChar(fnt.getFamilyname(), true).getBytes());
             }
             os.write(commaDelimiter);
             os.write(closeGroup);
@@ -1809,6 +1836,7 @@ public class RtfWriter extends DocWriter implements DocListener {
 
     /**
      * Write the colour list to the final <code>ByteArrayOutputStream</code>
+     * @throws IOException
      */
     private void writeColorList() throws IOException {
         Color color = null;
@@ -1834,6 +1862,7 @@ public class RtfWriter extends DocWriter implements DocListener {
 
     /**
      * Write the Information Group to the final <code>ByteArrayOutputStream</code>
+     * @throws IOException
      */
     private void writeInfoGroup() throws IOException {
         os.write(openGroup);
@@ -1845,6 +1874,7 @@ public class RtfWriter extends DocWriter implements DocListener {
 
     /**
      * Write the listtable and listoverridetable to the final <code>ByteArrayOutputStream</code>
+     * @throws IOException
      */
     private void writeList() throws IOException {
         listtable.write(closeGroup);
@@ -1859,6 +1889,7 @@ public class RtfWriter extends DocWriter implements DocListener {
      *
      * @param out The <code>OuputStream</code> to which the <code>int</code> value is to be written
      * @param i The <code>int</code> value to be written
+     * @throws IOException
      */
     public final static void writeInt(OutputStream out, int i) throws IOException {
         out.write(Integer.toString(i).getBytes());
@@ -1892,6 +1923,7 @@ public class RtfWriter extends DocWriter implements DocListener {
      * Write the current header and footer to a <code>ByteArrayOutputStream</code>
      *
      * @param os		The <code>ByteArrayOutputStream</code> to which the header and footer will be written.
+     * @throws IOException
      */
     public void writeHeadersFooters(ByteArrayOutputStream os) throws IOException {
         if (this.footer instanceof RtfHeaderFooters) {
@@ -1944,6 +1976,7 @@ public class RtfWriter extends DocWriter implements DocListener {
      * @param headerFooter	The <code>HeaderFooter</code> object to be written.
      * @param hfType		The type of header or footer to be added.
      * @param target		The <code>ByteArrayOutputStream</code> to which the <code>HeaderFooter</code> will be written.
+     * @throws IOException
      */
     private void writeHeaderFooter(HeaderFooter headerFooter, byte[] hfType, ByteArrayOutputStream target) throws IOException {
         inHeaderFooter = true;
@@ -1980,6 +2013,7 @@ public class RtfWriter extends DocWriter implements DocListener {
     /**
      *  Write the <code>Document</code>'s Paper and Margin Size
      *  to the final <code>ByteArrayOutputStream</code>
+     * @throws IOException
      */
     private void writeDocumentFormat() throws IOException {
 //        os.write(openGroup);
@@ -2039,6 +2073,7 @@ public class RtfWriter extends DocWriter implements DocListener {
      * Writes the default values for the current Section
      *
      * @param out The <code>ByteArrayOutputStream</code> to be written to
+     * @throws IOException
      */
     private void writeSectionDefaults(ByteArrayOutputStream out) throws IOException {
         if (header instanceof RtfHeaderFooters || footer instanceof RtfHeaderFooters) {
@@ -2065,6 +2100,13 @@ public class RtfWriter extends DocWriter implements DocListener {
             out.write(escape);
             out.write(sectionPageHeight);
             writeInt(out, pageHeight);
+        } else {
+            out.write(escape);
+            out.write(sectionPageWidth);
+            writeInt(out, pageWidth);
+            out.write(escape);
+            out.write(sectionPageHeight);
+            writeInt(out, pageHeight);
         }
     }
 
@@ -2073,6 +2115,9 @@ public class RtfWriter extends DocWriter implements DocListener {
      * If a match is found the pageWidth and pageHeight will be set according to values determined from files
      * generated by MS Word2000 and OpenOffice 641. If no match is found the method will try to match the rotated
      * Rectangle by calling itself with the parameter rotate set to true.
+     * @param pageSize a rectangle defining the size of the page
+     * @param rotate portrait or lanscape?
+     * @return true if the format parsing succeeded
      */
     private boolean parseFormat(Rectangle pageSize, boolean rotate) {
         if (rotate) {
@@ -2143,6 +2188,9 @@ public class RtfWriter extends DocWriter implements DocListener {
 
     /**
      * This method compares to Rectangles. They are considered equal if width and height are the same
+     * @param rect1
+     * @param rect2
+     * @return true if rect1 and rect2 represent the same rectangle
      */
     private boolean rectEquals(Rectangle rect1, Rectangle rect2) {
         return (rect1.width() == rect2.width()) && (rect1.height() == rect2.height());
@@ -2161,9 +2209,10 @@ public class RtfWriter extends DocWriter implements DocListener {
      * Replaces special characters with their unicode values
      *
      * @param str The original <code>String</code>
+     * @param useHex
      * @return The converted String
      */
-    public final static String filterSpecialChar(String str) {
+    public final static String filterSpecialChar(String str, boolean useHex) {
         int length = str.length();
         int z = (int) 'z';
         StringBuffer ret = new StringBuffer(length);
@@ -2175,12 +2224,25 @@ public class RtfWriter extends DocWriter implements DocListener {
             } else if (ch == '\n') {
                 ret.append("\\par ");
             } else if (((int) ch) > z) {
+                if(useHex) {
+                    ret.append("\\\'").append(Long.toHexString((long) ch));
+                } else {
                 ret.append("\\u").append((long) ch).append('?');
+                }
             } else {
                 ret.append(ch);
             }
         }
-        return ret.toString();
+        String s = ret.toString();
+        if(s.indexOf("$newpage$") >= 0) {
+            String before = s.substring(0, s.indexOf("$newpage$"));
+            String after = s.substring(s.indexOf("$newpage$") + 9);
+            ret = new StringBuffer(before);
+            ret.append("\\page\\par ");
+            ret.append(after);
+            return ret.toString();
+        }
+        return s;
     }
 
     private void addHeaderFooterFontColor(HeaderFooter hf) {
@@ -2225,5 +2287,13 @@ public class RtfWriter extends DocWriter implements DocListener {
             }
         }
     }
+    
+    /**
+     * @see com.lowagie.text.DocListener#setMarginMirroring(boolean)
+     */
+    public boolean setMarginMirroring(boolean MarginMirroring) {
+        return false;
+    }
+    
 }
 
