@@ -1,5 +1,5 @@
 /*
- * $Id: SAXiTextHandler.java,v 1.42 2002/08/23 10:45:34 blowagie Exp $
+ * $Id: SAXiTextHandler.java,v 1.53 2006/09/01 09:35:30 blowagie Exp $
  * $Name:  $
  *
  * Copyright 2001, 2002 by Bruno Lowagie.
@@ -50,78 +50,173 @@
 
 package com.lowagie.text.xml;
 
+import java.io.IOException;
+import java.lang.reflect.Field;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.EmptyStackException;
+import java.util.HashMap;
 import java.util.Iterator;
 import java.util.Properties;
 import java.util.Stack;
 
-import org.xml.sax.helpers.DefaultHandler;
 import org.xml.sax.Attributes;
+import org.xml.sax.helpers.DefaultHandler;
 
-import com.lowagie.text.*;
+import com.lowagie.text.Anchor;
+import com.lowagie.text.Annotation;
+import com.lowagie.text.BadElementException;
+import com.lowagie.text.Cell;
+import com.lowagie.text.Chapter;
+import com.lowagie.text.Chunk;
+import com.lowagie.text.DocListener;
+import com.lowagie.text.DocumentException;
+import com.lowagie.text.Element;
+import com.lowagie.text.ElementTags;
+import com.lowagie.text.Entities;
+import com.lowagie.text.ExceptionConverter;
+import com.lowagie.text.Font;
+import com.lowagie.text.Graphic;
+import com.lowagie.text.Image;
+import com.lowagie.text.List;
+import com.lowagie.text.ListItem;
+import com.lowagie.text.Meta;
+import com.lowagie.text.PageSize;
+import com.lowagie.text.Paragraph;
+import com.lowagie.text.Phrase;
+import com.lowagie.text.Rectangle;
+import com.lowagie.text.Row;
+import com.lowagie.text.Section;
+import com.lowagie.text.Table;
+import com.lowagie.text.TextElementArray;
+import com.lowagie.text.html.HtmlTagMap;
+import com.lowagie.text.pdf.BaseFont;
 
 /**
- * The <CODE>Tags</CODE>-class maps several XHTML-tags to iText-objects.
+ * This class is a Handler that controls the iText XML to PDF conversion.
+ * Subclass it, if you want to change the way iText translates XML to PDF.
  */
 
 public class SAXiTextHandler extends DefaultHandler {
-    
-/** This is the resulting document. */
+
+    /** This is the resulting document. */
     protected DocListener document;
-    
-/** This is a <CODE>Stack</CODE> of objects, waiting to be added to the document. */
+
+    /**
+     * This is a <CODE>Stack</CODE> of objects, waiting to be added to the
+     * document.
+     */
     protected Stack stack;
-    
-/** Counts the number of chapters in this document. */
+
+    /** Counts the number of chapters in this document. */
     protected int chapters = 0;
-    
-/** This is the current chunk to which characters can be added. */
+
+    /** This is the current chunk to which characters can be added. */
     protected Chunk currentChunk = null;
-    
-/** This is the current chunk to which characters can be added. */
+
+    /** This is the current chunk to which characters can be added. */
     protected boolean ignore = false;
 
-/** This is a flag that can be set, if you want to open and close the Document-object yourself. */
+    /**
+     * This is a flag that can be set, if you want to open and close the
+     * Document-object yourself.
+     */
     protected boolean controlOpenClose = true;
 
-/**
- * Constructs a new SAXiTextHandler that will translate all the events
- * triggered by the parser to actions on the <CODE>Document</CODE>-object.
- *
- * @param	document	this is the document on which events must be triggered
- */
-    
-    public SAXiTextHandler(DocListener document) {
+    /** current margin of a page. */
+    float topMargin = 36;
+
+    /** current margin of a page. */
+    float rightMargin = 36;
+
+    /** current margin of a page. */
+    float leftMargin = 36;
+
+    /** current margin of a page. */
+    float bottomMargin = 36;
+
+    /**
+     * @param document
+     * @throws IOException
+     */
+    public SAXiTextHandler(DocListener document)
+            throws IOException {
         super();
         this.document = document;
         stack = new Stack();
     }
 
-/**
- * Sets the parameter that allows you to enable/disable the control over the Document.open() and Document.close() method.
- * <P>
- * If you set this parameter to true (= default), the parser will open the Document object when the start-root-tag is encounterd
- * and close it when the end-root-tag is met. If you set it to false, you have to open and close the Document object
- * yourself.
- *
- * @param   controlOpenClose    set this to false if you plan to open/close the Document yourself
- */
+    protected HashMap myTags;
+
+    /**
+     * @param document
+     * @param myTags
+     * @throws IOException
+     */
+    public SAXiTextHandler(DocListener document, HtmlTagMap myTags)
+            throws IOException {
+        this(document);
+        this.myTags = myTags;
+    }
+
+    /**
+     * @param document
+     * @param myTags
+     * @param bf
+     * @throws IOException
+     */
+    public SAXiTextHandler(DocListener document, HtmlTagMap myTags,
+            BaseFont bf) throws IOException {
+        this(document, myTags);
+        this.bf = bf;
+    }
+
+    /**
+     * @param document
+     * @param myTags
+     * @throws IOException
+     */
+    public SAXiTextHandler(DocListener document, HashMap myTags)
+            throws IOException {
+        this(document);
+        this.myTags = myTags;
+    }
+
+    /**
+     * Sets the parameter that allows you to enable/disable the control over the
+     * Document.open() and Document.close() method.
+     * <P>
+     * If you set this parameter to true (= default), the parser will open the
+     * Document object when the start-root-tag is encounterd and close it when
+     * the end-root-tag is met. If you set it to false, you have to open and
+     * close the Document object yourself.
+     * 
+     * @param controlOpenClose
+     *            set this to false if you plan to open/close the Document
+     *            yourself
+     */
 
     public void setControlOpenClose(boolean controlOpenClose) {
         this.controlOpenClose = controlOpenClose;
     }
 
-/**
- * This method gets called when a start tag is encountered.
- *
- * @param	name		the name of the tag that is encountered
- * @param	attrs		the list of attributes
- */
-    
-    public void startElement(String uri, String lname, String name, Attributes attrs) {
-        
+    /**
+     * This method gets called when a start tag is encountered.
+     * 
+     * @param uri
+     *            the Uniform Resource Identifier
+     * @param lname
+     *            the local name (without prefix), or the empty string if
+     *            Namespace processing is not being performed.
+     * @param name
+     *            the name of the tag that is encountered
+     * @param attrs
+     *            the list of attributes
+     */
+
+    public void startElement(String uri, String lname, String name,
+            Attributes attrs) {
+
         Properties attributes = new Properties();
         if (attrs != null) {
             for (int i = 0; i < attrs.getLength(); i++) {
@@ -131,41 +226,50 @@ public class SAXiTextHandler extends DefaultHandler {
         }
         handleStartingTags(name, attributes);
     }
-    
-/**
- * This method deals with the starting tags.
- *
- * @param       name        the name of the tag
- * @param       attributes  the list of attributes
- */
-    
+
+    /**
+     * This method deals with the starting tags.
+     * 
+     * @param name
+     *            the name of the tag
+     * @param attributes
+     *            the list of attributes
+     */
+
     public void handleStartingTags(String name, Properties attributes) {
-        //System.err.println("Start: " + name);
+        // System.err.println("Start: " + name);
         if (ignore || ElementTags.IGNORE.equals(name)) {
             ignore = true;
             return;
         }
-        
+
         // maybe there is some meaningful data that wasn't between tags
         if (currentChunk != null) {
             TextElementArray current;
             try {
                 current = (TextElementArray) stack.pop();
-            }
-            catch(EmptyStackException ese) {
-                current = new Paragraph();
+            } catch (EmptyStackException ese) {
+            	if (bf == null) {
+            		current = new Paragraph("", new Font());
+            	}
+            	else {
+            		current = new Paragraph("", new Font(this.bf));
+            	}
             }
             current.add(currentChunk);
             stack.push(current);
             currentChunk = null;
         }
-        
+
         // chunks
         if (Chunk.isTag(name)) {
             currentChunk = new Chunk(attributes);
+            if (bf != null) {
+            	currentChunk.setFont(new Font(this.bf));
+            }
             return;
         }
-        
+
         // symbols
         if (Entities.isTag(name)) {
             Font f = new Font();
@@ -173,7 +277,8 @@ public class SAXiTextHandler extends DefaultHandler {
                 handleEndingTags(ElementTags.CHUNK);
                 f = currentChunk.font();
             }
-            currentChunk = Entities.get(attributes.getProperty(ElementTags.ID), f);
+            currentChunk = Entities.get(attributes.getProperty(ElementTags.ID),
+                    f);
             return;
         }
 
@@ -182,86 +287,83 @@ public class SAXiTextHandler extends DefaultHandler {
             stack.push(new Phrase(attributes));
             return;
         }
-        
+
         // anchors
         if (Anchor.isTag(name)) {
             stack.push(new Anchor(attributes));
             return;
         }
-        
+
         // paragraphs and titles
         if (Paragraph.isTag(name) || Section.isTitle(name)) {
             stack.push(new Paragraph(attributes));
             return;
         }
-        
+
         // lists
         if (List.isTag(name)) {
             stack.push(new List(attributes));
             return;
         }
-        
+
         // listitems
         if (ListItem.isTag(name)) {
             stack.push(new ListItem(attributes));
             return;
         }
-        
+
         // cells
         if (Cell.isTag(name)) {
             stack.push(new Cell(attributes));
             return;
         }
-        
+
         // tables
         if (Table.isTag(name)) {
             Table table = new Table(attributes);
             float widths[] = table.getProportionalWidths();
             for (int i = 0; i < widths.length; i++) {
                 if (widths[i] == 0) {
-                    widths[i] = 100.0f / (float)widths.length;
+                    widths[i] = 100.0f / (float) widths.length;
                 }
             }
             try {
                 table.setWidths(widths);
-            }
-            catch(BadElementException bee) {
+            } catch (BadElementException bee) {
                 // this shouldn't happen
                 throw new ExceptionConverter(bee);
             }
             stack.push(table);
             return;
         }
-        
+
         // sections
         if (Section.isTag(name)) {
             Element previous = (Element) stack.pop();
             Section section;
             try {
-                section = ((Section)previous).addSection(attributes);
-            }
-            catch(ClassCastException cce) {
+                section = ((Section) previous).addSection(attributes);
+            } catch (ClassCastException cce) {
                 throw new ExceptionConverter(cce);
             }
             stack.push(previous);
             stack.push(section);
             return;
         }
-        
+
         // chapters
         if (Chapter.isTag(name)) {
             String value; // changed after a suggestion by Serge S. Vasiljev
-            if ((value = (String)attributes.remove(ElementTags.NUMBER)) != null){
-                 chapters = Integer.parseInt(value);
-             }
-            else {
-               chapters++;
+            if ((value = (String) attributes.remove(ElementTags.NUMBER)) != null) {
+                chapters = Integer.parseInt(value);
+            } else {
+                chapters++;
             }
-            Chapter chapter = new Chapter(attributes,chapters);
+            Chapter chapter = new Chapter(attributes, chapters);
             stack.push(chapter);
             return;
         }
-        
+
         // images
         if (Image.isTag(name)) {
             try {
@@ -270,27 +372,32 @@ public class SAXiTextHandler extends DefaultHandler {
                 try {
                     // if there is an element on the stack...
                     current = stack.pop();
-                    // ...and it's a Chapter or a Section, the Image can be added directly
-                    if (current instanceof Chapter || current instanceof Section || current instanceof Cell) {
-                        ((TextElementArray)current).add(img);
+                    // ...and it's a Chapter or a Section, the Image can be
+                    // added directly
+                    if (current instanceof Chapter
+                            || current instanceof Section
+                            || current instanceof Cell) {
+                        ((TextElementArray) current).add(img);
                         stack.push(current);
                         return;
                     }
-                    // ...if not, the Image is wrapped in a Chunk before it's added
+                    // ...if not, the Image is wrapped in a Chunk before it's
+                    // added
                     else {
                         Stack newStack = new Stack();
                         try {
-                            while (! (current instanceof Chapter || current instanceof Section || current instanceof Cell)) {
+                            while (!(current instanceof Chapter
+                                    || current instanceof Section || current instanceof Cell)) {
                                 newStack.push(current);
                                 if (current instanceof Anchor) {
-                                    img.setAnnotation(new Annotation(0, 0, 0, 0, ((Anchor)current).reference()));
+                                    img.setAnnotation(new Annotation(0, 0, 0,
+                                            0, ((Anchor) current).reference()));
                                 }
                                 current = stack.pop();
                             }
-                            ((TextElementArray)current).add(img);
+                            ((TextElementArray) current).add(img);
                             stack.push(current);
-                        }
-                        catch(EmptyStackException ese) {
+                        } catch (EmptyStackException ese) {
                             document.add(img);
                         }
                         while (!newStack.empty()) {
@@ -298,23 +405,21 @@ public class SAXiTextHandler extends DefaultHandler {
                         }
                         return;
                     }
-                }
-                catch(EmptyStackException ese) {
-                    // if there is no element on the stack, the Image is added to the document
+                } catch (EmptyStackException ese) {
+                    // if there is no element on the stack, the Image is added
+                    // to the document
                     try {
                         document.add(img);
-                    }
-                    catch(DocumentException de) {
+                    } catch (DocumentException de) {
                         throw new ExceptionConverter(de);
                     }
                     return;
                 }
-            }
-            catch(Exception e) {
+            } catch (Exception e) {
                 throw new ExceptionConverter(e);
             }
         }
-        
+
         // annotations
         if (Annotation.isTag(name)) {
             Annotation annotation = new Annotation(attributes);
@@ -324,22 +429,19 @@ public class SAXiTextHandler extends DefaultHandler {
                     current = (TextElementArray) stack.pop();
                     try {
                         current.add(annotation);
-                    }
-                    catch(Exception e) {
+                    } catch (Exception e) {
                         document.add(annotation);
                     }
                     stack.push(current);
-                }
-                catch(EmptyStackException ese) {
+                } catch (EmptyStackException ese) {
                     document.add(annotation);
                 }
                 return;
-            }
-            catch(DocumentException de) {
+            } catch (DocumentException de) {
                 throw new ExceptionConverter(de);
             }
         }
-        
+
         // newlines
         if (isNewline(name)) {
             TextElementArray current;
@@ -347,23 +449,20 @@ public class SAXiTextHandler extends DefaultHandler {
                 current = (TextElementArray) stack.pop();
                 current.add(Chunk.NEWLINE);
                 stack.push(current);
-            }
-            catch(EmptyStackException ese) {
+            } catch (EmptyStackException ese) {
                 if (currentChunk == null) {
                     try {
                         document.add(Chunk.NEWLINE);
-                    }
-                    catch(DocumentException de) {
+                    } catch (DocumentException de) {
                         throw new ExceptionConverter(de);
                     }
-                }
-                else {
+                } else {
                     currentChunk.append("\n");
                 }
             }
             return;
         }
-        
+
         // newpage
         if (isNewpage(name)) {
             TextElementArray current;
@@ -371,20 +470,21 @@ public class SAXiTextHandler extends DefaultHandler {
                 current = (TextElementArray) stack.pop();
                 Chunk newPage = new Chunk("");
                 newPage.setNewPage();
+                if (bf != null) {
+                	newPage.setFont(new Font(this.bf));
+                }
                 current.add(newPage);
                 stack.push(current);
-            }
-            catch(EmptyStackException ese) {
+            } catch (EmptyStackException ese) {
                 try {
                     document.newPage();
-                }
-                catch(DocumentException de) {
+                } catch (DocumentException de) {
                     throw new ExceptionConverter(de);
                 }
             }
             return;
         }
-        
+
         // newpage
         if (ElementTags.HORIZONTALRULE.equals(name)) {
             TextElementArray current;
@@ -394,131 +494,205 @@ public class SAXiTextHandler extends DefaultHandler {
                 current = (TextElementArray) stack.pop();
                 current.add(hr);
                 stack.push(current);
-            }
-            catch(EmptyStackException ese) {
+            } catch (EmptyStackException ese) {
                 try {
                     document.add(hr);
-                }
-                catch(DocumentException de) {
+                } catch (DocumentException de) {
                     throw new ExceptionConverter(de);
                 }
             }
             return;
         }
-        
+
         // documentroot
         if (isDocumentRoot(name)) {
             String key;
             String value;
-            for (Iterator i = attributes.keySet().iterator(); i.hasNext(); ) {
+            // pagesize and orientation specific code suggested by Samuel Gabriel
+            // Updated by Ricardo Coutinho. Only use if set in html!
+			Rectangle pageSize = null;
+			String orientation = null;
+            for (Iterator i = attributes.keySet().iterator(); i.hasNext();) {
                 key = (String) i.next();
                 value = attributes.getProperty(key);
                 try {
-                    document.add(new Meta(key, value));
+                    // margin specific code suggested by Reza Nasiri
+                    if (ElementTags.LEFT.equalsIgnoreCase(key))
+                        leftMargin = Float.valueOf(value + "f").floatValue();
+                    if (ElementTags.RIGHT.equalsIgnoreCase(key))
+                        rightMargin = Float.valueOf(value + "f").floatValue();
+                    if (ElementTags.TOP.equalsIgnoreCase(key))
+                        topMargin = Float.valueOf(value + "f").floatValue();
+                    if (ElementTags.BOTTOM.equalsIgnoreCase(key))
+                        bottomMargin = Float.valueOf(value + "f").floatValue();
+                } catch (Exception ex) {
+                    throw new ExceptionConverter(ex);
                 }
-                catch(DocumentException de) {
-                    throw new ExceptionConverter(de);
+                if (ElementTags.PAGE_SIZE.equals(key)) {
+                    try {
+                        String pageSizeName = value;
+                        Field pageSizeField = PageSize.class
+                                .getField(pageSizeName);
+                        pageSize = (Rectangle) pageSizeField.get(null);
+                    } catch (Exception ex) {
+                        throw new ExceptionConverter(ex);
+                    }
+                } else if (ElementTags.ORIENTATION.equals(key)) {
+                    try {
+                        if ("landscape".equals(value)) {
+                            orientation = "landscape";
+                        }
+                    } catch (Exception ex) {
+                        throw new ExceptionConverter(ex);
+                    }
+                } else {
+                    try {
+                        document.add(new Meta(key, value));
+                    } catch (DocumentException de) {
+                        throw new ExceptionConverter(de);
+                    }
                 }
             }
-            if (controlOpenClose) document.open();
+            if(pageSize != null) {
+            	if ("landscape".equals(orientation)) {
+            		pageSize = pageSize.rotate();
+            	}
+            	document.setPageSize(pageSize);
+            }
+            document.setMargins(leftMargin, rightMargin, topMargin,
+                    bottomMargin);
+
+            if (controlOpenClose)
+                document.open();
         }
+
     }
-    
-/**
- * This method gets called when ignorable white space encountered.
- *
- * @param	ch		an array of characters
- * @param	start	the start position in the array
- * @param	length	the number of characters to read from the array
- */
-    
+
+    /**
+     * This method gets called when ignorable white space encountered.
+     * 
+     * @param ch
+     *            an array of characters
+     * @param start
+     *            the start position in the array
+     * @param length
+     *            the number of characters to read from the array
+     */
+
     public void ignorableWhitespace(char[] ch, int start, int length) {
         // do nothing: we handle white space ourselves in the characters method
     }
-    
-/**
- * This method gets called when characters are encountered.
- *
- * @param	ch		an array of characters
- * @param	start	the start position in the array
- * @param	length	the number of characters to read from the array
- */
-    
+
+    /**
+     * This method gets called when characters are encountered.
+     * 
+     * @param ch
+     *            an array of characters
+     * @param start
+     *            the start position in the array
+     * @param length
+     *            the number of characters to read from the array
+     */
+
     public void characters(char[] ch, int start, int length) {
-        
-        if (ignore) return;
-        
+
+        if (ignore)
+            return;
+
         String content = new String(ch, start, length);
-        //System.err.println("'" + content + "'");
-        
+        // System.err.println("'" + content + "'");
+
         if (content.trim().length() == 0) {
             return;
         }
-        
+
         StringBuffer buf = new StringBuffer();
         int len = content.length();
         char character;
         boolean newline = false;
         for (int i = 0; i < len; i++) {
-            switch(character = content.charAt(i)) {
-                case ' ':
-                    if (!newline) {
-                        buf.append(character);
-                    }
-                    break;
-                case '\n':
-                    if (i > 0) {
-                        newline = true;
-                        buf.append(' ');
-                    }
-                    break;
-                case '\r':
-                    break;
-                case '\t':
-                    break;
-                    default:
-                        newline = false;
-                        buf.append(character);
+            switch (character = content.charAt(i)) {
+            case ' ':
+                if (!newline) {
+                    buf.append(character);
+                }
+                break;
+            case '\n':
+                if (i > 0) {
+                    newline = true;
+                    buf.append(' ');
+                }
+                break;
+            case '\r':
+                break;
+            case '\t':
+                break;
+            default:
+                newline = false;
+                buf.append(character);
             }
         }
         if (currentChunk == null) {
-            currentChunk = new Chunk(buf.toString());
-        }
-        else {
+        	if (bf == null) {
+        		currentChunk = new Chunk(buf.toString());
+        	}
+        	else {
+        		currentChunk = new Chunk(buf.toString(), new Font(this.bf));
+        	}
+        } else {
             currentChunk.append(buf.toString());
         }
     }
+
+    private BaseFont bf = null;
     
-/**
- * This method gets called when an end tag is encountered.
- *
- * @param	name		the name of the tag that ends
- */
-    
+    /**
+     * Sets the font that has to be used.
+     * @param bf
+     */
+    public void setBaseFont(BaseFont bf) {
+    	this.bf = bf;
+    }
+
+    /**
+     * This method gets called when an end tag is encountered.
+     * 
+     * @param uri
+     *            the Uniform Resource Identifier
+     * @param lname
+     *            the local name (without prefix), or the empty string if
+     *            Namespace processing is not being performed.
+     * @param name
+     *            the name of the tag that ends
+     */
+
     public void endElement(String uri, String lname, String name) {
         handleEndingTags(name);
     }
-    
-/**
- * This method deals with the starting tags.
- *
- * @param       name        the name of the tag
- */
-    
+
+    /**
+     * This method deals with the starting tags.
+     * 
+     * @param name
+     *            the name of the tag
+     */
+
     public void handleEndingTags(String name) {
-        
-        //System.err.println("Stop: " + name);
-        
+
+        // System.err.println("Stop: " + name);
+
         if (ElementTags.IGNORE.equals(name)) {
             ignore = false;
             return;
         }
-        if (ignore) return;
+        if (ignore)
+            return;
         // tags that don't have any content
-        if (isNewpage(name) || Annotation.isTag(name) || Image.isTag(name) || isNewline(name)) {
+        if (isNewpage(name) || Annotation.isTag(name) || Image.isTag(name)
+                || isNewline(name)) {
             return;
         }
-        
+
         try {
             // titles of sections and chapters
             if (Section.isTitle(name)) {
@@ -532,40 +706,39 @@ public class SAXiTextHandler extends DefaultHandler {
                 stack.push(previous);
                 return;
             }
-            
+
             // all other endtags
             if (currentChunk != null) {
                 TextElementArray current;
                 try {
                     current = (TextElementArray) stack.pop();
-                }
-                catch(EmptyStackException ese) {
+                } catch (EmptyStackException ese) {
                     current = new Paragraph();
                 }
                 current.add(currentChunk);
                 stack.push(current);
                 currentChunk = null;
             }
-            
+
             // chunks
             if (Chunk.isTag(name)) {
                 return;
             }
-            
+
             // phrases, anchors, lists, tables
-            if (Phrase.isTag(name) || Anchor.isTag(name) || List.isTag(name) || Paragraph.isTag(name)) {
+            if (Phrase.isTag(name) || Anchor.isTag(name) || List.isTag(name)
+                    || Paragraph.isTag(name)) {
                 Element current = (Element) stack.pop();
                 try {
                     TextElementArray previous = (TextElementArray) stack.pop();
                     previous.add(current);
                     stack.push(previous);
-                }
-                catch(EmptyStackException ese) {
+                } catch (EmptyStackException ese) {
                     document.add(current);
                 }
                 return;
             }
-            
+
             // listitems
             if (ListItem.isTag(name)) {
                 ListItem listItem = (ListItem) stack.pop();
@@ -573,21 +746,20 @@ public class SAXiTextHandler extends DefaultHandler {
                 list.add(listItem);
                 stack.push(list);
             }
-            
+
             // tables
             if (Table.isTag(name)) {
-                Table table = (Table) stack.pop();           
+                Table table = (Table) stack.pop();
                 try {
-                    TextElementArray previous = (TextElementArray) stack.pop(); 
+                    TextElementArray previous = (TextElementArray) stack.pop();
                     previous.add(table);
                     stack.push(previous);
-                }
-                catch(EmptyStackException ese) {
+                } catch (EmptyStackException ese) {
                     document.add(table);
                 }
                 return;
             }
-            
+
             // rows
             if (Row.isTag(name)) {
                 ArrayList cells = new ArrayList();
@@ -600,8 +772,7 @@ public class SAXiTextHandler extends DefaultHandler {
                         cell = (Cell) element;
                         columns += cell.colspan();
                         cells.add(cell);
-                    }
-                    else {
+                    } else {
                         table = (Table) element;
                         break;
                     }
@@ -619,28 +790,26 @@ public class SAXiTextHandler extends DefaultHandler {
                 }
                 float total = 0;
                 int j = 0;
-                for (Iterator i = cells.iterator(); i.hasNext(); ) {
+                for (Iterator i = cells.iterator(); i.hasNext();) {
                     cell = (Cell) i.next();
                     if ((width = cell.cellWidth()) == null) {
                         if (cell.colspan() == 1 && cellWidths[j] == 0) {
                             try {
                                 cellWidths[j] = 100f / columns;
                                 total += cellWidths[j];
-                            }
-                            catch(Exception e) {
+                            } catch (Exception e) {
                                 // empty on purpose
                             }
-                        }
-                        else if (cell.colspan() == 1) {
+                        } else if (cell.colspan() == 1) {
                             cellNulls[j] = false;
                         }
-                    }
-                    else if (cell.colspan() == 1 && width.endsWith("%")) {
+                    } else if (cell.colspan() == 1 && width.endsWith("%")) {
                         try {
-                            cellWidths[j] = Float.valueOf(width.substring(0, width.length() - 1) + "f").floatValue();
+                            cellWidths[j] = Float.valueOf(
+                                    width.substring(0, width.length() - 1)
+                                            + "f").floatValue();
                             total += cellWidths[j];
-                        }
-                        catch(Exception e) {
+                        } catch (Exception e) {
                             // empty on purpose
                         }
                     }
@@ -659,7 +828,8 @@ public class SAXiTextHandler extends DefaultHandler {
                     if (100.0 >= total) {
                         for (int i = 0; i < widths.length; i++) {
                             if (cellWidths[i] == 0 && widths[i] != 0) {
-                                cellWidths[i] = (widths[i] / left) * (100.0f - total);
+                                cellWidths[i] = (widths[i] / left)
+                                        * (100.0f - total);
                             }
                         }
                     }
@@ -667,80 +837,83 @@ public class SAXiTextHandler extends DefaultHandler {
                 }
                 stack.push(table);
             }
-            
+
             // cells
             if (Cell.isTag(name)) {
                 return;
             }
-            
+
             // sections
             if (Section.isTag(name)) {
                 stack.pop();
                 return;
             }
-            
+
             // chapters
             if (Chapter.isTag(name)) {
                 document.add((Element) stack.pop());
                 return;
             }
-            
+
             // the documentroot
             if (isDocumentRoot(name)) {
                 try {
                     while (true) {
                         Element element = (Element) stack.pop();
                         try {
-                            TextElementArray previous = (TextElementArray) stack.pop();
+                            TextElementArray previous = (TextElementArray) stack
+                                    .pop();
                             previous.add(element);
                             stack.push(previous);
-                        }
-                        catch(EmptyStackException es) {
+                        } catch (EmptyStackException es) {
                             document.add(element);
                         }
                     }
-                }
-                catch(EmptyStackException ese) {
+                } catch (EmptyStackException ese) {
                     // empty on purpose
                 }
-                if (controlOpenClose) document.close();
+                if (controlOpenClose)
+                    document.close();
                 return;
             }
-        }
-        catch(DocumentException de) {
+        } catch (DocumentException de) {
             throw new ExceptionConverter(de);
         }
     }
-    
-/**
- * Checks if a certain tag corresponds with the newpage-tag.
- *
- * @param	tag			a presumed tagname
- * @return	<CODE>true</CODE> or <CODE>false</CODE>
- */
-    
+
+    /**
+     * Checks if a certain tag corresponds with the newpage-tag.
+     * 
+     * @param tag
+     *            a presumed tagname
+     * @return <CODE>true</CODE> or <CODE>false</CODE>
+     */
+
     private boolean isNewpage(String tag) {
         return ElementTags.NEWPAGE.equals(tag);
     }
-    
-/**
- * Checks if a certain tag corresponds with the newpage-tag.
- *
- * @param	tag			a presumed tagname
- * @return	<CODE>true</CODE> or <CODE>false</CODE>
- */
-    
+
+    /**
+     * Checks if a certain tag corresponds with the newpage-tag.
+     * 
+     * @param tag
+     *            a presumed tagname
+     * @return <CODE>true</CODE> or <CODE>false</CODE>
+     */
+
     private boolean isNewline(String tag) {
         return ElementTags.NEWLINE.equals(tag);
     }
-    
-/**
- * Checks if a certain tag corresponds with the roottag.
- *
- * @param	tag			a presumed tagname
- * @return	<CODE>true</CODE> if <VAR>tag</VAR> equals <CODE>itext</CODE>, <CODE>false</CODE> otherwise.
- */
-    
+
+    /**
+     * Checks if a certain tag corresponds with the roottag.
+     * 
+     * @param tag
+     *            a presumed tagname
+     * @return <CODE>true</CODE> if <VAR>tag </VAR> equals <CODE>itext
+     *         </CODE>,<CODE>false</CODE> otherwise.
+     */
+
     protected boolean isDocumentRoot(String tag) {
         return ElementTags.ITEXT.equals(tag);
     }
