@@ -1,5 +1,5 @@
 /*
- * $Id: XmlWriter.java,v 1.31 2003/06/26 12:11:09 blowagie Exp $
+ * $Id: XmlWriter.java,v 1.36 2006/06/26 11:23:40 psoares33 Exp $
  * $Name:  $
  *
  * Copyright 1999, 2000, 2001, 2002 by Bruno Lowagie.
@@ -216,7 +216,8 @@ public class XmlWriter extends DocWriter implements DocListener {
     
 /**
  * Signals that an <CODE>Element</CODE> was added to the <CODE>Document</CODE>.
- *
+ * 
+ * @param element A high level object that will be added to the XML
  * @return  <CODE>true</CODE> if the element was added, <CODE>false</CODE> if not.
  * @throws  DocumentException when a document isn't open yet, or has been closed
  */
@@ -316,6 +317,7 @@ public class XmlWriter extends DocWriter implements DocListener {
  *
  * @param   element     the element
  * @param   indent      the indentation
+ * @throws IOException
  */
     
     private void write(Element element, int indent) throws IOException {
@@ -590,7 +592,17 @@ public class XmlWriter extends DocWriter implements DocListener {
             }
             case Element.TABLE:
             {
-                Table table = (Table) element;
+                Table table;
+                try {
+                	table = (Table) element;
+                }
+                catch(ClassCastException cce) {
+                	try {
+						table = ((SimpleTable)element).createTable();
+					} catch (BadElementException e) {
+						throw new ExceptionConverter(e);
+					}
+                }
                 table.complete();
                 addTabs(indent);
                 writeStart(ElementTags.TABLE);
@@ -665,15 +677,15 @@ public class XmlWriter extends DocWriter implements DocListener {
                 addTabs(indent);
                 writeStart(ElementTags.IMAGE);
                 write(ElementTags.URL, image.url().toString());
-                if ((image.alignment() & Image.LEFT) > 0) {
-                    write(ElementTags.ALIGN, ElementTags.ALIGN_LEFT);
-                }
-                else if ((image.alignment() & Image.RIGHT) > 0) {
+                if ((image.alignment() & Image.RIGHT) > 0) {
                     write(ElementTags.ALIGN, ElementTags.ALIGN_RIGHT);
                 }
                 else if ((image.alignment() & Image.MIDDLE) > 0) {
                     write(ElementTags.ALIGN, ElementTags.ALIGN_MIDDLE);
                 }
+                else {
+                    write(ElementTags.ALIGN, ElementTags.ALIGN_LEFT);
+                }
                 if ((image.alignment() & Image.UNDERLYING) > 0) {
                     write(ElementTags.UNDERLYING, String.valueOf(true));
                 }
@@ -705,6 +717,7 @@ public class XmlWriter extends DocWriter implements DocListener {
  *
  * @param   section     the section to write
  * @param   indent      the indentation
+ * @throws IOException
  */
     
     private void writeSection(Section section, int indent) throws IOException {
@@ -752,6 +765,7 @@ public class XmlWriter extends DocWriter implements DocListener {
  * Writes the XML representation of this <CODE>Rectangle</CODE>.
  *
  * @param rectangle     a <CODE>Rectangle</CODE>
+ * @throws IOException
  */
     
     private void write(Rectangle rectangle) throws IOException {
@@ -780,15 +794,13 @@ public class XmlWriter extends DocWriter implements DocListener {
             write(ElementTags.BGGREEN, String.valueOf(rectangle.backgroundColor().getGreen()));
             write(ElementTags.BGBLUE, String.valueOf(rectangle.backgroundColor().getBlue()));
         }
-        if (rectangle.grayFill() > 0) {
-            write(ElementTags.GRAYFILL, String.valueOf(rectangle.grayFill()));
-        }
     }
     
 /**
  * Encodes a <CODE>String</CODE>.
  *
  * @param   string     the <CODE>String</CODE> to encode
+ * @param indent counter that keeps the number of tabs that has to be added for indentation
  * @return  the encoded <CODE>String</CODE>
  */
     
@@ -833,6 +845,7 @@ public class XmlWriter extends DocWriter implements DocListener {
  * Writes the XML representation of a <CODE>Font</CODE>.
  *
  * @param font  a <CODE>Font</CODE>
+ * @throws IOException
  */
     
     private void write(Font font) throws IOException {
@@ -847,27 +860,27 @@ public class XmlWriter extends DocWriter implements DocListener {
             os.write(QUOTE);
             switch(font.style() & Font.BOLDITALIC) {
                 case Font.NORMAL:
-                    write(MarkupTags.CSS_NORMAL);
+                    write(MarkupTags.CSS_VALUE_NORMAL);
                     break;
                 case Font.BOLD:
-                    write(MarkupTags.CSS_BOLD);
+                    write(MarkupTags.CSS_VALUE_BOLD);
                     break;
                 case Font.ITALIC:
-                    write(MarkupTags.CSS_ITALIC);
+                    write(MarkupTags.CSS_VALUE_ITALIC);
                     break;
                 case Font.BOLDITALIC:
-                    write(MarkupTags.CSS_BOLD);
+                    write(MarkupTags.CSS_VALUE_BOLD);
                     write(", ");
-                    write(MarkupTags.CSS_ITALIC);
+                    write(MarkupTags.CSS_VALUE_ITALIC);
                     break;
             }
             if ((font.style() & Font.UNDERLINE) > 0) {
                 write(", ");
-                write(MarkupTags.CSS_UNDERLINE);
+                write(MarkupTags.CSS_VALUE_UNDERLINE);
             }
             if ((font.style() & Font.STRIKETHRU) > 0) {
                 write(", ");
-                write(MarkupTags.CSS_LINETHROUGH);
+                write(MarkupTags.CSS_VALUE_LINETHROUGH);
             }
             os.write(QUOTE);
         }
